@@ -41,7 +41,6 @@
 #define DEFEND_INTERVAL		10
 
 #include "dhcpcd.h"
-#include "if.h"
 
 struct arp_msg {
 	uint16_t op;
@@ -58,7 +57,6 @@ struct arp_state {
 	void (*probed_cb)(struct arp_state *);
 	void (*announced_cb)(struct arp_state *);
 	void (*conflicted_cb)(struct arp_state *, const struct arp_msg *);
-	void (*free_cb)(struct arp_state *);
 
 	struct in_addr addr;
 	int probes;
@@ -67,19 +65,7 @@ struct arp_state {
 };
 TAILQ_HEAD(arp_statehead, arp_state);
 
-struct iarp_state {
-	int fd;
-	struct arp_statehead arp_states;
-};
-
-#define ARP_STATE(ifp)							       \
-	((struct iarp_state *)(ifp)->if_data[IF_DATA_ARP])
-#define ARP_CSTATE(ifp)							       \
-	((const struct iarp_state *)(ifp)->if_data[IF_DATA_ARP])
-
 #ifdef INET
-int arp_open(struct interface *);
-ssize_t arp_request(const struct interface *, in_addr_t, in_addr_t);
 void arp_report_conflicted(const struct arp_state *, const struct arp_msg *);
 void arp_announce(struct arp_state *);
 void arp_probe(struct arp_state *);
@@ -87,10 +73,9 @@ struct arp_state *arp_new(struct interface *, const struct in_addr *);
 void arp_cancel(struct arp_state *);
 void arp_free(struct arp_state *);
 void arp_free_but(struct arp_state *);
-struct arp_state *arp_find(struct interface *, const struct in_addr *);
 void arp_close(struct interface *);
 
-void arp_handleifa(int, struct ipv4_addr *);
+void arp_handleifa(int, struct interface *, const struct in_addr *, int);
 #else
 #define arp_close(a) {}
 #endif
