@@ -320,8 +320,8 @@ gottoken:
 
 	/* RFC3318, section 5.2 - zero giaddr and hops */
 	if (mp == 4) {
-		*(mm + offsetof(struct bootp, hops)) = '\0';
-		memset(mm + offsetof(struct bootp, giaddr), 0, 4);
+		*(mm + offsetof(struct dhcp_message, hwopcount)) = '\0';
+		memset(mm + offsetof(struct dhcp_message, giaddr), 0, 4);
 	}
 
 	memset(hmac, 0, sizeof(hmac));
@@ -415,8 +415,7 @@ get_next_rdm_monotonic_counter(struct auth *auth)
 	rdm++;
 	if (fseek(fp, 0, SEEK_SET) == -1 ||
 	    ftruncate(fileno(fp), 0) == -1 ||
-	    fprintf(fp, "0x%016" PRIu64 "\n", rdm) != 19 ||
-	    fflush(fp) == EOF)
+	    fprintf(fp, "0x%016" PRIu64 "\n", rdm) != 19)
 	{
 		if (!auth->last_replay_set) {
 			auth->last_replay = rdm;
@@ -425,6 +424,7 @@ get_next_rdm_monotonic_counter(struct auth *auth)
 			rdm = ++auth->last_replay;
 		/* report error? */
 	}
+	fflush(fp);
 #ifdef LOCK_EX
 	if (flocked == 0)
 		flock(fileno(fp), LOCK_UN);
@@ -638,10 +638,10 @@ dhcp_auth_encode(struct auth *auth, const struct token *t,
 
 	/* RFC3318, section 5.2 - zero giaddr and hops */
 	if (mp == 4) {
-		p = m + offsetof(struct bootp, hops);
+		p = m + offsetof(struct dhcp_message, hwopcount);
 		hops = *p;
 		*p = '\0';
-		p = m + offsetof(struct bootp, giaddr);
+		p = m + offsetof(struct dhcp_message, giaddr);
 		memcpy(&giaddr, p, sizeof(giaddr));
 		memset(p, 0, sizeof(giaddr));
 	} else {
@@ -660,9 +660,9 @@ dhcp_auth_encode(struct auth *auth, const struct token *t,
 
 	/* RFC3318, section 5.2 - restore giaddr and hops */
 	if (mp == 4) {
-		p = m + offsetof(struct bootp, hops);
+		p = m + offsetof(struct dhcp_message, hwopcount);
 		*p = hops;
-		p = m + offsetof(struct bootp, giaddr);
+		p = m + offsetof(struct dhcp_message, giaddr);
 		memcpy(p, &giaddr, sizeof(giaddr));
 	}
 
