@@ -30,10 +30,6 @@
 
 #include "config.h"
 
-#ifdef HAVE_SYS_QUEUE_H
-#include <sys/queue.h>
-#endif
-
 #define DHCPCD_AUTH_SEND	(1 << 0)
 #define DHCPCD_AUTH_REQUIRE	(1 << 1)
 #define DHCPCD_AUTH_RDM_COUNTER	(1 << 2)
@@ -63,14 +59,12 @@ TAILQ_HEAD(token_head, token);
 
 struct auth {
 	int options;
-#ifdef AUTH
 	uint8_t protocol;
 	uint8_t algorithm;
 	uint8_t rdm;
 	uint64_t last_replay;
 	uint8_t last_replay_set;
 	struct token_head tokens;
-#endif
 };
 
 struct authstate {
