@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2016 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2015 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -51,16 +51,11 @@
 #define TOSTRING(a)		STRINGIFY(a)
 #define UNUSED(a)		(void)(a)
 
-#define ROUNDUP4(a)		(1 + (((a) - 1) |  3))
-#define ROUNDUP8(a)		(1 + (((a) - 1) |  7))
-
 #define USEC_PER_SEC		1000000L
 #define USEC_PER_NSEC		1000L
 #define NSEC_PER_SEC		1000000000L
-#define NSEC_PER_MSEC		1000000L
 #define MSEC_PER_SEC		1000L
-#define CSEC_PER_SEC		100L
-#define NSEC_PER_CSEC		10000000L
+#define MSEC_PER_NSEC		1000000L
 
 /* Some systems don't define timespec macros */
 #ifndef timespecclear
@@ -100,12 +95,12 @@
 } while (0 /* CONSTCOND */);
 #define ts_to_ms(ms, tv) do {						     \
 	ms = (tv)->tv_sec * MSEC_PER_SEC;				     \
-	ms += (tv)->tv_nsec / NSEC_PER_MSEC;				     \
+	ms += (tv)->tv_nsec / MSEC_PER_NSEC;				     \
 } while (0 /* CONSTCOND */);
 #define ms_to_ts(tv, ms) do {						     \
 	(tv)->tv_sec = ms / MSEC_PER_SEC;				     \
 	(tv)->tv_nsec = (suseconds_t)(ms - ((tv)->tv_sec * MSEC_PER_SEC))    \
-	    * NSEC_PER_MSEC;						     \
+	    * MSEC_PER_NSEC;						     \
 } while (0 /* CONSTCOND */);
 
 #ifndef TIMEVAL_TO_TIMESPEC
@@ -116,34 +111,33 @@
 #endif
 
 #if __GNUC__ > 2 || defined(__INTEL_COMPILER)
-# ifndef __packed
-#  define __packed __attribute__((__packed__))
+# ifndef __dead
+#  define __dead __attribute__((__noreturn__))
 # endif
-# ifndef __sysloglike
-#  ifndef __syslog_attribute_
-#    define __syslog__ __printf__
-#  endif
-#  define __sysloglike(a, b) __attribute__((format(__syslog__, a, b)))
+# ifndef __packed
+#  define __packed   __attribute__((__packed__))
+# endif
+# ifndef __printflike
+#  define __printflike(a, b) __attribute__((format(printf, a, b)))
 # endif
 # ifndef __unused
-#  define __unused __attribute__((__unused__))
+#  define __unused   __attribute__((__unused__))
 # endif
 #else
+# ifndef __dead
+#  define __dead
+# endif
 # ifndef __packed
 #  define __packed
 # endif
-# ifndef __sysloglike
-#  define __sysloglike
+# ifndef __printflike
+#  define __printflike
 # endif
 # ifndef __unused
 #  define __unused
 # endif
 #endif
 
-#ifndef __arraycount
-#  define __arraycount(__x)       (sizeof(__x) / sizeof(__x[0]))
-#endif
-
 /* We don't really need this as our supported systems define __restrict
  * automatically for us, but it is here for completeness. */
 #ifndef __restrict
@@ -157,6 +151,7 @@
 #endif
 
 void get_line_free(void);
+const char *get_hostname(char *, size_t, int);
 extern int clock_monotonic;
 int get_monotonic(struct timespec *);
 
@@ -165,17 +160,13 @@ int get_monotonic(struct timespec *);
  * However, this results in a ugly output on the command line
  * and relies on syslogd(8) starting before dhcpcd which is not
  * always the case. */
-#ifdef SMALL
-# undef USE_LOGFILE
-# define USE_LOGFILE 0
-#endif
 #ifndef USE_LOGFILE
 # define USE_LOGFILE 1
 #endif
 #if USE_LOGFILE
 void logger_open(struct dhcpcd_ctx *);
 #define logger_mask(ctx, lvl) setlogmask((lvl))
-void logger(struct dhcpcd_ctx *, int, const char *, ...) __sysloglike(3, 4);
+__printflike(3, 4) void logger(struct dhcpcd_ctx *, int, const char *, ...);
 void logger_close(struct dhcpcd_ctx *);
 #else
 #define logger_open(ctx) openlog(PACKAGE, LOG_PERROR | LOG_PID, LOG_DAEMON)
@@ -189,17 +180,11 @@ void logger_close(struct dhcpcd_ctx *);
 #endif
 
 ssize_t setvar(struct dhcpcd_ctx *,
-    char **, const char *, const char *, const char *);
-ssize_t setvard(struct dhcpcd_ctx *,
-    char **, const char *, const char *, size_t);
-ssize_t addvar(struct dhcpcd_ctx *,
     char ***, const char *, const char *, const char *);
-ssize_t addvard(struct dhcpcd_ctx *,
+ssize_t setvard(struct dhcpcd_ctx *,
     char ***, const char *, const char *, size_t);
+time_t uptime(void);
 
-char *hwaddr_ntoa(const uint8_t *, size_t, char *, size_t);
-size_t hwaddr_aton(uint8_t *, const char *);
-size_t read_hwaddr_aton(uint8_t **, const char *);
-
-ssize_t recvmsg_realloc(int, struct msghdr *, int);
+char *hwaddr_ntoa(const unsigned char *, size_t, char *, size_t);
+size_t hwaddr_aton(unsigned char *, const char *);
 #endif
