@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2016 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2015 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -43,7 +43,6 @@
 #include "dhcpcd.h"
 #include "control.h"
 #include "eloop.h"
-#include "if.h"
 
 #ifndef SUN_LEN
 #define SUN_LEN(su) \
@@ -93,7 +92,7 @@ control_delete(struct fd_list *fd)
 {
 
 	TAILQ_REMOVE(&fd->ctx->control_fds, fd, next);
-	eloop_event_delete(fd->ctx->eloop, fd->fd);
+	eloop_event_delete(fd->ctx->eloop, fd->fd, 0);
 	close(fd->fd);
 	control_queue_free(fd);
 	free(fd);
@@ -181,7 +180,8 @@ control_handle1(struct dhcpcd_ctx *ctx, int lfd, unsigned int fd_flags)
 		TAILQ_INIT(&l->queue);
 		TAILQ_INIT(&l->free_queue);
 		TAILQ_INSERT_TAIL(&ctx->control_fds, l, next);
-		eloop_event_add(ctx->eloop, l->fd, control_handle_data, l);
+		eloop_event_add(ctx->eloop, l->fd,
+		    control_handle_data, l, NULL, NULL);
 	} else
 		close(fd);
 }
@@ -207,10 +207,28 @@ make_sock(struct sockaddr_un *sa, const char *ifname, int unpriv)
 {
 	int fd;
 
-#define SOCK_FLAGS	SOCK_CLOEXEC | SOCK_NONBLOCK
-	if ((fd = xsocket(AF_UNIX, SOCK_STREAM | SOCK_FLAGS, 0)) == -1)
+#ifdef SOCK_CLOEXEC
+	if ((fd = socket(AF_UNIX,
+	    SOCK_STREAM | SOCK_CLOEXEC | SOCK_NONBLOCK, 0)) == -1)
 		return -1;
-#undef SOCK_FLAGS
+#else
+	int flags;
+
+	if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1)
+		return -1;
+	if ((flags = fcntl(fd, F_GETFD, 0)) == -1 ||
+	    fcntl(fd, F_SETFD, flags | FD_CLOEXEC) == -1)
+	{
+		close(fd);
+	        return -1;
+	}
+	if ((flags = fcntl(fd, F_GETFL, 0)) == -1 ||
+	    fcntl(fd, F_SETFL, flags | O_NONBLOCK) == -1)
+	{
+		close(fd);
+	        return -1;
+	}
+#endif
 	memset(sa, 0, sizeof(*sa));
 	sa->sun_family = AF_UNIX;
 	if (unpriv)
@@ -246,7 +264,7 @@ control_start1(struct dhcpcd_ctx *ctx, const char *ifname, mode_t fmode)
 		unlink(sa.sun_path);
 		return -1;
 	}
-
+	
 	if ((fmode & S_UNPRIV) != S_UNPRIV)
 		strlcpy(ctx->control_sock, sa.sun_path,
 		    sizeof(ctx->control_sock));
@@ -262,13 +280,14 @@ control_start(struct dhcpcd_ctx *ctx, const char *ifname)
 		return -1;
 
 	ctx->control_fd = fd;
-	eloop_event_add(ctx->eloop, fd, control_handle, ctx);
+	eloop_event_add(ctx->eloop, fd, control_handle, ctx, NULL, NULL);
 
 	if (ifname == NULL && (fd = control_start1(ctx, NULL, S_UNPRIV)) != -1){
 		/* We must be in master mode, so create an unpriviledged socket
 		 * to allow normal users to learn the status of dhcpcd. */
 		ctx->control_unpriv_fd = fd;
-		eloop_event_add(ctx->eloop, fd, control_handle_unpriv, ctx);
+		eloop_event_add(ctx->eloop, fd, control_handle_unpriv,
+		    ctx, NULL, NULL);
 	}
 	return ctx->control_fd;
 }
@@ -284,14 +303,14 @@ control_stop(struct dhcpcd_ctx *ctx)
 
 	if (ctx->control_fd == -1)
 		return 0;
-	eloop_event_delete(ctx->eloop, ctx->control_fd);
+	eloop_event_delete(ctx->eloop, ctx->control_fd, 0);
 	close(ctx->control_fd);
 	ctx->control_fd = -1;
 	if (unlink(ctx->control_sock) == -1)
 		retval = -1;
 
 	if (ctx->control_unpriv_fd != -1) {
-		eloop_event_delete(ctx->eloop, ctx->control_unpriv_fd);
+		eloop_event_delete(ctx->eloop, ctx->control_unpriv_fd, 0);
 		close(ctx->control_unpriv_fd);
 		ctx->control_unpriv_fd = -1;
 		if (unlink(UNPRIVSOCKET) == -1)
@@ -301,7 +320,7 @@ control_stop(struct dhcpcd_ctx *ctx)
 freeit:
 	while ((l = TAILQ_FIRST(&ctx->control_fds))) {
 		TAILQ_REMOVE(&ctx->control_fds, l, next);
-		eloop_event_delete(ctx->eloop, l->fd);
+		eloop_event_delete(ctx->eloop, l->fd, 0);
 		close(l->fd);
 		control_queue_free(l);
 		free(l);
@@ -311,21 +330,20 @@ freeit:
 }
 
 int
-control_open(const char *ifname)
+control_open(struct dhcpcd_ctx *ctx, const char *ifname)
 {
 	struct sockaddr_un sa;
-	int fd;
+	socklen_t len;
 
-	if ((fd = make_sock(&sa, ifname, 0)) != -1) {
-		socklen_t len;
-		
-		len = (socklen_t)SUN_LEN(&sa);
-		if (connect(fd, (struct sockaddr *)&sa, len) == -1) {
-			close(fd);
-			fd = -1;
-		}
+	if ((ctx->control_fd = make_sock(&sa, ifname, 0)) == -1)
+		return -1;
+	len = (socklen_t)SUN_LEN(&sa);
+	if (connect(ctx->control_fd, (struct sockaddr *)&sa, len) == -1) {
+		close(ctx->control_fd);
+		ctx->control_fd = -1;
+		return -1;
 	}
-	return fd;
+	return 0;
 }
 
 ssize_t
@@ -381,7 +399,7 @@ control_writeone(void *arg)
 	TAILQ_INSERT_TAIL(&fd->free_queue, data, next);
 
 	if (TAILQ_FIRST(&fd->queue) == NULL)
-		eloop_event_remove_writecb(fd->ctx->eloop, fd->fd);
+		eloop_event_delete(fd->ctx->eloop, fd->fd, 1);
 }
 
 int
@@ -409,7 +427,8 @@ control_queue(struct fd_list *fd, char *data, size_t data_len, uint8_t fit)
 	d->data_len = data_len;
 	d->freeit = fit;
 	TAILQ_INSERT_TAIL(&fd->queue, d, next);
-	eloop_event_add_w(fd->ctx->eloop, fd->fd, control_writeone, fd);
+	eloop_event_add(fd->ctx->eloop, fd->fd,
+	    NULL, NULL, control_writeone, fd);
 	return 0;
 }
 
