@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2016 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2015 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -35,8 +35,6 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <arpa/nameser.h> /* after normal includes for sunos */
-
 #include "config.h"
 
 #include "common.h"
@@ -45,43 +43,6 @@
 #include "if.h"
 #include "ipv6.h"
 
-/* Support very old arpa/nameser.h as found in OpenBSD */
-#ifndef NS_MAXDNAME
-#define NS_MAXCDNAME MAXCDNAME
-#define NS_MAXDNAME MAXDNAME
-#define NS_MAXLABEL MAXLABEL
-#endif
-
-const char *
-dhcp_get_hostname(char *buf, size_t buf_len, const struct if_options *ifo)
-{
-
-	if (ifo->hostname[0] == '\0') {
-		if (gethostname(buf, buf_len) != 0)
-			return NULL;
-		buf[buf_len - 1] = '\0';
-	} else
-		strlcpy(buf, ifo->hostname, buf_len);
-
-	/* Deny sending of these local hostnames */
-	if (buf[0] == '\0' || buf[0] == '.' ||
-	    strcmp(buf, "(none)") == 0 ||
-	    strcmp(buf, "localhost") == 0 ||
-	    strncmp(buf, "localhost.", strlen("localhost.")) == 0)
-		return NULL;
-
-	/* Shorten the hostname if required */
-	if (ifo->options & DHCPCD_HOSTNAME_SHORT) {
-		char *hp;
-
-		hp = strchr(buf, '.');
-		if (hp != NULL)
-			*hp = '\0';
-	}
-
-	return buf;
-}
-
 void
 dhcp_print_option_encoding(const struct dhcp_opt *opt, int cols)
 {
@@ -91,53 +52,51 @@ dhcp_print_option_encoding(const struct dhcp_opt *opt, int cols)
 		cols++;
 	}
 	putchar('\t');
-	if (opt->type & OT_EMBED)
+	if (opt->type & EMBED)
 		printf(" embed");
-	if (opt->type & OT_ENCAP)
+	if (opt->type & ENCAP)
 		printf(" encap");
-	if (opt->type & OT_INDEX)
+	if (opt->type & INDEX)
 		printf(" index");
-	if (opt->type & OT_ARRAY)
+	if (opt->type & ARRAY)
 		printf(" array");
-	if (opt->type & OT_UINT8)
-		printf(" uint8");
-	else if (opt->type & OT_INT8)
-		printf(" int8");
-	else if (opt->type & OT_UINT16)
+	if (opt->type & UINT8)
+		printf(" byte");
+	else if (opt->type & UINT16)
 		printf(" uint16");
-	else if (opt->type & OT_INT16)
-		printf(" int16");
-	else if (opt->type & OT_UINT32)
+	else if (opt->type & SINT16)
+		printf(" sint16");
+	else if (opt->type & UINT32)
 		printf(" uint32");
-	else if (opt->type & OT_INT32)
-		printf(" int32");
-	else if (opt->type & OT_ADDRIPV4)
+	else if (opt->type & SINT32)
+		printf(" sint32");
+	else if (opt->type & ADDRIPV4)
 		printf(" ipaddress");
-	else if (opt->type & OT_ADDRIPV6)
+	else if (opt->type & ADDRIPV6)
 		printf(" ip6address");
-	else if (opt->type & OT_FLAG)
+	else if (opt->type & FLAG)
 		printf(" flag");
-	else if (opt->type & OT_BITFLAG)
-		printf(" bitflags");
-	else if (opt->type & OT_RFC1035)
+	else if (opt->type & RFC3397)
 		printf(" domain");
-	else if (opt->type & OT_DOMAIN)
+	else if (opt->type & DOMAIN)
 		printf(" dname");
-	else if (opt->type & OT_ASCII)
+	else if (opt->type & ASCII)
 		printf(" ascii");
-	else if (opt->type & OT_RAW)
+	else if (opt->type & RAW)
 		printf(" raw");
-	else if (opt->type & OT_BINHEX)
+	else if (opt->type & BINHEX)
 		printf(" binhex");
-	else if (opt->type & OT_STRING)
+	else if (opt->type & STRING)	
 		printf(" string");
-	if (opt->type & OT_RFC3361)
+	if (opt->type & RFC3361)	
 		printf(" rfc3361");
-	if (opt->type & OT_RFC3442)
+	if (opt->type & RFC3442)
 		printf(" rfc3442");
-	if (opt->type & OT_REQUEST)
+	if (opt->type & RFC5969)
+		printf(" rfc5969");
+	if (opt->type & REQUEST)
 		printf(" request");
-	if (opt->type & OT_NOREQ)
+	if (opt->type & NOREQ)
 		printf(" norequest");
 	putchar('\n');
 }
@@ -170,7 +129,7 @@ dhcp_vendor(char *str, size_t len)
 	char *p;
 	int l;
 
-	if (uname(&utn) == -1)
+	if (uname(&utn) != 0)
 		return (ssize_t)snprintf(str, len, "%s-%s",
 		    PACKAGE, VERSION);
 	p = str;
@@ -207,8 +166,6 @@ make_option_mask(const struct dhcp_opt *dopts, size_t dopts_len,
 			continue;
 		match = 0;
 		for (i = 0, opt = odopts; i < odopts_len; i++, opt++) {
-			if (opt->var == NULL || opt->option == 0)
-				continue; /* buggy dhcpcd-definitions.conf */
 			if (strcmp(opt->var, token) == 0)
 				match = 1;
 			else {
@@ -239,7 +196,7 @@ make_option_mask(const struct dhcp_opt *dopts, size_t dopts_len,
 			errno = ENOENT;
 			return -1;
 		}
-		if (add == 2 && !(opt->type & OT_ADDRIPV4)) {
+		if (add == 2 && !(opt->type & ADDRIPV4)) {
 			free(o);
 			errno = EINVAL;
 			return -1;
@@ -305,12 +262,12 @@ encode_rfc1035(const char *src, uint8_t *dst)
 	return len;
 }
 
-/* Decode an RFC1035 DNS search order option into a space
+/* Decode an RFC3397 DNS search order option into a space
  * separated string. Returns length of string (including
  * terminating zero) or zero on error. out may be NULL
  * to just determine output length. */
 ssize_t
-decode_rfc1035(char *out, size_t len, const uint8_t *p, size_t pl)
+decode_rfc3397(char *out, size_t len, const uint8_t *p, size_t pl)
 {
 	const char *start;
 	size_t start_len, l, count;
@@ -318,11 +275,6 @@ decode_rfc1035(char *out, size_t len, const uint8_t *p, size_t pl)
 	int hops;
 	uint8_t ltype;
 
-	if (pl > NS_MAXCDNAME) {
-		errno = E2BIG;
-		return -1;
-	}
-
 	count = 0;
 	start = out;
 	start_len = len;
@@ -335,13 +287,8 @@ decode_rfc1035(char *out, size_t len, const uint8_t *p, size_t pl)
 		 * the name isn't fully qualified (ie, not terminated) */
 		while (q < e && (l = (size_t)*q++)) {
 			ltype = l & 0xc0;
-			if (ltype == 0x80 || ltype == 0x40) {
-				/* Currently reserved for future use as noted
-				 * in RFC1035 4.1.4 as the 10 and 01
-				 * combinations. */
-				errno = ENOTSUP;
+			if (ltype == 0x80 || ltype == 0x40)
 				return -1;
-			}
 			else if (ltype == 0xc0) { /* pointer */
 				if (q == e) {
 					errno = ERANGE;
@@ -374,10 +321,6 @@ decode_rfc1035(char *out, size_t len, const uint8_t *p, size_t pl)
 						errno = ENOBUFS;
 						return -1;
 					}
-					if (l + 1 > NS_MAXLABEL) {
-						errno = EINVAL;
-						return -1;
-					}
 					memcpy(out, q, l);
 					out += l;
 					*out++ = '.';
@@ -405,10 +348,6 @@ decode_rfc1035(char *out, size_t len, const uint8_t *p, size_t pl)
 	if (count)
 		/* Don't count the trailing NUL */
 		count--;
-	if (count > NS_MAXDNAME) {
-		errno = E2BIG;
-		return -1;
-	}
 	return (ssize_t)count;
 }
 
@@ -437,7 +376,7 @@ valid_domainname(char *lbl, int type)
 		if (c == ' ') {
 			if (lbl - 1 == slbl) /* No space at start */
 				break;
-			if (!(type & OT_ARRAY))
+			if (!(type & ARRAY))
 				break;
 			/* Skip to the next label */
 			if (!start) {
@@ -458,7 +397,7 @@ valid_domainname(char *lbl, int type)
 		    !start && *lbl != ' ' && *lbl != '\0') ||
 		    isalnum(c))
 		{
-			if (++len > NS_MAXLABEL) {
+			if (++len > 63) {
 				errno = ERANGE;
 				errset = 1;
 				break;
@@ -499,7 +438,7 @@ print_string(char *dst, size_t len, int type, const uint8_t *data, size_t dl)
 
 	while (data < e) {
 		c = *data++;
-		if (type & OT_BINHEX) {
+		if (type & BINHEX) {
 			if (dst) {
 				if (len  == 0 || len == 1) {
 					errno = ENOSPC;
@@ -512,19 +451,19 @@ print_string(char *dst, size_t len, int type, const uint8_t *data, size_t dl)
 			bytes += 2;
 			continue;
 		}
-		if (type & OT_ASCII && (!isascii(c))) {
+		if (type & ASCII && (!isascii(c))) {
 			errno = EINVAL;
 			break;
 		}
-		if (!(type & (OT_ASCII | OT_RAW | OT_ESCSTRING | OT_ESCFILE)) &&
+		if (!(type & (ASCII | RAW | ESCSTRING | ESCFILE)) /* plain */ &&
 		    (!isascii(c) && !isprint(c)))
 		{
 			errno = EINVAL;
 			break;
 		}
-		if ((type & (OT_ESCSTRING | OT_ESCFILE) &&
+		if ((type & (ESCSTRING | ESCFILE) &&
 		    (c == '\\' || !isascii(c) || !isprint(c))) ||
-		    (type & OT_ESCFILE && (c == '/' || c == ' ')))
+		    (type & ESCFILE && (c == '/' || c == ' ')))
 		{
 			errno = EINVAL;
 			if (c == '\\') {
@@ -573,7 +512,7 @@ print_string(char *dst, size_t len, int type, const uint8_t *data, size_t dl)
 		*dst = '\0';
 
 		/* Now we've printed it, validate the domain */
-		if (type & OT_DOMAIN && !valid_domainname(odst, type)) {
+		if (type & DOMAIN && !valid_domainname(odst, type)) {
 			*odst = '\0';
 			return 1;
 		}
@@ -583,81 +522,62 @@ print_string(char *dst, size_t len, int type, const uint8_t *data, size_t dl)
 	return (ssize_t)bytes;
 }
 
+#define ADDRSZ		4
 #define ADDR6SZ		16
-static ssize_t
+static size_t
 dhcp_optlen(const struct dhcp_opt *opt, size_t dl)
 {
 	size_t sz;
 
-	if (opt->type & OT_ADDRIPV6)
-		sz = ADDR6SZ;
-	else if (opt->type & (OT_INT32 | OT_UINT32 | OT_ADDRIPV4))
-		sz = sizeof(uint32_t);
-	else if (opt->type & (OT_INT16 | OT_UINT16))
-		sz = sizeof(uint16_t);
-	else if (opt->type & (OT_INT8 | OT_UINT8 | OT_BITFLAG))
-		sz = sizeof(uint8_t);
-	else if (opt->type & OT_FLAG)
+	if (dl == 0)
 		return 0;
-	else {
-		/* All other types are variable length */
+
+	if (opt->type == 0 ||
+	    opt->type & (STRING | BINHEX | RFC3442 | RFC5969))
+	{
 		if (opt->len) {
-			if ((size_t)opt->len > dl) {
-				errno = EOVERFLOW;
-				return -1;
-			}
-			return (ssize_t)opt->len;
+			if ((size_t)opt->len > dl)
+				return 0;
+			return (size_t)opt->len;
 		}
-		return (ssize_t)dl;
-	}
-	if (dl < sz) {
-		errno = EOVERFLOW;
-		return -1;
+		return dl;
 	}
 
-	/* Trim any extra data.
-	 * Maybe we need a settng to reject DHCP options with extra data? */
-	if (opt->type & OT_ARRAY)
-		return (ssize_t)(dl - (dl % sz));
-	return (ssize_t)sz;
+	if ((opt->type & (ADDRIPV4 | ARRAY)) == (ADDRIPV4 | ARRAY)) {
+		if (dl < ADDRSZ)
+			return 0;
+		return dl - (dl % ADDRSZ);
+	}
+
+	if ((opt->type & (ADDRIPV6 | ARRAY)) == (ADDRIPV6 | ARRAY)) {
+		if (dl < ADDR6SZ)
+			return 0;
+		return dl - (dl % ADDR6SZ);
+	}
+
+	if (opt->type & (UINT32 | ADDRIPV4))
+		sz = sizeof(uint32_t);
+	else if (opt->type & UINT16)
+		sz = sizeof(uint16_t);
+	else if (opt->type & UINT8)
+		sz = sizeof(uint8_t);
+	else if (opt->type & ADDRIPV6)
+		sz = ADDR6SZ;
+	else
+		/* If we don't know the size, assume it's valid */
+		return dl;
+	return (dl < sz ? 0 : sz);
 }
 
-/* It's possible for DHCPv4 to contain an IPv6 address */
-static ssize_t
-ipv6_printaddr(char *s, size_t sl, const uint8_t *d, const char *ifname)
-{
-	char buf[INET6_ADDRSTRLEN];
-	const char *p;
-	size_t l;
+#ifdef INET6
+#define PO_IFNAME
+#else
+#define PO_IFNAME __unused
+#endif
 
-	p = inet_ntop(AF_INET6, d, buf, sizeof(buf));
-	if (p == NULL)
-		return -1;
-
-	l = strlen(p);
-	if (d[0] == 0xfe && (d[1] & 0xc0) == 0x80)
-		l += 1 + strlen(ifname);
-
-	if (s == NULL)
-		return (ssize_t)l;
-
-	if (sl < l) {
-		errno = ENOMEM;
-		return -1;
-	}
-
-	s += strlcpy(s, p, sl);
-	if (d[0] == 0xfe && (d[1] & 0xc0) == 0x80) {
-		*s++ = '%';
-		s += strlcpy(s, ifname, sl);
-	}
-	*s = '\0';
-	return (ssize_t)l;
-}
-
-static ssize_t
-print_option(char *s, size_t len, const struct dhcp_opt *opt,
-    const uint8_t *data, size_t dl, const char *ifname)
+ssize_t
+print_option(char *s, size_t len, int type, const uint8_t *data, size_t dl,
+    PO_IFNAME const char *ifname)
 {
 	const uint8_t *e, *t;
 	uint16_t u16;
@@ -669,38 +589,41 @@ print_option(char *s, size_t len, const struct dhcp_opt *opt,
 	size_t l;
 	char *tmp;
 
-	if (opt->type & OT_RFC1035) {
-		sl = decode_rfc1035(NULL, 0, data, dl);
+	if (type & RFC3397) {
+		sl = decode_rfc3397(NULL, 0, data, dl);
 		if (sl == 0 || sl == -1)
 			return sl;
 		l = (size_t)sl + 1;
 		tmp = malloc(l);
 		if (tmp == NULL)
 			return -1;
-		decode_rfc1035(tmp, l, data, dl);
-		sl = print_string(s, len, opt->type, (uint8_t *)tmp, l - 1);
+		decode_rfc3397(tmp, l, data, dl);
+		sl = print_string(s, len, type, (uint8_t *)tmp, l - 1);
 		free(tmp);
 		return sl;
 	}
 
 #ifdef INET
-	if (opt->type & OT_RFC3361) {
+	if (type & RFC3361) {
 		if ((tmp = decode_rfc3361(data, dl)) == NULL)
 			return -1;
 		l = strlen(tmp);
-		sl = print_string(s, len, opt->type, (uint8_t *)tmp, l);
+		sl = print_string(s, len, type, (uint8_t *)tmp, l);
 		free(tmp);
 		return sl;
 	}
 
-	if (opt->type & OT_RFC3442)
+	if (type & RFC3442)
 		return decode_rfc3442(s, len, data, dl);
+
+	if (type & RFC5969)
+		return decode_rfc5969(s, len, data, dl);
 #endif
 
-	if (opt->type & OT_STRING)
-		return print_string(s, len, opt->type, data, dl);
+	if (type & STRING)
+		return print_string(s, len, type, data, dl);
 
-	if (opt->type & OT_FLAG) {
+	if (type & FLAG) {
 		if (s) {
 			*s++ = '1';
 			*s = '\0';
@@ -708,63 +631,41 @@ print_option(char *s, size_t len, const struct dhcp_opt *opt,
 		return 1;
 	}
 
-	if (opt->type & OT_BITFLAG) {
-		/* bitflags are a string, MSB first, such as ABCDEFGH
-		 * where A is 10000000, B is 01000000, etc. */
-		bytes = 0;
-		for (l = 0, sl = sizeof(opt->bitflags) - 1;
-		    l < sizeof(opt->bitflags);
-		    l++, sl--)
-		{
-			/* Don't print NULL or 0 flags */
-			if (opt->bitflags[l] != '\0' &&
-			    opt->bitflags[l] != '0' &&
-			    *data & (1 << sl))
-			{
-				if (s)
-					*s++ = opt->bitflags[l];
-				bytes++;
-			}
-		}
-		if (s)
-			*s = '\0';
-		return bytes;
-	}
-
 	if (!s) {
-		if (opt->type & OT_UINT8)
+		if (type & UINT8)
 			l = 3;
-		else if (opt->type & OT_INT8)
-			l = 4;
-		else if (opt->type & OT_UINT16) {
+		else if (type & UINT16) {
 			l = 5;
 			dl /= 2;
-		} else if (opt->type & OT_INT16) {
+		} else if (type & SINT16) {
 			l = 6;
 			dl /= 2;
-		} else if (opt->type & OT_UINT32) {
+		} else if (type & UINT32) {
 			l = 10;
 			dl /= 4;
-		} else if (opt->type & OT_INT32) {
+		} else if (type & SINT32) {
 			l = 11;
 			dl /= 4;
-		} else if (opt->type & OT_ADDRIPV4) {
+		} else if (type & ADDRIPV4) {
 			l = 16;
 			dl /= 4;
-		} else if (opt->type & OT_ADDRIPV6) {
+		}
+#ifdef INET6
+		else if (type & ADDRIPV6) {
 			e = data + dl;
 			l = 0;
 			while (data < e) {
 				if (l)
 					l++; /* space */
 				sl = ipv6_printaddr(NULL, 0, data, ifname);
-				if (sl == -1)
-					return l == 0 ? -1 : (ssize_t)l;
-				l += (size_t)sl;
+				if (sl != -1)
+					l += (size_t)sl;
 				data += 16;
 			}
 			return (ssize_t)l;
-		} else {
+		}
+#endif
+		else {
 			errno = EINVAL;
 			return -1;
 		}
@@ -779,45 +680,48 @@ print_option(char *s, size_t len, const struct dhcp_opt *opt,
 			bytes++;
 			len--;
 		}
-		if (opt->type & OT_UINT8) {
+		if (type & UINT8) {
 			sl = snprintf(s, len, "%u", *data);
 			data++;
-		} else if (opt->type & OT_INT8) {
-			sl = snprintf(s, len, "%d", *data);
-			data++;
-		} else if (opt->type & OT_UINT16) {
+		} else if (type & UINT16) {
 			memcpy(&u16, data, sizeof(u16));
 			u16 = ntohs(u16);
 			sl = snprintf(s, len, "%u", u16);
 			data += sizeof(u16);
-		} else if (opt->type & OT_INT16) {
+		} else if (type & SINT16) {
 			memcpy(&u16, data, sizeof(u16));
 			s16 = (int16_t)ntohs(u16);
 			sl = snprintf(s, len, "%d", s16);
 			data += sizeof(u16);
-		} else if (opt->type & OT_UINT32) {
+		} else if (type & UINT32) {
 			memcpy(&u32, data, sizeof(u32));
 			u32 = ntohl(u32);
 			sl = snprintf(s, len, "%u", u32);
 			data += sizeof(u32);
-		} else if (opt->type & OT_INT32) {
+		} else if (type & SINT32) {
 			memcpy(&u32, data, sizeof(u32));
 			s32 = (int32_t)ntohl(u32);
 			sl = snprintf(s, len, "%d", s32);
 			data += sizeof(u32);
-		} else if (opt->type & OT_ADDRIPV4) {
+		} else if (type & ADDRIPV4) {
 			memcpy(&addr.s_addr, data, sizeof(addr.s_addr));
 			sl = snprintf(s, len, "%s", inet_ntoa(addr));
 			data += sizeof(addr.s_addr);
-		} else if (opt->type & OT_ADDRIPV6) {
-			sl = ipv6_printaddr(s, len, data, ifname);
-			data += 16;
-		} else {
-			errno = EINVAL;
-			return -1;
 		}
-		if (sl == -1)
-			return bytes == 0 ? -1 : bytes;
+#ifdef INET6
+		else if (type & ADDRIPV6) {
+			ssize_t r;
+
+			r = ipv6_printaddr(s, len, data, ifname);
+			if (r != -1)
+				sl = r;
+			else
+				sl = 0;
+			data += 16;
+		}
+#endif
+		else
+			sl = 0;
 		len -= (size_t)sl;
 		bytes += sl;
 		s += sl;
@@ -828,9 +732,9 @@ print_option(char *s, size_t len, const struct dhcp_opt *opt,
 
 int
 dhcp_set_leasefile(char *leasefile, size_t len, int family,
-    const struct interface *ifp)
+    const struct interface *ifp, const char *extra)
 {
-	char ssid[1 + (IF_SSIDLEN * 4) + 1]; /* - prefix and NUL terminated. */
+	char ssid[len];
 
 	if (ifp->name[0] == '\0') {
 		strlcpy(leasefile, ifp->ctx->pidfile, len);
@@ -849,17 +753,17 @@ dhcp_set_leasefile(char *leasefile, size_t len, int family,
 	if (ifp->wireless) {
 		ssid[0] = '-';
 		print_string(ssid + 1, sizeof(ssid) - 1,
-		    OT_ESCFILE,
+		    ESCFILE,
 		    (const uint8_t *)ifp->ssid, ifp->ssid_len);
 	} else
 		ssid[0] = '\0';
 	return snprintf(leasefile, len,
 	    family == AF_INET ? LEASEFILE : LEASEFILE6,
-	    ifp->name, ssid);
+	    ifp->name, ssid, extra);
 }
 
 static size_t
-dhcp_envoption1(char **env, const char *prefix,
+dhcp_envoption1(struct dhcpcd_ctx *ctx, char **env, const char *prefix,
     const struct dhcp_opt *opt, int vname, const uint8_t *od, size_t ol,
     const char *ifname)
 {
@@ -867,12 +771,9 @@ dhcp_envoption1(char **env, const char *prefix,
 	size_t e;
 	char *v, *val;
 
-	/* Ensure a valid length */
-	ol = (size_t)dhcp_optlen(opt, ol);
-	if ((ssize_t)ol == -1)
-		return 0;
-
-	len = print_option(NULL, 0, opt, od, ol, ifname);
+	if (opt->len && opt->len < ol)
+		ol = opt->len;
+	len = print_option(NULL, 0, opt->type, od, ol, ifname);
 	if (len < 0)
 		return 0;
 	if (vname)
@@ -885,14 +786,16 @@ dhcp_envoption1(char **env, const char *prefix,
 	if (env == NULL)
 		return e;
 	v = val = *env = malloc(e);
-	if (v == NULL)
+	if (v == NULL) {
+		logger(ctx, LOG_ERR, "%s: %m", __func__);
 		return 0;
+	}
 	if (vname)
 		v += snprintf(val, e, "%s_%s=", prefix, opt->var);
 	else
 		v += snprintf(val, e, "%s=", prefix);
 	if (len != 0)
-		print_option(v, (size_t)len + 1, opt, od, ol, ifname);
+		print_option(v, (size_t)len + 1, opt->type, od, ol, ifname);
 	return e;
 }
 
@@ -905,7 +808,6 @@ dhcp_envoption(struct dhcpcd_ctx *ctx, char **env, const char *prefix,
     const uint8_t *od, size_t ol)
 {
 	size_t e, i, n, eos, eol;
-	ssize_t eo;
 	unsigned int eoc;
 	const uint8_t *eod;
 	int ov;
@@ -914,20 +816,15 @@ dhcp_envoption(struct dhcpcd_ctx *ctx, char **env, const char *prefix,
 
 	/* If no embedded or encapsulated options, it's easy */
 	if (opt->embopts_len == 0 && opt->encopts_len == 0) {
-		if (!(opt->type & OT_RESERVED)) {
-			if (dhcp_envoption1(env == NULL ? NULL : &env[0],
-			    prefix, opt, 1, od, ol, ifname))
-				return 1;
-			else
-				logger(ctx, LOG_ERR, "%s: %s %d: %m",
-				    ifname, __func__, opt->option);
-		}
+		if (dhcp_envoption1(ctx, env == NULL ? NULL : &env[0],
+		    prefix, opt, 1, od, ol, ifname))
+			return 1;
 		return 0;
 	}
 
 	/* Create a new prefix based on the option */
 	if (env) {
-		if (opt->type & OT_INDEX) {
+		if (opt->type & INDEX) {
 			if (opt->index > 999) {
 				errno = ENOBUFS;
 				logger(ctx, LOG_ERR, "%s: %m", __func__);
@@ -935,13 +832,13 @@ dhcp_envoption(struct dhcpcd_ctx *ctx, char **env, const char *prefix,
 			}
 		}
 		e = strlen(prefix) + strlen(opt->var) + 2 +
-		    (opt->type & OT_INDEX ? 3 : 0);
+		    (opt->type & INDEX ? 3 : 0);
 		pfx = malloc(e);
 		if (pfx == NULL) {
 			logger(ctx, LOG_ERR, "%s: %m", __func__);
 			return 0;
 		}
-		if (opt->type & OT_INDEX)
+		if (opt->type & INDEX)
 			snprintf(pfx, e, "%s_%s%d", prefix,
 			    opt->var, ++opt->index);
 		else
@@ -953,46 +850,19 @@ dhcp_envoption(struct dhcpcd_ctx *ctx, char **env, const char *prefix,
 	 * is a fixed layout */
 	n = 0;
 	for (i = 0, eopt = opt->embopts; i < opt->embopts_len; i++, eopt++) {
-		eo = dhcp_optlen(eopt, ol);
-		if (eo == -1) {
-			if (env == NULL)
-				logger(ctx, LOG_ERR,
-				    "%s: %s %d.%d/%zu: "
-				    "malformed embedded option",
-				    ifname, __func__, opt->option,
-				    eopt->option, i);
-			goto out;
-		}
-		if (eo == 0) {
-			/* An option was expected, but there is no data
-			 * data for it.
-			 * This may not be an error as some options like
-			 * DHCP FQDN in RFC4702 have a string as the last
-			 * option which is optional. */
-			if (env == NULL &&
-			    (ol != 0 || !(eopt->type & OT_OPTIONAL)))
-				logger(ctx, LOG_ERR,
-				    "%s: %s %d.%d/%zu: missing embedded option",
-				    ifname, __func__, opt->option,
-				    eopt->option, i);
-			goto out;
-		}
+		e = dhcp_optlen(eopt, ol);
+		if (e == 0)
+			/* Report error? */
+			return 0;
 		/* Use the option prefix if the embedded option
 		 * name is different.
 		 * This avoids new_fqdn_fqdn which would be silly. */
-		if (!(eopt->type & OT_RESERVED)) {
-			ov = strcmp(opt->var, eopt->var);
-			if (dhcp_envoption1(env == NULL ? NULL : &env[n],
-			    pfx, eopt, ov, od, (size_t)eo, ifname))
-				n++;
-			else if (env == NULL)
-				logger(ctx, LOG_ERR,
-				    "%s: %s %d.%d/%zu: %m",
-				    ifname, __func__,
-				    opt->option, eopt->option, i);
-		}
-		od += (size_t)eo;
-		ol -= (size_t)eo;
+		ov = strcmp(opt->var, eopt->var);
+		if (dhcp_envoption1(ctx, env == NULL ? NULL : &env[n],
+		    pfx, eopt, ov, od, e, ifname))
+			n++;
+		od += e;
+		ol -= e;
 	}
 
 	/* Enumerate our encapsulated options */
@@ -1005,7 +875,7 @@ dhcp_envoption(struct dhcpcd_ctx *ctx, char **env, const char *prefix,
 		    i++, eopt++)
 		{
 			eoc = opt->option;
-			if (eopt->type & OT_OPTION) {
+			if (eopt->type & OPTION) {
 				dgetopt(ctx, NULL, &eoc, NULL, NULL, 0, &oopt);
 				if (oopt)
 					oopt->index = 0;
@@ -1018,7 +888,7 @@ dhcp_envoption(struct dhcpcd_ctx *ctx, char **env, const char *prefix,
 			    i++, eopt++)
 			{
 				if (eopt->option == eoc) {
-					if (eopt->type & OT_OPTION) {
+					if (eopt->type & OPTION) {
 						if (oopt == NULL)
 							/* Report error? */
 							continue;
@@ -1026,7 +896,7 @@ dhcp_envoption(struct dhcpcd_ctx *ctx, char **env, const char *prefix,
 					n += dhcp_envoption(ctx,
 					    env == NULL ? NULL : &env[n], pfx,
 					    ifname,
-					    eopt->type & OT_OPTION ? oopt:eopt,
+					    eopt->type & OPTION ? oopt : eopt,
 					    dgetopt, eod, eol);
 					break;
 				}
@@ -1036,7 +906,6 @@ dhcp_envoption(struct dhcpcd_ctx *ctx, char **env, const char *prefix,
 		}
 	}
 
-out:
 	if (env)
 		free(pfx);
 
@@ -1056,39 +925,3 @@ dhcp_zero_index(struct dhcp_opt *opt)
 	for (i = 0, o = opt->encopts; i < opt->encopts_len; i++, o++)
 		dhcp_zero_index(o);
 }
-
-size_t
-dhcp_read_lease_fd(int fd, uint8_t **lease)
-{
-	uint8_t *buf, *nbuf;
-	size_t len, new_len;
-	ssize_t bytes;
-
-	if ((buf = malloc(BUFSIZ)) == NULL)
-		goto out;
-
-	len = 0;
-	for (;;) {
-		bytes = read(fd, buf + len, BUFSIZ);
-		if (bytes == -1)
-			break;
-		if (bytes < BUFSIZ) {
-			*lease = buf;
-			return len + (size_t)bytes;
-		}
-		new_len = len + (BUFSIZ * 2);
-		if (new_len > UINT32_MAX || new_len < len) {
-			errno = E2BIG;
-			break;
-		}
-		if ((nbuf = realloc(buf, new_len)) == NULL)
-			break;
-		buf = nbuf;
-		len += BUFSIZ;
-	}
-
-	free(buf);
-out:
-	*lease = NULL;
-	return 0;
-}
