@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2016 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2015 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -35,12 +35,13 @@
 #include <sys/uio.h>
 #include <sys/utsname.h>
 
-#include "config.h"
-
 #include <arpa/inet.h>
 #include <net/bpf.h>
 #include <net/if.h>
 #include <net/if_dl.h>
+#ifdef __FreeBSD__ /* Needed so that including netinet6/in6_var.h works */
+#  include <net/if_var.h>
+#endif
 #include <net/if_media.h>
 #include <net/route.h>
 #include <netinet/if_ether.h>
@@ -57,7 +58,6 @@
 #  include <net80211/ieee80211_ioctl.h>
 #endif
 
-#include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <fnmatch.h>
@@ -75,12 +75,12 @@
 #undef IPV6CTL_ACCEPT_RTADV
 #endif
 
+#include "config.h"
 #include "common.h"
 #include "dhcp.h"
 #include "if.h"
 #include "if-options.h"
 #include "ipv4.h"
-#include "ipv4ll.h"
 #include "ipv6.h"
 #include "ipv6nd.h"
 
@@ -124,7 +124,27 @@ int
 if_openlinksocket(void)
 {
 
-	return xsocket(PF_ROUTE, SOCK_RAW, 0, O_NONBLOCK|O_CLOEXEC);
+#ifdef SOCK_CLOEXEC
+	return socket(PF_ROUTE, SOCK_RAW | SOCK_CLOEXEC | SOCK_NONBLOCK, 0);
+#else
+	int s, flags;
+
+	if ((s = socket(PF_ROUTE, SOCK_RAW, 0)) == -1)
+		return -1;
+	if ((flags = fcntl(s, F_GETFD, 0)) == -1 ||
+	    fcntl(s, F_SETFD, flags | FD_CLOEXEC) == -1)
+	{
+		close(s);
+	        return -1;
+	}
+	if ((flags = fcntl(s, F_GETFL, 0)) == -1 ||
+	    fcntl(s, F_SETFL, flags | O_NONBLOCK) == -1)
+	{
+		close(s);
+	        return -1;
+	}
+	return s;
+#endif
 }
 
 #if defined(INET) || defined(INET6)
@@ -141,9 +161,9 @@ if_linkaddr(struct sockaddr_dl *sdl, const struct interface *ifp)
 #endif
 
 static int
-if_getssid1(int s, const char *ifname, uint8_t *ssid)
+if_getssid1(const char *ifname, uint8_t *ssid)
 {
-	int retval = -1;
+	int s, retval = -1;
 #if defined(SIOCG80211NWID)
 	struct ifreq ifr;
 	struct ieee80211_nwid nwid;
@@ -152,6 +172,9 @@ if_getssid1(int s, const char *ifname, uint8_t *ssid)
 	char nwid[IEEE80211_NWID_LEN + 1];
 #endif
 
+	if ((s = socket(PF_INET, SOCK_DGRAM, 0)) == -1)
+		return -1;
+
 #if defined(SIOCG80211NWID) /* NetBSD */
 	memset(&ifr, 0, sizeof(ifr));
 	strlcpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
@@ -190,6 +213,7 @@ if_getssid1(int s, const char *ifname, uint8_t *ssid)
 	}
 #endif
 
+	close(s);
 	return retval;
 }
 
@@ -198,7 +222,7 @@ if_getssid(struct interface *ifp)
 {
 	int r;
 
-	r = if_getssid1(ifp->ctx->pf_inet_fd, ifp->name, ifp->ssid);
+	r = if_getssid1(ifp->name, ifp->ssid);
 	if (r != -1)
 		ifp->ssid_len = (unsigned int)r;
 	return r;
@@ -212,20 +236,23 @@ if_getssid(struct interface *ifp)
  * returning the SSID gives an error.
  */
 int
-if_vimaster(const struct dhcpcd_ctx *ctx, const char *ifname)
+if_vimaster(const char *ifname)
 {
-	int r;
+	int s, r;
 	struct ifmediareq ifmr;
 
+	if ((s = socket(PF_INET, SOCK_DGRAM, 0)) == -1)
+		return -1;
 	memset(&ifmr, 0, sizeof(ifmr));
 	strlcpy(ifmr.ifm_name, ifname, sizeof(ifmr.ifm_name));
-	r = ioctl(ctx->pf_inet_fd, SIOCGIFMEDIA, &ifmr);
+	r = ioctl(s, SIOCGIFMEDIA, &ifmr);
+	close(s);
 	if (r == -1)
 		return -1;
 	if (ifmr.ifm_status & IFM_AVALID &&
 	    IFM_TYPE(ifmr.ifm_active) == IFM_IEEE80211)
 	{
-		if (if_getssid1(ctx->pf_inet_fd, ifname, NULL) == -1)
+		if (if_getssid1(ifname, NULL) == -1)
 			return 1;
 	}
 	return 0;
@@ -250,27 +277,12 @@ static struct interface *
 if_findsdl(struct dhcpcd_ctx *ctx, struct sockaddr_dl *sdl)
 {
 
-	if (sdl->sdl_index)
-		return if_findindex(ctx->ifaces, sdl->sdl_index);
-
 	if (sdl->sdl_nlen) {
 		char ifname[IF_NAMESIZE];
-
 		memcpy(ifname, sdl->sdl_data, sdl->sdl_nlen);
 		ifname[sdl->sdl_nlen] = '\0';
 		return if_find(ctx->ifaces, ifname);
 	}
-	if (sdl->sdl_alen) {
-		struct interface *ifp;
-
-		TAILQ_FOREACH(ifp, ctx->ifaces, next) {
-			if (ifp->hwlen == sdl->sdl_alen &&
-			    memcmp(ifp->hwaddr,
-			    sdl->sdl_data, sdl->sdl_alen) == 0)
-				return ifp;
-		}
-	}
-
 	return NULL;
 }
 #endif
@@ -281,7 +293,7 @@ const char *if_pfname = "Berkley Packet Filter";
 int
 if_openrawsocket(struct interface *ifp, uint16_t protocol)
 {
-	struct ipv4_state *state;
+	struct dhcp_state *state;
 	int fd = -1;
 	struct ifreq ifr;
 	int ibuf_len = 0;
@@ -306,7 +318,8 @@ if_openrawsocket(struct interface *ifp, uint16_t protocol)
 	if (fd == -1)
 		return -1;
 
-	state = IPV4_STATE(ifp);
+	state = D_STATE(ifp);
+
 	memset(&pv, 0, sizeof(pv));
 	if (ioctl(fd, BIOCVERSION, &pv) == -1)
 		goto eexit;
@@ -368,6 +381,7 @@ if_sendrawpacket(const struct interface *ifp, uint16_t protocol,
 	struct iovec iov[2];
 	struct ether_header hw;
 	int fd;
+	const struct dhcp_state *state;
 
 	memset(&hw, 0, ETHER_HDR_LEN);
 	memset(&hw.ether_dhost, 0xff, ETHER_ADDR_LEN);
@@ -376,7 +390,11 @@ if_sendrawpacket(const struct interface *ifp, uint16_t protocol,
 	iov[0].iov_len = ETHER_HDR_LEN;
 	iov[1].iov_base = UNCONST(data);
 	iov[1].iov_len = len;
-	fd = ipv4_protocol_fd(ifp, protocol);
+	state = D_CSTATE(ifp);
+	if (protocol == ETHERTYPE_ARP)
+		fd = state->arp_fd;
+	else
+		fd = state->raw_fd;
 	return writev(fd, iov, 2);
 }
 
@@ -390,10 +408,13 @@ if_readrawpacket(struct interface *ifp, uint16_t protocol,
 	struct bpf_hdr packet;
 	ssize_t bytes;
 	const unsigned char *payload;
-	struct ipv4_state *state;
+	struct dhcp_state *state;
 
-	state = IPV4_STATE(ifp);
-	fd = ipv4_protocol_fd(ifp, protocol);
+	state = D_STATE(ifp);
+	if (protocol == ETHERTYPE_ARP)
+		fd = state->arp_fd;
+	else
+		fd = state->raw_fd;
 
 	*flags = 0;
 	for (;;) {
@@ -435,9 +456,12 @@ if_address(const struct interface *ifp, const struct in_addr *address,
     const struct in_addr *netmask, const struct in_addr *broadcast,
     int action)
 {
-	int r;
+	int s, r;
 	struct in_aliasreq ifra;
 
+	if ((s = socket(PF_INET, SOCK_DGRAM, 0)) == -1)
+		return -1;
+
 	memset(&ifra, 0, sizeof(ifra));
 	strlcpy(ifra.ifra_name, ifp->name, sizeof(ifra.ifra_name));
 
@@ -452,8 +476,9 @@ if_address(const struct interface *ifp, const struct in_addr *address,
 		ADDADDR(&ifra.ifra_broadaddr, broadcast);
 #undef ADDADDR
 
-	r = ioctl(ifp->ctx->pf_inet_fd,
+	r = ioctl(s,
 	    action < 0 ? SIOCDIFADDR : SIOCAIFADDR, &ifra);
+	close(s);
 	return r;
 }
 
@@ -462,7 +487,6 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, struct rt_msghdr *rtm)
 {
 	char *cp;
 	struct sockaddr *sa, *rti_info[RTAX_MAX];
-	struct sockaddr_dl *sdl;
 
 	cp = (char *)(void *)(rtm + 1);
 	sa = (struct sockaddr *)(void *)cp;
@@ -485,27 +509,21 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, struct rt_msghdr *rtm)
 
 	get_addrs(rtm->rtm_addrs, cp, rti_info);
 	memset(rt, 0, sizeof(*rt));
-	rt->flags = (unsigned int)rtm->rtm_flags;
 	COPYOUT(rt->dest, rti_info[RTAX_DST]);
 	if (rtm->rtm_addrs & RTA_NETMASK)
 		COPYOUT(rt->net, rti_info[RTAX_NETMASK]);
 	else
 		rt->net.s_addr = INADDR_BROADCAST;
 	COPYOUT(rt->gate, rti_info[RTAX_GATEWAY]);
-	COPYOUT(rt->src, rti_info[RTAX_IFA]);
-	rt->mtu = (unsigned int)rtm->rtm_rmx.rmx_mtu;
 
 	if (rtm->rtm_index)
 		rt->iface = if_findindex(ctx->ifaces, rtm->rtm_index);
 	else if (rtm->rtm_addrs & RTA_IFP) {
+		struct sockaddr_dl *sdl;
+
 		sdl = (struct sockaddr_dl *)(void *)rti_info[RTAX_IFP];
 		rt->iface = if_findsdl(ctx, sdl);
-	} else if (rtm->rtm_addrs & RTA_GATEWAY) {
-		sdl = (struct sockaddr_dl *)(void *)rti_info[RTAX_GATEWAY];
-		if (sdl->sdl_family == AF_LINK)
-			rt->iface = if_findsdl(ctx, sdl);
 	}
-
 	/* If we don't have an interface and it's a host route, it maybe
 	 * to a local ip via the loopback interface. */
 	if (rt->iface == NULL &&
@@ -517,14 +535,13 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, struct rt_msghdr *rtm)
 			rt->iface = ia->iface;
 	}
 
-	assert(rt->iface != NULL);
-
 	return 0;
 }
 
 int
 if_route(unsigned char cmd, const struct rt *rt)
 {
+	const struct dhcp_state *state;
 	union sockunion {
 		struct sockaddr sa;
 		struct sockaddr_in sin;
@@ -537,12 +554,10 @@ if_route(unsigned char cmd, const struct rt *rt)
 	} rtm;
 	char *bp = rtm.buffer;
 	size_t l;
-	struct in_addr src_addr;
+	int s, retval;
 
-	if ((cmd == RTM_ADD || cmd == RTM_DELETE || cmd == RTM_CHANGE) &&
-	    rt->iface->ctx->options & DHCPCD_DAEMONISE &&
-	    !(rt->iface->ctx->options & DHCPCD_DAEMONISED))
-		rt->iface->ctx->options |= DHCPCD_RTM_PPID;
+	if ((s = socket(PF_ROUTE, SOCK_RAW, 0)) == -1)
+		return -1;
 
 #define ADDSU {								      \
 		l = RT_ROUNDUP(su.sa.sa_len);				      \
@@ -553,40 +568,42 @@ if_route(unsigned char cmd, const struct rt *rt)
 		memset(&su, 0, sizeof(su));				      \
 		su.sin.sin_family = AF_INET;				      \
 		su.sin.sin_len = sizeof(su.sin);			      \
-		(&su.sin)->sin_addr = *(addr);				      \
+		(&su.sin)->sin_addr = *addr;				      \
 		ADDSU;							      \
 	}
 
+	if (cmd != RTM_DELETE)
+		state = D_CSTATE(rt->iface);
+	else	/* appease GCC */
+		state = NULL;
 	memset(&rtm, 0, sizeof(rtm));
 	rtm.hdr.rtm_version = RTM_VERSION;
+	rtm.hdr.rtm_seq = 1;
 	rtm.hdr.rtm_type = cmd;
 	rtm.hdr.rtm_addrs = RTA_DST;
+	if (cmd == RTM_ADD || cmd == RTM_CHANGE)
+		rtm.hdr.rtm_addrs |= RTA_GATEWAY;
 	rtm.hdr.rtm_flags = RTF_UP;
-	rtm.hdr.rtm_pid = getpid();
-	rtm.hdr.rtm_seq = ++rt->iface->ctx->seq;
 #ifdef RTF_PINNED
 	if (cmd != RTM_ADD)
 		rtm.hdr.rtm_flags |= RTF_PINNED;
 #endif
 
-	if (cmd == RTM_ADD || cmd == RTM_CHANGE) {
-		int subnet;
-
-		rtm.hdr.rtm_addrs |= RTA_GATEWAY | RTA_IFA | RTA_IFP;
-		/* Subnet routes are clonning or connected if supported.
-		 * All other routes are static. */
-		subnet = ipv4_srcaddr(rt, &src_addr);
-		if (subnet == 1) {
+	if (cmd != RTM_DELETE) {
+		rtm.hdr.rtm_addrs |= RTA_IFA | RTA_IFP;
+		/* None interface subnet routes are static. */
+		if (rt->gate.s_addr != INADDR_ANY ||
+		    rt->net.s_addr != state->net.s_addr ||
+		    rt->dest.s_addr != (state->addr.s_addr & state->net.s_addr))
+			rtm.hdr.rtm_flags |= RTF_STATIC;
+		else {
 #ifdef RTF_CLONING
 			rtm.hdr.rtm_flags |= RTF_CLONING;
 #endif
 #ifdef RTP_CONNECTED
 			rtm.hdr.rtm_priority = RTP_CONNECTED;
 #endif
-		} else
-			rtm.hdr.rtm_flags |= RTF_STATIC;
-		if (subnet == -1) /* unikely */
-			rtm.hdr.rtm_addrs &= ~RTA_IFA;
+		}
 	}
 	if (rt->net.s_addr == htonl(INADDR_BROADCAST) &&
 	    rt->gate.s_addr == htonl(INADDR_ANY))
@@ -614,20 +631,19 @@ if_route(unsigned char cmd, const struct rt *rt)
 		rtm.hdr.rtm_addrs |= RTA_NETMASK;
 		if (rtm.hdr.rtm_flags & RTF_STATIC)
 			rtm.hdr.rtm_flags |= RTF_GATEWAY;
-		if (rt->net.s_addr == htonl(INADDR_BROADCAST))
-			rtm.hdr.rtm_flags |= RTF_HOST;
 	}
 	if ((cmd == RTM_ADD || cmd == RTM_CHANGE) &&
 	    !(rtm.hdr.rtm_flags & RTF_GATEWAY))
-		rtm.hdr.rtm_addrs |= RTA_IFP;
+		rtm.hdr.rtm_addrs |= RTA_IFA | RTA_IFP;
 
 	ADDADDR(&rt->dest);
 	if (rtm.hdr.rtm_addrs & RTA_GATEWAY) {
-		if ((rtm.hdr.rtm_flags & RTF_HOST &&
-		    rt->gate.s_addr == htonl(INADDR_ANY)) ||
 #ifdef RTF_CLONING
-		    rtm.hdr.rtm_flags & RTF_CLONING ||
+		if ((rtm.hdr.rtm_flags & (RTF_HOST | RTF_CLONING) &&
+#else
+		if ((rtm.hdr.rtm_flags & RTF_HOST &&
 #endif
+		    rt->gate.s_addr != htonl(INADDR_LOOPBACK)) ||
 		    !(rtm.hdr.rtm_flags & RTF_STATIC))
 		{
 			if_linkaddr(&su.sdl, rt->iface);
@@ -649,22 +665,16 @@ if_route(unsigned char cmd, const struct rt *rt)
 		}
 
 		if (rtm.hdr.rtm_addrs & RTA_IFA)
-			ADDADDR(&src_addr);
-
-		if (rt->mtu) {
-			rtm.hdr.rtm_inits |= RTV_MTU;
-			rtm.hdr.rtm_rmx.rmx_mtu = rt->mtu;
-		}
+			ADDADDR(&state->addr);
 	}
 
 #undef ADDADDR
 #undef ADDSU
 
 	rtm.hdr.rtm_msglen = (unsigned short)(bp - (char *)&rtm);
-	if (write(rt->iface->ctx->link_fd, &rtm, rtm.hdr.rtm_msglen) == -1)
-		return -1;
-	rt->iface->ctx->sseq = rt->iface->ctx->seq;
-	return 0;
+	retval = write(s, &rtm, rtm.hdr.rtm_msglen) == -1 ? -1 : 0;
+	close(s);
+	return retval;
 }
 
 int
@@ -708,17 +718,23 @@ if_initrt(struct interface *ifp)
 int
 if_addrflags(const struct in_addr *addr, const struct interface *ifp)
 {
+	int s, flags;
 	struct ifreq ifr;
 	struct sockaddr_in *sin;
 
-	memset(&ifr, 0, sizeof(ifr));
-	strlcpy(ifr.ifr_name, ifp->name, sizeof(ifr.ifr_name));
-	sin = (struct sockaddr_in *)(void *)&ifr.ifr_addr;
-	sin->sin_family = AF_INET;
-	sin->sin_addr = *addr;
-	if (ioctl(ifp->ctx->pf_inet_fd, SIOCGIFAFLAG_IN, &ifr) == -1)
-		return -1;
-	return ifr.ifr_addrflags;
+	s = socket(PF_INET, SOCK_DGRAM, 0);
+	flags = -1;
+	if (s != -1) {
+		memset(&ifr, 0, sizeof(ifr));
+		strlcpy(ifr.ifr_name, ifp->name, sizeof(ifr.ifr_name));
+		sin = (struct sockaddr_in *)(void *)&ifr.ifr_addr;
+		sin->sin_family = AF_INET;
+		sin->sin_addr = *addr;
+		if (ioctl(s, SIOCGIFAFLAG_IN, &ifr) != -1)
+			flags = ifr.ifr_addrflags;
+		close(s);
+	}
+	return flags;
 }
 #else
 int
@@ -764,13 +780,17 @@ ifa_scope(struct sockaddr_in6 *sin, unsigned int ifindex)
 #endif
 
 int
-if_address6(const struct ipv6_addr *ia, int action)
+if_address6(const struct ipv6_addr *a, int action)
 {
+	int s, r;
 	struct in6_aliasreq ifa;
 	struct in6_addr mask;
 
+	if ((s = socket(PF_INET6, SOCK_DGRAM, 0)) == -1)
+		return -1;
+
 	memset(&ifa, 0, sizeof(ifa));
-	strlcpy(ifa.ifra_name, ia->iface->name, sizeof(ifa.ifra_name));
+	strlcpy(ifa.ifra_name, a->iface->name, sizeof(ifa.ifra_name));
 	/*
 	 * We should not set IN6_IFF_TENTATIVE as the kernel should be
 	 * able to work out if it's a new address or not.
@@ -780,30 +800,31 @@ if_address6(const struct ipv6_addr *ia, int action)
 	 * either.
 	 */
 #if 0
-	if (ia->autoconf)
+	if (a->autoconf)
 		ifa.ifra_flags |= IN6_IFF_AUTOCONF;
 #endif
 #ifdef IPV6_MANGETEMPADDR
-	if (ia->flags & IPV6_AF_TEMPORARY)
+	if (a->flags & IPV6_AF_TEMPORARY)
 		ifa.ifra_flags |= IN6_IFF_TEMPORARY;
 #endif
 
 #define ADDADDR(v, addr) {						      \
 		(v)->sin6_family = AF_INET6;				      \
 		(v)->sin6_len = sizeof(*v);				      \
-		(v)->sin6_addr = *(addr);				      \
+		(v)->sin6_addr = *addr;					      \
 	}
 
-	ADDADDR(&ifa.ifra_addr, &ia->addr);
-	ifa_scope(&ifa.ifra_addr, ia->iface->index);
-	ipv6_mask(&mask, ia->prefix_len);
+	ADDADDR(&ifa.ifra_addr, &a->addr);
+	ifa_scope(&ifa.ifra_addr, a->iface->index);
+	ipv6_mask(&mask, a->prefix_len);
 	ADDADDR(&ifa.ifra_prefixmask, &mask);
-	ifa.ifra_lifetime.ia6t_vltime = ia->prefix_vltime;
-	ifa.ifra_lifetime.ia6t_pltime = ia->prefix_pltime;
+	ifa.ifra_lifetime.ia6t_vltime = a->prefix_vltime;
+	ifa.ifra_lifetime.ia6t_pltime = a->prefix_pltime;
 #undef ADDADDR
 
-	return ioctl(ia->iface->ctx->pf_inet6_fd,
-	    action < 0 ? SIOCDIFADDR_IN6 : SIOCAIFADDR_IN6, &ifa);
+	r = ioctl(s, action < 0 ? SIOCDIFADDR_IN6 : SIOCAIFADDR_IN6, &ifa);
+	close(s);
+	return r;
 }
 
 
@@ -812,7 +833,6 @@ if_copyrt6(struct dhcpcd_ctx *ctx, struct rt6 *rt, struct rt_msghdr *rtm)
 {
 	char *cp;
 	struct sockaddr *sa, *rti_info[RTAX_MAX];
-	struct sockaddr_dl *sdl;
 
 	cp = (char *)(void *)(rtm + 1);
 	sa = (struct sockaddr *)(void *)cp;
@@ -884,19 +904,15 @@ if_copyrt6(struct dhcpcd_ctx *ctx, struct rt6 *rt, struct rt_msghdr *rtm)
 	} else
 		ipv6_mask(&rt->net, 128);
 	COPYOUT6(rt->gate, rti_info[RTAX_GATEWAY]);
-	rt->mtu = (unsigned int)rtm->rtm_rmx.rmx_mtu;
 
 	if (rtm->rtm_index)
 		rt->iface = if_findindex(ctx->ifaces, rtm->rtm_index);
 	else if (rtm->rtm_addrs & RTA_IFP) {
+		struct sockaddr_dl *sdl;
+
 		sdl = (struct sockaddr_dl *)(void *)rti_info[RTAX_IFP];
 		rt->iface = if_findsdl(ctx, sdl);
-	} else if (rtm->rtm_addrs & RTA_GATEWAY) {
-		sdl = (struct sockaddr_dl *)(void *)rti_info[RTAX_GATEWAY];
-		if (sdl->sdl_family == AF_LINK)
-			rt->iface = if_findsdl(ctx, sdl);
 	}
-
 	/* If we don't have an interface and it's a host route, it maybe
 	 * to a local ip via the loopback interface. */
 	if (rt->iface == NULL &&
@@ -908,8 +924,6 @@ if_copyrt6(struct dhcpcd_ctx *ctx, struct rt6 *rt, struct rt_msghdr *rtm)
 			rt->iface = ia->iface;
 	}
 
-	assert(rt->iface != NULL);
-
 	return 0;
 }
 
@@ -928,11 +942,10 @@ if_route6(unsigned char cmd, const struct rt6 *rt)
 	} rtm;
 	char *bp = rtm.buffer;
 	size_t l;
+	int s, retval;
 
-	if ((cmd == RTM_ADD || cmd == RTM_DELETE || cmd == RTM_CHANGE) &&
-	    rt->iface->ctx->options & DHCPCD_DAEMONISE &&
-	    !(rt->iface->ctx->options & DHCPCD_DAEMONISED))
-		rt->iface->ctx->options |= DHCPCD_RTM_PPID;
+	if ((s = socket(PF_ROUTE, SOCK_RAW, 0)) == -1)
+		return -1;
 
 #define ADDSU {								      \
 		l = RT_ROUNDUP(su.sa.sa_len);				      \
@@ -952,10 +965,9 @@ if_route6(unsigned char cmd, const struct rt6 *rt)
 
 	memset(&rtm, 0, sizeof(rtm));
 	rtm.hdr.rtm_version = RTM_VERSION;
+	rtm.hdr.rtm_seq = 1;
 	rtm.hdr.rtm_type = cmd;
 	rtm.hdr.rtm_flags = RTF_UP | (int)rt->flags;
-	rtm.hdr.rtm_pid = getpid();
-	rtm.hdr.rtm_seq = ++rt->iface->ctx->seq;
 #ifdef RTF_PINNED
 	if (rtm.hdr.rtm_type != RTM_ADD)
 		rtm.hdr.rtm_flags |= RTF_PINNED;
@@ -1018,10 +1030,9 @@ if_route6(unsigned char cmd, const struct rt6 *rt)
 #undef ADDSU
 
 	rtm.hdr.rtm_msglen = (unsigned short)(bp - (char *)&rtm);
-	if (write(rt->iface->ctx->link_fd, &rtm, rtm.hdr.rtm_msglen) == -1)
-		return -1;
-	rt->iface->ctx->sseq = rt->iface->ctx->seq;
-	return 0;
+	retval = write(s, &rtm, rtm.hdr.rtm_msglen) == -1 ? -1 : 0;
+	close(s);
+	return retval;
 }
 
 int
@@ -1064,54 +1075,67 @@ if_initrt6(struct interface *ifp)
 int
 if_addrflags6(const struct in6_addr *addr, const struct interface *ifp)
 {
-	int flags;
+	int s, flags;
 	struct in6_ifreq ifr6;
 
-	memset(&ifr6, 0, sizeof(ifr6));
-	strlcpy(ifr6.ifr_name, ifp->name, sizeof(ifr6.ifr_name));
-	ifr6.ifr_addr.sin6_family = AF_INET6;
-	ifr6.ifr_addr.sin6_addr = *addr;
-	ifa_scope(&ifr6.ifr_addr, ifp->index);
-	if (ioctl(ifp->ctx->pf_inet6_fd, SIOCGIFAFLAG_IN6, &ifr6) != -1)
-		flags = ifr6.ifr_ifru.ifru_flags6;
-	else
-		flags = -1;
+	s = socket(PF_INET6, SOCK_DGRAM, 0);
+	flags = -1;
+	if (s != -1) {
+		memset(&ifr6, 0, sizeof(ifr6));
+		strlcpy(ifr6.ifr_name, ifp->name, sizeof(ifr6.ifr_name));
+		ifr6.ifr_addr.sin6_family = AF_INET6;
+		ifr6.ifr_addr.sin6_addr = *addr;
+		ifa_scope(&ifr6.ifr_addr, ifp->index);
+		if (ioctl(s, SIOCGIFAFLAG_IN6, &ifr6) != -1)
+			flags = ifr6.ifr_ifru.ifru_flags6;
+		close(s);
+	}
 	return flags;
 }
 
 int
 if_getlifetime6(struct ipv6_addr *ia)
 {
+	int s, r;
 	struct in6_ifreq ifr6;
-	time_t t;
-	struct in6_addrlifetime *lifetime;
 
-	memset(&ifr6, 0, sizeof(ifr6));
-	strlcpy(ifr6.ifr_name, ia->iface->name, sizeof(ifr6.ifr_name));
-	ifr6.ifr_addr.sin6_family = AF_INET6;
-	ifr6.ifr_addr.sin6_addr = ia->addr;
-	ifa_scope(&ifr6.ifr_addr, ia->iface->index);
-	if (ioctl(ia->iface->ctx->pf_inet6_fd,
-	    SIOCGIFALIFETIME_IN6, &ifr6) == -1)
-		return -1;
+	s = socket(PF_INET6, SOCK_DGRAM, 0);
+	r = -1;
+	if (s != -1) {
+		memset(&ifr6, 0, sizeof(ifr6));
+		strlcpy(ifr6.ifr_name, ia->iface->name, sizeof(ifr6.ifr_name));
+		ifr6.ifr_addr.sin6_family = AF_INET6;
+		ifr6.ifr_addr.sin6_addr = ia->addr;
+		ifa_scope(&ifr6.ifr_addr, ia->iface->index);
+		if (ioctl(s, SIOCGIFALIFETIME_IN6, &ifr6) != -1) {
+			time_t t;
+			struct in6_addrlifetime *lifetime;
 
-	t = time(NULL);
-	lifetime = &ifr6.ifr_ifru.ifru_lifetime;
+			t = time(NULL);
+			lifetime = &ifr6.ifr_ifru.ifru_lifetime;
 
-	if (lifetime->ia6t_preferred)
-		ia->prefix_pltime = (uint32_t)(lifetime->ia6t_preferred -
-		    MIN(t, lifetime->ia6t_preferred));
-	else
-		ia->prefix_pltime = ND6_INFINITE_LIFETIME;
-	if (lifetime->ia6t_expire) {
-		ia->prefix_vltime = (uint32_t)(lifetime->ia6t_expire -
-		    MIN(t, lifetime->ia6t_expire));
-		/* Calculate the created time */
-		clock_gettime(CLOCK_MONOTONIC, &ia->created);
-		ia->created.tv_sec -= lifetime->ia6t_vltime - ia->prefix_vltime;
-	} else
-		ia->prefix_vltime = ND6_INFINITE_LIFETIME;
-	return 0;
+			if (lifetime->ia6t_preferred)
+				ia->prefix_pltime =
+				    (uint32_t)(lifetime->ia6t_preferred -
+				    MIN(t, lifetime->ia6t_preferred));
+			else
+				ia->prefix_pltime = ND6_INFINITE_LIFETIME;
+			if (lifetime->ia6t_expire) {
+				ia->prefix_vltime =
+				    (uint32_t)(lifetime->ia6t_expire -
+				    MIN(t, lifetime->ia6t_expire));
+				/* Calculate the created time */
+				get_monotonic(&ia->created);
+				ia->created.tv_sec -=
+				    lifetime->ia6t_vltime - ia->prefix_vltime;
+			} else
+				ia->prefix_vltime = ND6_INFINITE_LIFETIME;
+
+			r = 0;
+		}
+		close(s);
+	}
+	return r;
 }
 #endif
 
@@ -1146,26 +1170,9 @@ if_managelink(struct dhcpcd_ctx *ctx)
 	e = msg + bytes;
 	for (p = msg; p < e; p += rtm->rtm_msglen) {
 		rtm = (struct rt_msghdr *)(void *)p;
-		if (rtm->rtm_type == RTM_MISS)
-			continue;
-		/* Ignore messages generated by us */
-		if (rtm->rtm_pid == getpid()) {
-			ctx->options &= ~DHCPCD_RTM_PPID;
-			continue;
-		}
-		/* Ignore messages sent by the parent process after forking */
-		if ((ctx->options & (DHCPCD_RTM_PPID | DHCPCD_DAEMONISED)) ==
-		    (DHCPCD_RTM_PPID | DHCPCD_DAEMONISED) &&
-		    rtm->rtm_pid == ctx->ppid)
-		{
-			/* If this is the last successful message sent clear
-			 * the check flag as it's possible another process could
-			 * re-use the same pid and also manipulate the kernel
-			 * routing table. */
-			if (rtm->rtm_seq == ctx->pseq)
-				ctx->options &= ~DHCPCD_RTM_PPID;
-			continue;
-		}
+		// Ignore messages generated by us
+		if (rtm->rtm_pid == getpid())
+			break;
 		switch(rtm->rtm_type) {
 #ifdef RTM_IFANNOUNCE
 		case RTM_IFANNOUNCE:
@@ -1525,6 +1532,24 @@ _if_checkipv6(int s, struct dhcpcd_ctx *ctx,
 		int override;
 #endif
 
+#ifdef ND6_IFF_IFDISABLED
+		if (del_if_nd6_flag(s, ifp, ND6_IFF_IFDISABLED) == -1) {
+			logger(ifp->ctx, LOG_ERR,
+			    "%s: del_if_nd6_flag: ND6_IFF_IFDISABLED: %m",
+			    ifp->name);
+			return -1;
+		}
+#endif
+
+#ifdef ND6_IFF_PERFORMNUD
+		if (set_if_nd6_flag(s, ifp, ND6_IFF_PERFORMNUD) == -1) {
+			logger(ifp->ctx, LOG_ERR,
+			    "%s: set_if_nd6_flag: ND6_IFF_PERFORMNUD: %m",
+			    ifp->name);
+			return -1;
+		}
+#endif
+
 #ifdef ND6_IFF_AUTO_LINKLOCAL
 		if (own) {
 			int all;
@@ -1553,11 +1578,18 @@ _if_checkipv6(int s, struct dhcpcd_ctx *ctx,
 		}
 #endif
 
-#ifdef ND6_IFF_PERFORMNUD
-		if (set_if_nd6_flag(s, ifp, ND6_IFF_PERFORMNUD) == -1) {
+#ifdef SIOCIFAFATTACH
+		if (af_attach(s, ifp, AF_INET6) == -1) {
 			logger(ifp->ctx, LOG_ERR,
-			    "%s: set_if_nd6_flag: ND6_IFF_PERFORMNUD: %m",
-			    ifp->name);
+			    "%s: af_attach: %m", ifp->name);
+			return 1;
+		}
+#endif
+
+#ifdef SIOCGIFXFLAGS
+		if (set_ifxflags(s, ifp, own) == -1) {
+			logger(ifp->ctx, LOG_ERR,
+			    "%s: set_ifxflags: %m", ifp->name);
 			return -1;
 		}
 #endif
@@ -1601,38 +1633,6 @@ _if_checkipv6(int s, struct dhcpcd_ctx *ctx,
 		} else if (ra == 0 && !own)
 			logger(ifp->ctx, LOG_WARNING,
 			    "%s: IPv6 kernel autoconf disabled", ifp->name);
-#endif
-
-		/* Enabling IPv6 by whatever means must be the
-		 * last action undertaken to ensure kernel RS and
-		 * LLADDR auto configuration are disabled where applicable. */
-
-#ifdef SIOCIFAFATTACH
-		if (af_attach(s, ifp, AF_INET6) == -1) {
-			logger(ifp->ctx, LOG_ERR,
-			    "%s: af_attach: %m", ifp->name);
-			return 1;
-		}
-#endif
-
-#ifdef SIOCGIFXFLAGS
-		if (set_ifxflags(s, ifp, own) == -1) {
-			logger(ifp->ctx, LOG_ERR,
-			    "%s: set_ifxflags: %m", ifp->name);
-			return -1;
-		}
-#endif
-
-#ifdef ND6_IFF_IFDISABLED
-		if (del_if_nd6_flag(s, ifp, ND6_IFF_IFDISABLED) == -1) {
-			logger(ifp->ctx, LOG_ERR,
-			    "%s: del_if_nd6_flag: ND6_IFF_IFDISABLED: %m",
-			    ifp->name);
-			return -1;
-		}
-#endif
-
-#ifdef ND6_IFF_ACCEPT_RTADV
 #ifdef ND6_IFF_OVERRIDE_RTADV
 		if (override == 0 && ra)
 			return ctx->ra_global;
@@ -1648,12 +1648,12 @@ _if_checkipv6(int s, struct dhcpcd_ctx *ctx,
 	if (ra == -1)
 		/* The sysctl probably doesn't exist, but this isn't an
 		 * error as such so just log it and continue */
-		logger(ctx, errno == ENOENT ? LOG_DEBUG : LOG_WARNING,
+		logger(ifp->ctx, errno == ENOENT ? LOG_DEBUG : LOG_WARNING,
 		    "IPV6CTL_ACCEPT_RTADV: %m");
 	else if (ra != 0 && own) {
-		logger(ctx, LOG_DEBUG, "disabling Kernel IPv6 RA support");
+		logger(ifp->ctx, LOG_DEBUG, "disabling Kernel IPv6 RA support");
 		if (set_inet6_sysctl(IPV6CTL_ACCEPT_RTADV, 0) == -1) {
-			logger(ctx, LOG_ERR, "IPV6CTL_ACCEPT_RTADV: %m");
+			logger(ifp->ctx, LOG_ERR, "IPV6CTL_ACCEPT_RTADV: %m");
 			return ra;
 		}
 		ra = 0;
@@ -1675,7 +1675,12 @@ _if_checkipv6(int s, struct dhcpcd_ctx *ctx,
 int
 if_checkipv6(struct dhcpcd_ctx *ctx, const struct interface *ifp, int own)
 {
+	int s, r;
 
-	return _if_checkipv6(ctx->pf_inet6_fd, ctx, ifp, own);
+	if ((s = socket(PF_INET6, SOCK_DGRAM, 0)) == -1)
+		return -1;
+	r = _if_checkipv6(s, ctx, ifp, own);
+	close(s);
+	return r;
 }
 #endif
