@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2016 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2015 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -46,10 +46,27 @@
 #include <netinet/in.h>
 #include <net/route.h>
 
+/* Support older kernels */
+#ifndef IFLA_WIRELESS
+# define IFLA_WIRELESS (IFLA_MASTER + 1)
+#endif
+
+/* Linux has these in an enum and there is just no way to work
+ * out of they exist at compile time. Silly silly silly. */
+#define IFLA_AF_SPEC			26
+#define IFLA_INET6_ADDR_GEN_MODE	8
+#define IN6_ADDR_GEN_MODE_NONE		1
+
+/* For some reason, glibc doesn't include newer flags from linux/if.h
+ * However, we cannot include linux/if.h directly as it conflicts
+ * with the glibc version. D'oh! */
+#ifndef IFF_LOWER_UP
+#define IFF_LOWER_UP	0x10000		/* driver signals L1 up		*/
+#endif
+
 #include <errno.h>
 #include <fcntl.h>
 #include <ctype.h>
-#include <stdbool.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -63,7 +80,6 @@
 #include "dhcp.h"
 #include "if.h"
 #include "ipv4.h"
-#include "ipv4ll.h"
 #include "ipv6.h"
 #include "ipv6nd.h"
 
@@ -73,18 +89,6 @@
 #endif
 int if_getssid_wext(const char *ifname, uint8_t *ssid);
 
-/* Support older kernels */
-#ifndef IFLA_WIRELESS
-#define IFLA_WIRELESS (IFLA_MASTER + 1)
-#endif
-
-/* For some reason, glibc doesn't include newer flags from linux/if.h
- * However, we cannot include linux/if.h directly as it conflicts
- * with the glibc version. D'oh! */
-#ifndef IFF_LOWER_UP
-#define IFF_LOWER_UP	0x10000		/* driver signals L1 up		*/
-#endif
-
 #define bpf_insn		sock_filter
 #define BPF_SKIPTYPE
 #define BPF_ETHCOOK		-ETH_HLEN
@@ -92,12 +96,6 @@ int if_getssid_wext(const char *ifname, uint8_t *ssid);
 
 #include "bpf-filter.h"
 
-struct priv {
-	int route_fd;
-	uint32_t route_pid;
-	struct iovec sndrcv_iov[1];
-};
-
 /* Broadcast address for IPoIB */
 static const uint8_t ipv4_bcast_addr[] = {
 	0x00, 0xff, 0xff, 0xff,
@@ -244,19 +242,35 @@ if_conf(struct interface *ifp)
 
 /* XXX work out Virtal Interface Masters */
 int
-if_vimaster(__unused const struct dhcpcd_ctx *ctx, __unused const char *ifname)
+if_vimaster(__unused const char *ifname)
 {
 
 	return 0;
 }
 
 static int
-_open_link_socket(struct sockaddr_nl *nl, int protocol)
+_open_link_socket(struct sockaddr_nl *nl, int flags, int protocol)
 {
 	int fd;
 
-	if ((fd = xsocket(AF_NETLINK, SOCK_RAW | SOCK_CLOEXEC, protocol)) == -1)
+#ifdef SOCK_CLOEXEC
+	if (flags)
+		flags = SOCK_CLOEXEC;
+	fd = socket(AF_NETLINK, SOCK_RAW | flags, protocol);
+	if (fd == -1)
 		return -1;
+#else
+	fd = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
+	if (fd == -1)
+		return -1;
+	if (flags &&
+	    (flags = fcntl(fd, F_GETFD, 0)) == -1 ||
+	    fcntl(fd, F_SETFD, flags | FD_CLOEXEC) == -1)
+	{
+		close(fd);
+	        return -1;
+	}
+#endif
 	nl->nl_family = AF_NETLINK;
 	if (bind(fd, (struct sockaddr *)nl, sizeof(*nl)) == -1) {
 		close(fd);
@@ -266,15 +280,10 @@ _open_link_socket(struct sockaddr_nl *nl, int protocol)
 }
 
 int
-if_opensockets_os(struct dhcpcd_ctx *ctx)
+if_openlinksocket(void)
 {
-	struct priv *priv;
 	struct sockaddr_nl snl;
-	socklen_t len;
 
-	/* Open the link socket first so it gets pid() for the socket.
-	 * Then open our persistent route socket so we get a unique
-	 * pid that doesn't clash with a process id for after we fork. */
 	memset(&snl, 0, sizeof(snl));
 	snl.nl_groups = RTMGRP_LINK;
 
@@ -285,104 +294,99 @@ if_opensockets_os(struct dhcpcd_ctx *ctx)
 	snl.nl_groups |= RTMGRP_IPV6_ROUTE | RTMGRP_IPV6_IFADDR | RTMGRP_NEIGH;
 #endif
 
-	ctx->link_fd = _open_link_socket(&snl, NETLINK_ROUTE);
-	if (ctx->link_fd == -1)
-		return -1;
-
-	if ((priv = calloc(1, sizeof(*priv))) == NULL)
-		return -1;
-
-	ctx->priv = priv;
-	memset(&snl, 0, sizeof(snl));
-	priv->route_fd = _open_link_socket(&snl, NETLINK_ROUTE);
-	if (priv->route_fd == -1)
-		return -1;
-	len = sizeof(snl);
-	if (getsockname(priv->route_fd, (struct sockaddr *)&snl, &len) == -1)
-		return -1;
-	priv->route_pid = snl.nl_pid;
-	return 0;
-}
-
-void
-if_closesockets_os(struct dhcpcd_ctx *ctx)
-{
-	struct priv *priv;
-
-	if (ctx->priv != NULL) {
-		priv = (struct priv *)ctx->priv;
-		free(priv->sndrcv_iov[0].iov_base);
-		close(priv->route_fd);
-	}
+	return _open_link_socket(&snl, 1, NETLINK_ROUTE);
 }
 
 static int
-get_netlink(struct dhcpcd_ctx *ctx, struct iovec *iov,
-    struct interface *ifp, int fd, int flags,
+err_netlink(struct nlmsghdr *nlm)
+{
+	struct nlmsgerr *err;
+	size_t len;
+
+	if (nlm->nlmsg_type != NLMSG_ERROR)
+		return 0;
+	len = nlm->nlmsg_len - sizeof(*nlm);
+	if (len < sizeof(*err)) {
+		errno = EBADMSG;
+		return -1;
+	}
+	err = (struct nlmsgerr *)NLMSG_DATA(nlm);
+	if (err->error == 0)
+		return (int)len;
+	errno = -err->error;
+	return -1;
+}
+
+static int
+get_netlink(struct dhcpcd_ctx *ctx, struct interface *ifp, int fd, int flags,
     int (*callback)(struct dhcpcd_ctx *, struct interface *, struct nlmsghdr *))
 {
-	struct msghdr msg;
-	struct sockaddr_nl nladdr;
-	ssize_t len;
+	char *buf = NULL, *nbuf;
+	ssize_t bytes;
+	size_t buflen;
 	struct nlmsghdr *nlm;
+	struct sockaddr_nl nladdr;
+	socklen_t nladdr_len;
 	int r;
-	unsigned int again;
 
-	memset(&msg, 0, sizeof(msg));
-	msg.msg_name = &nladdr;
-	msg.msg_namelen = sizeof(nladdr);
-	memset(&nladdr, 0, sizeof(nladdr));
-	msg.msg_iov = iov;
-	msg.msg_iovlen = 1;
-recv_again:
-	if ((len = recvmsg_realloc(fd, &msg, flags)) == -1)
-		return -1;
-	if (len == 0)
-		return 0;
+	buflen = 0;
+	r = -1;
+	for (;;) {
+		bytes = recv(fd, NULL, 0,
+		    flags | MSG_PEEK | MSG_DONTWAIT | MSG_TRUNC);
+		if (bytes == -1)
+			goto eexit;
+		if ((size_t)bytes == buflen) {
+			/* Support kernels older than 2.6.22 */
+			if (bytes == 0)
+				bytes = 512;
+			else
+				bytes *= 2;
+		}
+		if (buflen < (size_t)bytes) {
+			/* Alloc 1 more so we work with older kernels */
+			buflen = (size_t)bytes + 1;
+			nbuf = realloc(buf, buflen);
+			if (nbuf == NULL)
+				goto eexit;
+			buf = nbuf;
+		}
+		nladdr_len = sizeof(nladdr);
+		bytes = recvfrom(fd, buf, buflen, flags,
+		    (struct sockaddr *)&nladdr, &nladdr_len);
+		if (bytes == -1 || bytes == 0)
+			goto eexit;
 
-	/* Check sender */
-	if (msg.msg_namelen != sizeof(nladdr)) {
-		errno = EINVAL;
-		return -1;
-	}
-	/* Ignore message if it is not from kernel */
-	if (nladdr.nl_pid != 0)
-		return 0;
-
-	r = 0;
-	again = 0;	/* Appease static analysis */
-	for (nlm = iov->iov_base;
-	     nlm && NLMSG_OK(nlm, (size_t)len);
-	     nlm = NLMSG_NEXT(nlm, len))
-	{
-		again = (nlm->nlmsg_flags & NLM_F_MULTI);
-		if (nlm->nlmsg_type == NLMSG_NOOP)
+		/* Check sender */
+		if (nladdr_len != sizeof(nladdr)) {
+			errno = EINVAL;
+			goto eexit;
+		}
+		/* Ignore message if it is not from kernel */
+		if (nladdr.nl_pid != 0) {
+			r = 0;
 			continue;
-		if (nlm->nlmsg_type == NLMSG_ERROR) {
-			struct nlmsgerr *err;
+		}
 
-			if (nlm->nlmsg_len - sizeof(*nlm) < sizeof(*err)) {
-				errno = EBADMSG;
-				return -1;
+		for (nlm = (struct nlmsghdr *)(void *)buf;
+		     nlm && NLMSG_OK(nlm, (size_t)bytes);
+		     nlm = NLMSG_NEXT(nlm, bytes))
+		{
+			r = err_netlink(nlm);
+			if (r == -1)
+				goto eexit;
+			if (r)
+				continue;
+			if (callback) {
+				r = callback(ctx, ifp, nlm);
+				if (r != 0)
+					goto eexit;
 			}
-			err = (struct nlmsgerr *)NLMSG_DATA(nlm);
-			if (err->error != 0) {
-				errno = -err->error;
-				return -1;
-			}
-			break;
 		}
-		if (nlm->nlmsg_type == NLMSG_DONE) {
-			again = 0;
-			break;
-		}
-		if (callback && (r = callback(ctx, ifp, nlm)) != 0)
-			break;
 	}
 
-	if (r == 0 && again)
-		goto recv_again;
-
+eexit:
+	free(buf);
 	return r;
 }
 
@@ -393,7 +397,7 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, struct nlmsghdr *nlm)
 	size_t len;
 	struct rtmsg *rtm;
 	struct rtattr *rta;
-	unsigned int ifindex;
+	struct in_addr prefsrc;
 
 	len = nlm->nlmsg_len - sizeof(*nlm);
 	if (len < sizeof(*rtm)) {
@@ -410,6 +414,7 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, struct nlmsghdr *nlm)
 	if (rtm->rtm_scope == RT_SCOPE_HOST)
 		rt->flags |= RTF_HOST;
 
+	prefsrc.s_addr = INADDR_ANY;
 	rta = (struct rtattr *)RTM_RTA(rtm);
 	len = RTM_PAYLOAD(nlm);
 	while (RTA_OK(rta, len)) {
@@ -423,53 +428,30 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, struct nlmsghdr *nlm)
 			    sizeof(rt->gate.s_addr));
 			break;
 		case RTA_PREFSRC:
-			memcpy(&rt->src.s_addr, RTA_DATA(rta),
-			    sizeof(rt->src.s_addr));
+			memcpy(&prefsrc.s_addr, RTA_DATA(rta),
+			    sizeof(prefsrc.s_addr));
 			break;
 		case RTA_OIF:
-			ifindex = *(unsigned int *)RTA_DATA(rta);
-			rt->iface = if_findindex(ctx->ifaces, ifindex);
+			rt->iface = if_findindex(ctx->ifaces,
+			    *(unsigned int *)RTA_DATA(rta));
 			break;
 		case RTA_PRIORITY:
 			rt->metric = *(unsigned int *)RTA_DATA(rta);
 			break;
-		case RTA_METRICS:
-		{
-			struct rtattr *r2;
-			size_t l2;
-
-			l2 = rta->rta_len;
-			r2 = (struct rtattr *)RTA_DATA(rta);
-			while (RTA_OK(r2, l2)) {
-				switch (r2->rta_type) {
-				case RTAX_MTU:
-					rt->mtu = *(unsigned int *)RTA_DATA(r2);
-					break;
-				}
-				r2 = RTA_NEXT(r2, l2);
-			}
-			break;
 		}
-		}
-
 		rta = RTA_NEXT(rta, len);
 	}
 
-	inet_cidrtoaddr(rtm->rtm_dst_len, &rt->mask);
-	if (rt->iface == NULL && rt->src.s_addr != INADDR_ANY) {
+	inet_cidrtoaddr(rtm->rtm_dst_len, &rt->net);
+	if (rt->iface == NULL && prefsrc.s_addr != INADDR_ANY) {
 		struct ipv4_addr *ap;
 
 		/* For some reason the default route comes back with the
 		 * loopback interface in RTA_OIF? Lets find it by
 		 * preferred source address */
-		if ((ap = ipv4_findaddr(ctx, &rt->src)))
+		if ((ap = ipv4_findaddr(ctx, &prefsrc)))
 			rt->iface = ap->iface;
 	}
-
-	if (rt->iface == NULL) {
-		errno = ESRCH;
-		return -1;
-	}
 	return 0;
 }
 #endif
@@ -481,7 +463,6 @@ if_copyrt6(struct dhcpcd_ctx *ctx, struct rt6 *rt, struct nlmsghdr *nlm)
 	size_t len;
 	struct rtmsg *rtm;
 	struct rtattr *rta;
-	unsigned int ifindex;
 
 	len = nlm->nlmsg_len - sizeof(*nlm);
 	if (len < sizeof(*rtm)) {
@@ -497,7 +478,7 @@ if_copyrt6(struct dhcpcd_ctx *ctx, struct rt6 *rt, struct nlmsghdr *nlm)
 		rt->flags = RTF_REJECT;
 	if (rtm->rtm_scope == RT_SCOPE_HOST)
 		rt->flags |= RTF_HOST;
-	ipv6_mask(&rt->mask, rtm->rtm_dst_len);
+	ipv6_mask(&rt->net, rtm->rtm_dst_len);
 
 	rta = (struct rtattr *)RTM_RTA(rtm);
 	len = RTM_PAYLOAD(nlm);
@@ -512,41 +493,32 @@ if_copyrt6(struct dhcpcd_ctx *ctx, struct rt6 *rt, struct nlmsghdr *nlm)
 			    sizeof(rt->gate.s6_addr));
 			break;
 		case RTA_OIF:
-			ifindex = *(unsigned int *)RTA_DATA(rta);
-			rt->iface = if_findindex(ctx->ifaces, ifindex);
+			rt->iface = if_findindex(ctx->ifaces,
+			    *(unsigned int *)RTA_DATA(rta));
 			break;
 		case RTA_PRIORITY:
 			rt->metric = *(unsigned int *)RTA_DATA(rta);
 			break;
-		case RTA_METRICS:
-		{
-			struct rtattr *r2;
-			size_t l2;
-
-			l2 = rta->rta_len;
-			r2 = (struct rtattr *)RTA_DATA(rta);
-			while (RTA_OK(r2, l2)) {
-				switch (r2->rta_type) {
-				case RTAX_MTU:
-					rt->mtu = *(unsigned int *)RTA_DATA(r2);
-					break;
-				}
-				r2 = RTA_NEXT(r2, l2);
-			}
-			break;
-		}
 		}
 		rta = RTA_NEXT(rta, len);
 	}
 
-	if (rt->iface == NULL) {
-		errno = ESRCH;
-		return -1;
-	}
 	return 0;
 }
 #endif
 
+/* Work out the maximum pid size */
+static inline long long
+get_max_pid_t()
+{
+
+	if (sizeof(pid_t) == sizeof(short))		return SHRT_MAX;
+	if (sizeof(pid_t) == sizeof(int))		return INT_MAX;
+	if (sizeof(pid_t) == sizeof(long))		return LONG_MAX;
+	if (sizeof(pid_t) == sizeof(long long))		return LLONG_MAX;
+	abort();
+}
+
 static int
 link_route(struct dhcpcd_ctx *ctx, __unused struct interface *ifp,
     struct nlmsghdr *nlm)
@@ -554,7 +526,6 @@ link_route(struct dhcpcd_ctx *ctx, __unused struct interface *ifp,
 	size_t len;
 	struct rtmsg *rtm;
 	int cmd;
-	struct priv *priv;
 #ifdef INET
 	struct rt rt;
 #endif
@@ -578,17 +549,19 @@ link_route(struct dhcpcd_ctx *ctx, __unused struct interface *ifp,
 		return -1;
 	}
 
-	/* Ignore messages we sent. */
-	priv = (struct priv *)ctx->priv;
-	if (nlm->nlmsg_pid == priv->route_pid)
-		return 0;
+	/* Ignore messages generated by us.
+	 * For some reason we get messages generated by us
+	 * with a very large value in nlmsg_pid that seems to be
+	 * sequentially changing. Is there a better test for this? */
+	if (nlm->nlmsg_pid > get_max_pid_t())
+		return 1;
 
 	rtm = NLMSG_DATA(nlm);
 	switch (rtm->rtm_family) {
 #ifdef INET
 	case AF_INET:
 		if (if_copyrt(ctx, &rt, nlm) == 0)
-			ipv4_handlert(ctx, cmd, &rt, 0);
+			ipv4_handlert(ctx, cmd, &rt);
 		break;
 #endif
 #ifdef INET6
@@ -608,9 +581,8 @@ link_addr(struct dhcpcd_ctx *ctx, struct interface *ifp, struct nlmsghdr *nlm)
 	size_t len;
 	struct rtattr *rta;
 	struct ifaddrmsg *ifa;
-	struct priv *priv;
 #ifdef INET
-	struct in_addr addr, net, brd;
+	struct in_addr addr, net, dest;
 #endif
 #ifdef INET6
 	struct in6_addr addr6;
@@ -624,12 +596,6 @@ link_addr(struct dhcpcd_ctx *ctx, struct interface *ifp, struct nlmsghdr *nlm)
 		errno = EBADMSG;
 		return -1;
 	}
-
-	/* Ignore messages we sent. */
-	priv = (struct priv*)ctx->priv;
-	if (nlm->nlmsg_pid == priv->route_pid)
-		return 0;
-
 	ifa = NLMSG_DATA(nlm);
 	if ((ifp = if_findindex(ctx->ifaces, ifa->ifa_index)) == NULL) {
 		/* We don't know about the interface the address is for
@@ -641,29 +607,26 @@ link_addr(struct dhcpcd_ctx *ctx, struct interface *ifp, struct nlmsghdr *nlm)
 	switch (ifa->ifa_family) {
 #ifdef INET
 	case AF_INET:
-		addr.s_addr = brd.s_addr = INADDR_ANY;
+		addr.s_addr = dest.s_addr = INADDR_ANY;
+		dest.s_addr = INADDR_ANY;
 		inet_cidrtoaddr(ifa->ifa_prefixlen, &net);
 		while (RTA_OK(rta, len)) {
 			switch (rta->rta_type) {
 			case IFA_ADDRESS:
 				if (ifp->flags & IFF_POINTOPOINT) {
-					memcpy(&brd.s_addr, RTA_DATA(rta),
-					    sizeof(brd.s_addr));
+					memcpy(&dest.s_addr, RTA_DATA(rta),
+					       sizeof(addr.s_addr));
 				}
 				break;
-			case IFA_BROADCAST:
-				memcpy(&brd.s_addr, RTA_DATA(rta),
-				    sizeof(brd.s_addr));
-				break;
 			case IFA_LOCAL:
 				memcpy(&addr.s_addr, RTA_DATA(rta),
-				    sizeof(addr.s_addr));
+				       sizeof(addr.s_addr));
 				break;
 			}
 			rta = RTA_NEXT(rta, len);
 		}
 		ipv4_handleifa(ctx, nlm->nlmsg_type, NULL, ifp->name,
-		    &addr, &net, &brd, ifa->ifa_flags);
+		    &addr, &net, &dest, ifa->ifa_flags);
 		break;
 #endif
 #ifdef INET6
@@ -799,7 +762,7 @@ link_netlink(struct dhcpcd_ctx *ctx, struct interface *ifp,
 	ifi = NLMSG_DATA(nlm);
 	if (ifi->ifi_flags & IFF_LOOPBACK)
 		return 0;
-	rta = (void *)((char *)ifi + NLMSG_ALIGN(sizeof(*ifi)));
+	rta = (struct rtattr *)(void *)((char *)ifi +NLMSG_ALIGN(sizeof(*ifi)));
 	len = NLMSG_PAYLOAD(nlm, sizeof(*ifi));
 	*ifn = '\0';
 	hwaddr = NULL;
@@ -865,10 +828,10 @@ link_netlink(struct dhcpcd_ctx *ctx, struct interface *ifp,
 }
 
 int
-if_handlelink(struct dhcpcd_ctx *ctx)
+if_managelink(struct dhcpcd_ctx *ctx)
 {
 
-	return get_netlink(ctx, ctx->iov, NULL,
+	return get_netlink(ctx, NULL,
 	    ctx->link_fd, MSG_DONTWAIT, &link_netlink);
 }
 
@@ -879,43 +842,30 @@ send_netlink(struct dhcpcd_ctx *ctx, struct interface *ifp,
 {
 	int s, r;
 	struct sockaddr_nl snl;
-	struct iovec iov[1];
+	struct iovec iov;
 	struct msghdr msg;
+	static unsigned int seq;
 
 	memset(&snl, 0, sizeof(snl));
-	snl.nl_family = AF_NETLINK;
-
-	if (protocol == NETLINK_ROUTE) {
-		struct priv *priv;
-
-		priv = (struct priv *)ctx->priv;
-		s = priv->route_fd;
-	} else {
-		if ((s = _open_link_socket(&snl, protocol)) == -1)
-			return -1;
-	}
-
+	if ((s = _open_link_socket(&snl, 0, protocol)) == -1)
+		return -1;
+	memset(&iov, 0, sizeof(iov));
+	iov.iov_base = hdr;
+	iov.iov_len = hdr->nlmsg_len;
 	memset(&msg, 0, sizeof(msg));
 	msg.msg_name = &snl;
 	msg.msg_namelen = sizeof(snl);
-	memset(&iov, 0, sizeof(iov));
-	iov[0].iov_base = hdr;
-	iov[0].iov_len = hdr->nlmsg_len;
-	msg.msg_iov = iov;
+	msg.msg_iov = &iov;
 	msg.msg_iovlen = 1;
 	/* Request a reply */
 	hdr->nlmsg_flags |= NLM_F_ACK;
-	hdr->nlmsg_seq = (uint32_t)++ctx->seq;
-	if (sendmsg(s, &msg, 0) != -1) {
-		struct priv *priv;
+	hdr->nlmsg_seq = ++seq;
 
-		priv = (struct priv *)ctx->priv;
-		ctx->sseq = ctx->seq;
-		r = get_netlink(ctx, priv->sndrcv_iov, ifp, s, 0, callback);
-	} else
+	if (sendmsg(s, &msg, 0) != -1)
+		r = get_netlink(ctx, ifp, s, 0, callback);
+	else
 		r = -1;
-	if (protocol != NETLINK_ROUTE)
-		close(s);
+	close(s);
 	return r;
 }
 
@@ -965,33 +915,13 @@ add_attr_32(struct nlmsghdr *n, unsigned short maxlen, unsigned short type,
 	return 0;
 }
 
-static int
-rta_add_attr_32(struct rtattr *rta, unsigned short maxlen,
-    unsigned short type, uint32_t data)
-{
-	unsigned short len = RTA_LENGTH(sizeof(data));
-	struct rtattr *subrta;
-
-	if (RTA_ALIGN(rta->rta_len) + len > maxlen) {
-		errno = ENOBUFS;
-		return -1;
-	}
-
-	subrta = (void *)((char*)rta + RTA_ALIGN(rta->rta_len));
-	subrta->rta_type = type;
-	subrta->rta_len = len;
-	memcpy(RTA_DATA(subrta), &data, sizeof(data));
-	rta->rta_len = (unsigned short)(NLMSG_ALIGN(rta->rta_len) + len);
-	return 0;
-}
-
 #ifdef HAVE_NL80211_H
 static struct nlattr *
 nla_next(struct nlattr *nla, size_t *rem)
 {
 
-	*rem -= (size_t)NLA_ALIGN(nla->nla_len);
-	return (void *)((char *)nla + NLA_ALIGN(nla->nla_len));
+	*rem -= NLA_ALIGN(nla->nla_len);
+	return (struct nlattr *)(void *)((char *)nla + NLA_ALIGN(nla->nla_len));
 }
 
 #define NLA_TYPE(nla) ((nla)->nla_type & NLA_TYPE_MASK)
@@ -1002,9 +932,7 @@ nla_next(struct nlattr *nla, size_t *rem)
 	(nla)->nla_len <= rem)
 #define NLA_DATA(nla) ((char *)(nla) + NLA_HDRLEN)
 #define NLA_FOR_EACH_ATTR(pos, head, len, rem) \
-	for (pos = head, rem = len; \
-	     NLA_OK(pos, rem); \
-	     pos = nla_next(pos, &(rem)))
+	for (pos = head, rem = len; NLA_OK(pos, rem); pos = nla_next(pos, &(rem)))
 
 struct nlmg
 {
@@ -1067,7 +995,7 @@ gnl_parse(struct nlmsghdr *nlm, struct nlattr *tb[], int maxtype)
 
 	memset(tb, 0, sizeof(*tb) * ((unsigned int)maxtype + 1));
 	ghdr = NLMSG_DATA(nlm);
-	head = (void *)((char *)ghdr + GENL_HDRLEN);
+	head = (struct nlattr *)(void *)((char *) ghdr + GENL_HDRLEN);
 	len = nlm->nlmsg_len - GENL_HDRLEN - NLMSG_HDRLEN;
 	NLA_FOR_EACH_ATTR(nla, head, len, rem) {
 		type = NLA_TYPE(nla);
@@ -1130,7 +1058,7 @@ _if_getssid(__unused struct dhcpcd_ctx *ctx, struct interface *ifp,
 	}
 
 	ifp->ssid_len = NLA_LEN(tb[NL80211_ATTR_SSID]);
-	if (ifp->ssid_len > IF_SSIDLEN) {
+	if (ifp->ssid_len > sizeof(ifp->ssid)) {
 		errno = ENOBUFS;
 		ifp->ssid_len = 0;
 		return -1;
@@ -1138,6 +1066,7 @@ _if_getssid(__unused struct dhcpcd_ctx *ctx, struct interface *ifp,
 	memcpy(ifp->ssid, NLA_DATA(tb[NL80211_ATTR_SSID]), ifp->ssid_len);
 
 out:
+	ifp->ssid[ifp->ssid_len] = '\0';
 	return (int)ifp->ssid_len;
 }
 
@@ -1171,16 +1100,10 @@ if_getssid(struct interface *ifp)
 	r = if_getssid_wext(ifp->name, ifp->ssid);
 	if (r != -1)
 		ifp->ssid_len = (unsigned int)r;
-	else {
 #ifdef HAVE_NL80211_H
+	else if (r == -1)
 		r = if_getssid_nl80211(ifp);
-		if (r == -1)
-			ifp->ssid_len = 0;
-#else
-		ifp->ssid_len = 0;
 #endif
-	}
-	ifp->ssid[ifp->ssid_len] = '\0';
 	return r;
 }
 
@@ -1201,15 +1124,8 @@ struct nlmr
 #ifdef INET
 const char *if_pfname = "Packet Socket";
 
-void
-if_closeraw(__unused struct interface *ifp, int fd)
-{
-
-	close(fd);
-}
-
 int
-if_openraw(struct interface *ifp, uint16_t protocol)
+if_openrawsocket(struct interface *ifp, uint16_t protocol)
 {
 	int s;
 	union sockunion {
@@ -1222,19 +1138,36 @@ if_openraw(struct interface *ifp, uint16_t protocol)
 	int n;
 #endif
 
-#define SF	SOCK_CLOEXEC | SOCK_NONBLOCK
-	if ((s = xsocket(PF_PACKET, SOCK_DGRAM | SF, htons(protocol))) == -1)
+#ifdef SOCK_CLOEXEC
+	if ((s = socket(PF_PACKET, SOCK_DGRAM | SOCK_CLOEXEC | SOCK_NONBLOCK,
+	    htons(protocol))) == -1)
 		return -1;
-#undef SF
+#else
+	int flags;
 
-	/* Install the filter. */
+	if ((s = socket(PF_PACKET, SOCK_DGRAM, htons(protocol))) == -1)
+		return -1;
+	if ((flags = fcntl(s, F_GETFD, 0)) == -1 ||
+	    fcntl(s, F_SETFD, flags | FD_CLOEXEC) == -1)
+	{
+		close(s);
+	        return -1;
+	}
+	if ((flags = fcntl(s, F_GETFL, 0)) == -1 ||
+	    fcntl(s, F_SETFL, flags | O_NONBLOCK) == -1)
+	{
+		close(s);
+	        return -1;
+	}
+#endif
+	/* Install the DHCP filter */
 	memset(&pf, 0, sizeof(pf));
 	if (protocol == ETHERTYPE_ARP) {
 		pf.filter = UNCONST(arp_bpf_filter);
 		pf.len = arp_bpf_filter_len;
 	} else {
-		pf.filter = UNCONST(bootp_bpf_filter);
-		pf.len = bootp_bpf_filter_len;
+		pf.filter = UNCONST(dhcp_bpf_filter);
+		pf.len = dhcp_bpf_filter_len;
 	}
 	if (setsockopt(s, SOL_SOCKET, SO_ATTACH_FILTER, &pf, sizeof(pf)) != 0)
 		goto eexit;
@@ -1260,14 +1193,16 @@ eexit:
 }
 
 ssize_t
-if_sendraw(const struct interface *ifp, int fd, uint16_t protocol,
+if_sendrawpacket(const struct interface *ifp, uint16_t protocol,
     const void *data, size_t len)
 {
+	const struct dhcp_state *state;
 	union sockunion {
 		struct sockaddr sa;
 		struct sockaddr_ll sll;
 		struct sockaddr_storage ss;
 	} su;
+	int fd;
 
 	memset(&su, 0, sizeof(su));
 	su.sll.sll_family = AF_PACKET;
@@ -1275,22 +1210,22 @@ if_sendraw(const struct interface *ifp, int fd, uint16_t protocol,
 	su.sll.sll_ifindex = (int)ifp->index;
 	su.sll.sll_hatype = htons(ifp->family);
 	su.sll.sll_halen = (unsigned char)ifp->hwlen;
-	if (ifp->family == ARPHRD_INFINIBAND) {
-		/* sockaddr_ll is not big enough for IPoIB which is why
-		 * sockaddr_storage is included in the union.
-		 * Ugly as sin, but it works. */
-		/* coverity[buffer_size] */
-		/* coverity[overrun-buffer-arg] */
+	if (ifp->family == ARPHRD_INFINIBAND)
 		memcpy(&su.sll.sll_addr,
 		    &ipv4_bcast_addr, sizeof(ipv4_bcast_addr));
-	} else
+	else
 		memset(&su.sll.sll_addr, 0xff, ifp->hwlen);
+	state = D_CSTATE(ifp);
+	if (protocol == ETHERTYPE_ARP)
+		fd = state->arp_fd;
+	else
+		fd = state->raw_fd;
 
 	return sendto(fd, data, len, 0, &su.sa, sizeof(su.sll));
 }
 
 ssize_t
-if_readraw(__unused struct interface *ifp, int fd,
+if_readrawpacket(struct interface *ifp, uint16_t protocol,
     void *data, size_t len, int *flags)
 {
 	struct iovec iov = {
@@ -1301,6 +1236,7 @@ if_readraw(__unused struct interface *ifp, int fd,
 		.msg_iov = &iov,
 		.msg_iovlen = 1,
 	};
+	struct dhcp_state *state;
 #ifdef PACKET_AUXDATA
 	unsigned char cmsgbuf[CMSG_LEN(sizeof(struct tpacket_auxdata))];
 	struct cmsghdr *cmsg;
@@ -1308,16 +1244,22 @@ if_readraw(__unused struct interface *ifp, int fd,
 #endif
 
 	ssize_t bytes;
+	int fd = -1;
 
 #ifdef PACKET_AUXDATA
 	msg.msg_control = cmsgbuf;
 	msg.msg_controllen = sizeof(cmsgbuf);
 #endif
 
+	state = D_STATE(ifp);
+	if (protocol == ETHERTYPE_ARP)
+		fd = state->arp_fd;
+	else
+		fd = state->raw_fd;
 	bytes = recvmsg(fd, &msg, 0);
 	if (bytes == -1)
 		return -1;
-	*flags = RAW_EOF; /* We only ever read one packet. */
+	*flags = RAW_EOF; /* We only ever read one packet */
 	if (bytes) {
 #ifdef PACKET_AUXDATA
 		for (cmsg = CMSG_FIRSTHDR(&msg);
@@ -1337,7 +1279,9 @@ if_readraw(__unused struct interface *ifp, int fd,
 }
 
 int
-if_address(unsigned char cmd, const struct ipv4_addr *addr)
+if_address(const struct interface *iface,
+    const struct in_addr *address, const struct in_addr *netmask,
+    const struct in_addr *broadcast, int action)
 {
 	struct nlma nlm;
 	int retval = 0;
@@ -1345,26 +1289,24 @@ if_address(unsigned char cmd, const struct ipv4_addr *addr)
 	memset(&nlm, 0, sizeof(nlm));
 	nlm.hdr.nlmsg_len = NLMSG_LENGTH(sizeof(struct ifaddrmsg));
 	nlm.hdr.nlmsg_flags = NLM_F_REQUEST;
-	nlm.hdr.nlmsg_type = cmd;
-	if (cmd == RTM_NEWADDR)
+	if (action >= 0) {
 		nlm.hdr.nlmsg_flags |= NLM_F_CREATE | NLM_F_REPLACE;
-	nlm.ifa.ifa_index = addr->iface->index;
+		nlm.hdr.nlmsg_type = RTM_NEWADDR;
+	} else
+		nlm.hdr.nlmsg_type = RTM_DELADDR;
+	nlm.ifa.ifa_index = iface->index;
 	nlm.ifa.ifa_family = AF_INET;
-	nlm.ifa.ifa_prefixlen = inet_ntocidr(addr->mask);
-#if 0
+	nlm.ifa.ifa_prefixlen = inet_ntocidr(*netmask);
 	/* This creates the aliased interface */
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LABEL,
-	    addr->iface->alias,
-	    (unsigned short)(strlen(addr->iface->alias) + 1));
-#endif
+	    iface->alias, (unsigned short)(strlen(iface->alias) + 1));
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LOCAL,
-	    &addr->addr.s_addr, sizeof(addr->addr.s_addr));
-	if (cmd == RTM_NEWADDR)
+	    &address->s_addr, sizeof(address->s_addr));
+	if (action >= 0 && broadcast)
 		add_attr_l(&nlm.hdr, sizeof(nlm), IFA_BROADCAST,
-		    &addr->brd.s_addr, sizeof(addr->brd.s_addr));
+		    &broadcast->s_addr, sizeof(broadcast->s_addr));
 
-	if (send_netlink(addr->iface->ctx, NULL,
-	    NETLINK_ROUTE, &nlm.hdr, NULL) == -1)
+	if (send_netlink(iface->ctx, NULL, NETLINK_ROUTE, &nlm.hdr, NULL) == -1)
 		retval = -1;
 	return retval;
 }
@@ -1373,6 +1315,8 @@ int
 if_route(unsigned char cmd, const struct rt *rt)
 {
 	struct nlmr nlm;
+	int retval = 0;
+	struct dhcp_state *state;
 
 	memset(&nlm, 0, sizeof(nlm));
 	nlm.hdr.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
@@ -1393,11 +1337,14 @@ if_route(unsigned char cmd, const struct rt *rt)
 	nlm.rt.rtm_family = AF_INET;
 	nlm.rt.rtm_table = RT_TABLE_MAIN;
 
-	if (cmd == RTM_DELETE) {
+	state = D_STATE(rt->iface);
+	if (cmd == RTM_DELETE)
 		nlm.rt.rtm_scope = RT_SCOPE_NOWHERE;
-	} else {
-		/* Subnet routes are RTPROT_KERNEL otherwise RTPROT_BOOT */
-		if (rt->gate.s_addr == ntohl(INADDR_ANY))
+	else {
+		/* We only change route metrics for kernel routes */
+		if (rt->dest.s_addr ==
+		    (state->addr.s_addr & state->net.s_addr) &&
+		    rt->net.s_addr == state->net.s_addr)
 			nlm.rt.rtm_protocol = RTPROT_KERNEL;
 		else
 			nlm.rt.rtm_protocol = RTPROT_BOOT;
@@ -1405,46 +1352,35 @@ if_route(unsigned char cmd, const struct rt *rt)
 			nlm.rt.rtm_scope = RT_SCOPE_HOST;
 		else if (rt->gate.s_addr == INADDR_ANY ||
 		    (rt->gate.s_addr == rt->dest.s_addr &&
-			rt->mask.s_addr == INADDR_BROADCAST))
+			rt->net.s_addr == INADDR_BROADCAST))
 			nlm.rt.rtm_scope = RT_SCOPE_LINK;
 		else
 			nlm.rt.rtm_scope = RT_SCOPE_UNIVERSE;
 		nlm.rt.rtm_type = RTN_UNICAST;
 	}
 
-	nlm.rt.rtm_dst_len = inet_ntocidr(rt->mask);
+	nlm.rt.rtm_dst_len = inet_ntocidr(rt->net);
 	add_attr_l(&nlm.hdr, sizeof(nlm), RTA_DST,
 	    &rt->dest.s_addr, sizeof(rt->dest.s_addr));
-	if (cmd == RTM_ADD || cmd == RTM_CHANGE) {
-		if (rt->gate.s_addr != htonl(INADDR_ANY))
-			add_attr_l(&nlm.hdr, sizeof(nlm), RTA_GATEWAY,
-			    &rt->gate.s_addr, sizeof(rt->gate.s_addr));
-		if (rt->src.s_addr != ntohl(INADDR_ANY)) {
-			add_attr_l(&nlm.hdr, sizeof(nlm), RTA_PREFSRC,
-			    &rt->src.s_addr, sizeof(rt->src.s_addr));
-		}
-		if (rt->mtu) {
-			char metricsbuf[32];
-			struct rtattr *metrics = (void *)metricsbuf;
-
-			metrics->rta_type = RTA_METRICS;
-			metrics->rta_len = RTA_LENGTH(0);
-			rta_add_attr_32(metrics, sizeof(metricsbuf),
-			    RTAX_MTU, rt->mtu);
-			add_attr_l(&nlm.hdr, sizeof(nlm), RTA_METRICS,
-			    RTA_DATA(metrics),
-			    (unsigned short)RTA_PAYLOAD(metrics));
-		}
+	if (nlm.rt.rtm_protocol == RTPROT_KERNEL) {
+		add_attr_l(&nlm.hdr, sizeof(nlm), RTA_PREFSRC,
+		    &state->addr.s_addr, sizeof(state->addr.s_addr));
 	}
+	/* If a host route then don't add the gateway */
+	if ((cmd == RTM_ADD || cmd == RTM_CHANGE) &&
+	    rt->net.s_addr != INADDR_BROADCAST)
+		add_attr_l(&nlm.hdr, sizeof(nlm), RTA_GATEWAY,
+		    &rt->gate.s_addr, sizeof(rt->gate.s_addr));
 
 	if (rt->gate.s_addr != htonl(INADDR_LOOPBACK))
 		add_attr_32(&nlm.hdr, sizeof(nlm), RTA_OIF, rt->iface->index);
-
 	if (rt->metric)
 		add_attr_32(&nlm.hdr, sizeof(nlm), RTA_PRIORITY, rt->metric);
 
-	return send_netlink(rt->iface->ctx, NULL,
-	    NETLINK_ROUTE, &nlm.hdr, NULL);
+	if (send_netlink(rt->iface->ctx, NULL,
+	    NETLINK_ROUTE, &nlm.hdr, NULL) == -1)
+		retval = -1;
+	return retval;
 }
 
 static int
@@ -1454,30 +1390,33 @@ _if_initrt(struct dhcpcd_ctx *ctx, __unused struct interface *ifp,
 	struct rt rt;
 
 	if (if_copyrt(ctx, &rt, nlm) == 0)
-		ipv4_handlert(ctx, RTM_ADD, &rt, 1);
+		ipv4_handlert(ctx, RTM_ADD, &rt);
 	return 0;
 }
 
 int
-if_initrt(struct dhcpcd_ctx *ctx)
+if_initrt(struct interface *ifp)
 {
 	struct nlmr nlm;
 
-	ipv4_freerts(ctx->ipv4_kroutes);
+	ipv4_freerts(ifp->ctx->ipv4_kroutes);
 
 	memset(&nlm, 0, sizeof(nlm));
 	nlm.hdr.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
 	nlm.hdr.nlmsg_type = RTM_GETROUTE;
 	nlm.hdr.nlmsg_flags = NLM_F_REQUEST | NLM_F_MATCH;
+	nlm.hdr.nlmsg_flags |= NLM_F_REQUEST;
 	nlm.rt.rtm_family = AF_INET;
 	nlm.rt.rtm_table = RT_TABLE_MAIN;
+	add_attr_32(&nlm.hdr, sizeof(nlm), RTA_OIF, ifp->index);
 
-	return send_netlink(ctx, NULL, NETLINK_ROUTE, &nlm.hdr, &_if_initrt);
+	return send_netlink(ifp->ctx, ifp,
+	    NETLINK_ROUTE, &nlm.hdr, &_if_initrt);
 }
 
 int
-if_addrflags(__unused const struct interface *ifp,
-__unused const struct in_addr *addr, __unused const char *alias)
+if_addrflags(__unused const struct in_addr *addr,
+    __unused const struct interface *ifp)
 {
 
 	/* Linux has no support for IPv4 address flags */
@@ -1487,10 +1426,11 @@ __unused const struct in_addr *addr, __unused const char *alias)
 
 #ifdef INET6
 int
-if_address6(unsigned char cmd, const struct ipv6_addr *ia)
+if_address6(const struct ipv6_addr *ap, int action)
 {
 	struct nlma nlm;
 	struct ifa_cacheinfo cinfo;
+	int retval = 0;
 /* IFA_FLAGS is not a define, but is was added at the same time
  * IFA_F_NOPREFIXROUTE was do use that. */
 #if defined(IFA_F_NOPREFIXROUTE) || defined(IFA_F_MANAGETEMPADDR)
@@ -1500,45 +1440,44 @@ if_address6(unsigned char cmd, const struct ipv6_addr *ia)
 	memset(&nlm, 0, sizeof(nlm));
 	nlm.hdr.nlmsg_len = NLMSG_LENGTH(sizeof(struct ifaddrmsg));
 	nlm.hdr.nlmsg_flags = NLM_F_REQUEST;
-	nlm.hdr.nlmsg_type = cmd;
-	if (cmd == RTM_NEWADDR)
+	if (action >= 0) {
 		nlm.hdr.nlmsg_flags |= NLM_F_CREATE | NLM_F_REPLACE;
-	nlm.ifa.ifa_index = ia->iface->index;
+		nlm.hdr.nlmsg_type = RTM_NEWADDR;
+	} else
+		nlm.hdr.nlmsg_type = RTM_DELADDR;
+	nlm.ifa.ifa_index = ap->iface->index;
 	nlm.ifa.ifa_family = AF_INET6;
-#ifdef IPV6_MANAGETEMPADDR
-	if (ia->flags & IPV6_AF_TEMPORARY) {
-		/* Currently the kernel filters out these flags */
+	if (ap->addr_flags & IFA_F_TEMPORARY) {
 #ifdef IFA_F_NOPREFIXROUTE
 		flags |= IFA_F_TEMPORARY;
 #else
 		nlm.ifa.ifa_flags |= IFA_F_TEMPORARY;
 #endif
 	}
-#elif IFA_F_MANAGETEMPADDR
-	if (ia->flags & IPV6_AF_AUTOCONF)
+#ifdef IFA_F_MANAGETEMPADDR
+	else if (ap->flags & IPV6_AF_AUTOCONF &&
+	    ip6_use_tempaddr(ap->iface->name))
 		flags |= IFA_F_MANAGETEMPADDR;
 #endif
 
 	/* Add as /128 if no IFA_F_NOPREFIXROUTE ? */
-	nlm.ifa.ifa_prefixlen = ia->prefix_len;
-#if 0
+	nlm.ifa.ifa_prefixlen = ap->prefix_len;
 	/* This creates the aliased interface */
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LABEL,
-	    ia->iface->alias, (unsigned short)(strlen(ia->iface->alias) + 1));
-#endif
+	    ap->iface->alias, (unsigned short)(strlen(ap->iface->alias) + 1));
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LOCAL,
-	    &ia->addr.s6_addr, sizeof(ia->addr.s6_addr));
+	    &ap->addr.s6_addr, sizeof(ap->addr.s6_addr));
 
-	if (cmd == RTM_NEWADDR) {
+	if (action >= 0) {
 		memset(&cinfo, 0, sizeof(cinfo));
-		cinfo.ifa_prefered = ia->prefix_pltime;
-		cinfo.ifa_valid = ia->prefix_vltime;
+		cinfo.ifa_prefered = ap->prefix_pltime;
+		cinfo.ifa_valid = ap->prefix_vltime;
 		add_attr_l(&nlm.hdr, sizeof(nlm), IFA_CACHEINFO,
 		    &cinfo, sizeof(cinfo));
 	}
 
 #ifdef IFA_F_NOPREFIXROUTE
-	if (!IN6_IS_ADDR_LINKLOCAL(&ia->addr))
+	if (!IN6_IS_ADDR_LINKLOCAL(&ap->addr))
 		flags |= IFA_F_NOPREFIXROUTE;
 #endif
 #if defined(IFA_F_NOPREFIXROUTE) || defined(IFA_F_MANAGETEMPADDR)
@@ -1546,14 +1485,38 @@ if_address6(unsigned char cmd, const struct ipv6_addr *ia)
 		add_attr_32(&nlm.hdr, sizeof(nlm), IFA_FLAGS, flags);
 #endif
 
-	return send_netlink(ia->iface->ctx, NULL,
-	    NETLINK_ROUTE, &nlm.hdr, NULL);
+	if (send_netlink(ap->iface->ctx, NULL, NETLINK_ROUTE, &nlm.hdr,
+	    NULL) == -1)
+		retval = -1;
+	return retval;
+}
+
+static int
+rta_add_attr_32(struct rtattr *rta, unsigned short maxlen,
+    unsigned short type, uint32_t data)
+{
+	unsigned short len = RTA_LENGTH(sizeof(data));
+	struct rtattr *subrta;
+
+	if (RTA_ALIGN(rta->rta_len) + len > maxlen) {
+		errno = ENOBUFS;
+		return -1;
+	}
+
+	subrta = (struct rtattr*)(void *)
+	    (((char*)rta) + RTA_ALIGN(rta->rta_len));
+	subrta->rta_type = type;
+	subrta->rta_len = len;
+	memcpy(RTA_DATA(subrta), &data, sizeof(data));
+	rta->rta_len = (unsigned short)(NLMSG_ALIGN(rta->rta_len) + len);
+	return 0;
 }
 
 int
 if_route6(unsigned char cmd, const struct rt6 *rt)
 {
 	struct nlmr nlm;
+	int retval = 0;
 
 	memset(&nlm, 0, sizeof(nlm));
 	nlm.hdr.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
@@ -1591,7 +1554,7 @@ if_route6(unsigned char cmd, const struct rt6 *rt)
 			nlm.rt.rtm_type = RTN_UNICAST;
 	}
 
-	nlm.rt.rtm_dst_len = ipv6_prefixlen(&rt->mask);
+	nlm.rt.rtm_dst_len = ipv6_prefixlen(&rt->net);
 	add_attr_l(&nlm.hdr, sizeof(nlm), RTA_DST,
 	    &rt->dest.s6_addr, sizeof(rt->dest.s6_addr));
 
@@ -1599,34 +1562,27 @@ if_route6(unsigned char cmd, const struct rt6 *rt)
 		add_attr_l(&nlm.hdr, sizeof(nlm), RTA_GATEWAY,
 		    &rt->gate.s6_addr, sizeof(rt->gate.s6_addr));
 
-	add_attr_32(&nlm.hdr, sizeof(nlm), RTA_OIF, rt->iface->index);
-	if (rt->metric)
-		add_attr_32(&nlm.hdr, sizeof(nlm), RTA_PRIORITY, rt->metric);
+	if (!(rt->flags & RTF_REJECT)) {
+		add_attr_32(&nlm.hdr, sizeof(nlm), RTA_OIF, rt->iface->index);
+		if (rt->metric)
+			add_attr_32(&nlm.hdr, sizeof(nlm),
+			    RTA_PRIORITY, rt->metric);
+	}
+	if (cmd == RTM_ADD && rt->mtu) {
+		char metricsbuf[32];
+		struct rtattr *metrics = (void *)metricsbuf;
 
-	if (cmd == RTM_ADD || cmd == RTM_CHANGE) {
-#if 0
-		/* XXX This fails to work, why? */
-		if (!IN6_IS_ADDR_UNSPECIFIED(&rt->src)) {
-			add_attr_l(&nlm.hdr, sizeof(nlm), RTA_PREFSRC,
-			    &rt->src.s6_addr, sizeof(rt->src.s6_addr));
-		}
-#endif
-		if (rt->mtu) {
-			char metricsbuf[32];
-			struct rtattr *metrics = (void *)metricsbuf;
-
-			metrics->rta_type = RTA_METRICS;
-			metrics->rta_len = RTA_LENGTH(0);
-			rta_add_attr_32(metrics, sizeof(metricsbuf),
-			    RTAX_MTU, rt->mtu);
-			add_attr_l(&nlm.hdr, sizeof(nlm), RTA_METRICS,
-			    RTA_DATA(metrics),
-			    (unsigned short)RTA_PAYLOAD(metrics));
-		}
+		metrics->rta_type = RTA_METRICS;
+		metrics->rta_len = RTA_LENGTH(0);
+		rta_add_attr_32(metrics, sizeof(metricsbuf), RTAX_MTU, rt->mtu);
+		add_attr_l(&nlm.hdr, sizeof(nlm), RTA_METRICS,
+		    RTA_DATA(metrics), (unsigned short)RTA_PAYLOAD(metrics));
 	}
 
-	return send_netlink(rt->iface->ctx, NULL,
-	    NETLINK_ROUTE, &nlm.hdr, NULL);
+	if (send_netlink(rt->iface->ctx, NULL,
+	    NETLINK_ROUTE, &nlm.hdr, NULL) == -1)
+		retval = -1;
+	return retval;
 }
 
 static int
@@ -1641,11 +1597,11 @@ _if_initrt6(struct dhcpcd_ctx *ctx, __unused struct interface *ifp,
 }
 
 int
-if_initrt6(struct dhcpcd_ctx *ctx)
+if_initrt6(struct interface *ifp)
 {
 	struct nlmr nlm;
 
-	ipv6_freerts(&ctx->ipv6->kroutes);
+	ipv6_freerts(&ifp->ctx->ipv6->kroutes);
 
 	memset(&nlm, 0, sizeof(nlm));
 	nlm.hdr.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
@@ -1654,13 +1610,14 @@ if_initrt6(struct dhcpcd_ctx *ctx)
 	nlm.hdr.nlmsg_flags |= NLM_F_REQUEST;
 	nlm.rt.rtm_family = AF_INET6;
 	nlm.rt.rtm_table = RT_TABLE_MAIN;
+	add_attr_32(&nlm.hdr, sizeof(nlm), RTA_OIF, ifp->index);
 
-	return send_netlink(ctx, NULL, NETLINK_ROUTE, &nlm.hdr, &_if_initrt6);
+	return send_netlink(ifp->ctx, ifp,
+	    NETLINK_ROUTE, &nlm.hdr, &_if_initrt6);
 }
 
 int
-if_addrflags6(const struct interface *ifp, const struct in6_addr *addr,
-    __unused const char *alias)
+if_addrflags6(const struct in6_addr *addr, const struct interface *ifp)
 {
 	FILE *fp;
 	char *p, ifaddress[33], address[33], name[IF_NAMESIZE + 1];
@@ -1682,7 +1639,7 @@ if_addrflags6(const struct interface *ifp, const struct in6_addr *addr,
 	{
 		if (strlen(address) != 32) {
 			fclose(fp);
-			errno = EINVAL;
+			errno = ENOTSUP;
 			return -1;
 		}
 		if (strcmp(name, ifp->name) == 0 &&
@@ -1740,9 +1697,8 @@ add_attr_nest_end(struct nlmsghdr *n, struct rtattr *nest)
 }
 
 static int
-if_disable_autolinklocal(struct dhcpcd_ctx *ctx, unsigned int ifindex)
+if_disable_autolinklocal(struct dhcpcd_ctx *ctx, int ifindex)
 {
-#ifdef HAVE_IN6_ADDR_GEN_MODE_NONE
 	struct nlml nlm;
 	struct rtattr *afs, *afs6;
 
@@ -1751,7 +1707,7 @@ if_disable_autolinklocal(struct dhcpcd_ctx *ctx, unsigned int ifindex)
 	nlm.hdr.nlmsg_type = RTM_NEWLINK;
 	nlm.hdr.nlmsg_flags = NLM_F_REQUEST;
 	nlm.i.ifi_family = AF_INET6;
-	nlm.i.ifi_index = (int)ifindex;
+	nlm.i.ifi_index = ifindex;
 	afs = add_attr_nest(&nlm.hdr, sizeof(nlm), IFLA_AF_SPEC);
 	afs6 = add_attr_nest(&nlm.hdr, sizeof(nlm), AF_INET6);
 	add_attr_8(&nlm.hdr, sizeof(nlm), IFLA_INET6_ADDR_GEN_MODE,
@@ -1760,12 +1716,6 @@ if_disable_autolinklocal(struct dhcpcd_ctx *ctx, unsigned int ifindex)
 	add_attr_nest_end(&nlm.hdr, afs);
 
 	return send_netlink(ctx, NULL, NETLINK_ROUTE, &nlm.hdr, NULL);
-#else
-	UNUSED(ctx);
-	UNUSED(ifindex);
-	errno = ENOTSUP;
-	return -1;
-#endif
 }
 
 static const char *prefix = "/proc/sys/net/ipv6/conf";
@@ -1780,7 +1730,7 @@ if_checkipv6(struct dhcpcd_ctx *ctx, const struct interface *ifp, int own)
 	if (ifp == NULL)
 		ifname = "all";
 	else if (own) {
-		if (if_disable_autolinklocal(ctx, ifp->index) == -1)
+		if (if_disable_autolinklocal(ctx, (int)ifp->index) == -1)
 			logger(ctx, LOG_DEBUG,
 			    "%s: if_disable_autolinklocal: %m", ifp->name);
 	}
