@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2016 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2015 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -25,6 +25,8 @@
  * SUCH DAMAGE.
  */
 
+#define _WITH_GETLINE /* Stop FreeBSD bitching */
+
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -92,16 +94,12 @@
 #define O_CONTROLGRP		O_BASE + 34
 #define O_SLAAC			O_BASE + 35
 #define O_GATEWAY		O_BASE + 36
-#define O_NOUP			O_BASE + 37
+#define O_PFXDLGMIX		O_BASE + 37
 #define O_IPV6RA_AUTOCONF	O_BASE + 38
 #define O_IPV6RA_NOAUTOCONF	O_BASE + 39
 #define O_REJECT		O_BASE + 40
+#define O_IPV6RA_ACCEPT_NOPUBLIC	O_BASE + 41
 #define O_BOOTP			O_BASE + 42
-#define O_DEFINEND		O_BASE + 43
-#define O_NODELAY		O_BASE + 44
-#define O_INFORM6		O_BASE + 45
-#define O_LASTLEASE_EXTEND	O_BASE + 46
-#define O_INACTIVE		O_BASE + 47
 
 const struct option cf_options[] = {
 	{"background",      no_argument,       NULL, 'b'},
@@ -122,7 +120,6 @@ const struct option cf_options[] = {
 	{"quiet",           no_argument,       NULL, 'q'},
 	{"request",         optional_argument, NULL, 'r'},
 	{"inform",          optional_argument, NULL, 's'},
-	{"inform6",         optional_argument, NULL, O_INFORM6},
 	{"timeout",         required_argument, NULL, 't'},
 	{"userclass",       required_argument, NULL, 'u'},
 	{"vendor",          required_argument, NULL, 'v'},
@@ -143,9 +140,7 @@ const struct option cf_options[] = {
 	{"nolink",          no_argument,       NULL, 'K'},
 	{"noipv4ll",        no_argument,       NULL, 'L'},
 	{"master",          no_argument,       NULL, 'M'},
-	{"renew",           no_argument,       NULL, 'N'},
-	{"nooption",        required_argument, NULL, 'O'},
-	{"printpidfile",    no_argument,       NULL, 'P'},
+	{"nooption",        optional_argument, NULL, 'O'},
 	{"require",         required_argument, NULL, 'Q'},
 	{"static",          required_argument, NULL, 'S'},
 	{"test",            no_argument,       NULL, 'T'},
@@ -154,9 +149,6 @@ const struct option cf_options[] = {
 	{"whitelist",       required_argument, NULL, 'W'},
 	{"blacklist",       required_argument, NULL, 'X'},
 	{"denyinterfaces",  required_argument, NULL, 'Z'},
-	{"oneshot",         no_argument,       NULL, '1'},
-	{"ipv4only",        no_argument,       NULL, '4'},
-	{"ipv6only",        no_argument,       NULL, '6'},
 	{"arping",          required_argument, NULL, O_ARPING},
 	{"destination",     required_argument, NULL, O_DESTINATION},
 	{"fallback",        required_argument, NULL, O_FALLBACK},
@@ -167,6 +159,9 @@ const struct option cf_options[] = {
 	{"ipv6ra_fork",     no_argument,       NULL, O_IPV6RA_FORK},
 	{"ipv6ra_own",      no_argument,       NULL, O_IPV6RA_OWN},
 	{"ipv6ra_own_default", no_argument,    NULL, O_IPV6RA_OWN_D},
+	{"ipv6ra_accept_nopublic", no_argument, NULL, O_IPV6RA_ACCEPT_NOPUBLIC},
+	{"ipv4only",        no_argument,       NULL, '4'},
+	{"ipv6only",        no_argument,       NULL, '6'},
 	{"ipv4",            no_argument,       NULL, O_IPV4},
 	{"noipv4",          no_argument,       NULL, O_NOIPV4},
 	{"ipv6",            no_argument,       NULL, O_IPV6},
@@ -180,7 +175,6 @@ const struct option cf_options[] = {
 	{"dev",             required_argument, NULL, O_DEV},
 	{"nodev",           no_argument,       NULL, O_NODEV},
 	{"define",          required_argument, NULL, O_DEFINE},
-	{"definend",        required_argument, NULL, O_DEFINEND},
 	{"define6",         required_argument, NULL, O_DEFINE6},
 	{"embed",           required_argument, NULL, O_EMBED},
 	{"encap",           required_argument, NULL, O_ENCAP},
@@ -196,12 +190,9 @@ const struct option cf_options[] = {
 	{"controlgroup",    required_argument, NULL, O_CONTROLGRP},
 	{"slaac",           required_argument, NULL, O_SLAAC},
 	{"gateway",         no_argument,       NULL, O_GATEWAY},
+	{"ia_pd_mix",       no_argument,       NULL, O_PFXDLGMIX},
 	{"reject",          required_argument, NULL, O_REJECT},
 	{"bootp",           no_argument,       NULL, O_BOOTP},
-	{"nodelay",         no_argument,       NULL, O_NODELAY},
-	{"noup",            no_argument,       NULL, O_NOUP},
-	{"lastleaseextend", no_argument,       NULL, O_LASTLEASE_EXTEND},
-	{"inactive",        no_argument,       NULL, O_INACTIVE},
 	{NULL,              0,                 NULL, '\0'}
 };
 
@@ -287,7 +278,7 @@ parse_string_hwaddr(char *sbuf, size_t slen, const char *str, int clid)
 	size_t l;
 	const char *p;
 	int i, punt_last = 0;
-	char c[4], cmd;
+	char c[4];
 
 	/* If surrounded by quotes then it's a string */
 	if (*str == '"') {
@@ -325,25 +316,28 @@ parse_string_hwaddr(char *sbuf, size_t slen, const char *str, int clid)
 		}
 		if (*str == '\\') {
 			str++;
-			switch((cmd = *str++)) {
+			switch(*str) {
 			case '\0':
-				str--;
 				break;
 			case 'b':
 				if (sbuf)
 					*sbuf++ = '\b';
+				str++;
 				break;
 			case 'n':
 				if (sbuf)
 					*sbuf++ = '\n';
+				str++;
 				break;
 			case 'r':
 				if (sbuf)
 					*sbuf++ = '\r';
+				str++;
 				break;
 			case 't':
 				if (sbuf)
 					*sbuf++ = '\t';
+				str++;
 				break;
 			case 'x':
 				/* Grab a hex code */
@@ -377,7 +371,8 @@ parse_string_hwaddr(char *sbuf, size_t slen, const char *str, int clid)
 				break;
 			default:
 				if (sbuf)
-					*sbuf++ = cmd;
+					*sbuf++ = *str;
+				str++;
 				break;
 			}
 		} else {
@@ -427,14 +422,12 @@ parse_iaid(uint8_t *iaid, const char *arg, size_t len)
 	return parse_iaid1(iaid, arg, len, 1);
 }
 
-#ifdef AUTH
 static int
 parse_uint32(uint32_t *i, const char *arg)
 {
 
 	return parse_iaid1((uint8_t *)i, arg, sizeof(uint32_t), 0);
 }
-#endif
 
 static char **
 splitv(struct dhcpcd_ctx *ctx, int *argc, char **argv, const char *arg)
@@ -474,6 +467,7 @@ parse_addr(struct dhcpcd_ctx *ctx,
     struct in_addr *addr, struct in_addr *net, const char *arg)
 {
 	char *p;
+	int i;
 
 	if (arg == NULL || *arg == '\0') {
 		if (addr != NULL)
@@ -483,13 +477,10 @@ parse_addr(struct dhcpcd_ctx *ctx,
 		return 0;
 	}
 	if ((p = strchr(arg, '/')) != NULL) {
-		int e;
-		intmax_t i;
-
 		*p++ = '\0';
-		i = strtoi(p, NULL, 10, 0, 32, &e);
-		if (e != 0 ||
-		    (net != NULL && inet_cidrtoaddr((int)i, net) != 0))
+		if (net != NULL &&
+		    (sscanf(p, "%d", &i) != 1 ||
+			inet_cidrtoaddr(i, net) != 0))
 		{
 			logger(ctx, LOG_ERR, "`%s' is not a valid CIDR", p);
 			return -1;
@@ -528,22 +519,11 @@ set_option_space(struct dhcpcd_ctx *ctx,
 {
 
 #if !defined(INET) && !defined(INET6)
-	UNUSED(ctx);
+	/* Satisfy use */
+	ctx = NULL;
 #endif
 
 #ifdef INET6
-	if (strncmp(arg, "nd_", strlen("nd_")) == 0) {
-		*d = ctx->nd_opts;
-		*dl = ctx->nd_opts_len;
-		*od = ifo->nd_override;
-		*odl = ifo->nd_override_len;
-		*request = ifo->requestmasknd;
-		*require = ifo->requiremasknd;
-		*no = ifo->nomasknd;
-		*reject = ifo->rejectmasknd;
-		return arg + strlen("nd_");
-	}
-
 	if (strncmp(arg, "dhcp6_", strlen("dhcp6_")) == 0) {
 		*d = ctx->dhcp6_opts;
 		*dl = ctx->dhcp6_opts_len;
@@ -624,7 +604,6 @@ strskipwhite(const char *s)
 	return UNCONST(s);
 }
 
-#ifdef AUTH
 /* Find the end pointer of a string. */
 static char *
 strend(const char *s)
@@ -646,7 +625,6 @@ strend(const char *s)
 	}
 	return UNCONST(++s);
 }
-#endif
 
 static int
 parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
@@ -655,7 +633,7 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 	int e, i, t;
 	long l;
 	unsigned long u;
-	char *p = NULL, *bp, *fp, *np, **nconf;
+	char *p = NULL, *fp, *np, **nconf;
 	ssize_t s;
 	struct in_addr addr, addr2;
 	in_addr_t *naddr;
@@ -665,14 +643,12 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 	struct dhcp_opt **dop, *ndop;
 	size_t *dop_len, dl, odl;
 	struct vivco *vivco;
-	struct group *grp;
-#ifdef AUTH
 	struct token *token;
-#endif
+	struct group *grp;
 #ifdef _REENTRANT
 	struct group grpbuf;
 #endif
-#ifdef DHCP6
+#ifdef INET6
 	size_t sl;
 	struct if_ia *ia;
 	uint8_t iaid[4];
@@ -684,19 +660,11 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 #ifdef INET6
 	i = 0;
 #endif
-
-/* Add a guard for static analysers.
- * This should not be needed really because of the argument_required option
- * in the options declaration above. */
-#define ARG_REQUIRED if (arg == NULL) goto arg_required
-
 	switch(opt) {
 	case 'f': /* FALLTHROUGH */
 	case 'g': /* FALLTHROUGH */
 	case 'n': /* FALLTHROUGH */
 	case 'x': /* FALLTHROUGH */
-	case 'N': /* FALLTHROUGH */
-	case 'P': /* FALLTHROUGH */
 	case 'T': /* FALLTHROUGH */
 	case 'U': /* FALLTHROUGH */
 	case 'V': /* We need to handle non interface options */
@@ -705,7 +673,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		ifo->options |= DHCPCD_BACKGROUND;
 		break;
 	case 'c':
-		ARG_REQUIRED;
 		free(ifo->script);
 		ifo->script = strdup(arg);
 		if (ifo->script == NULL)
@@ -715,7 +682,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		ifo->options |= DHCPCD_DEBUG;
 		break;
 	case 'e':
-		ARG_REQUIRED;
 		add_environ(ctx, ifo, arg, 1);
 		break;
 	case 'h':
@@ -751,7 +717,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		*ifo->vendorclassid = (uint8_t)s;
 		break;
 	case 'j':
-		ARG_REQUIRED;
 		/* per interface logging is not supported
 		 * don't want to overide the commandline */
 		if (ifname == NULL && ctx->logfile == NULL) {
@@ -764,7 +729,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		ifo->options |= DHCPCD_RELEASE;
 		break;
 	case 'l':
-		ARG_REQUIRED;
 		ifo->leasetime = (uint32_t)strtou(arg, NULL,
 		    0, 0, UINT32_MAX, &e);
 		if (e) {
@@ -773,7 +737,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		}
 		break;
 	case 'm':
-		ARG_REQUIRED;
 		ifo->metric = (int)strtoi(arg, NULL, 0, 0, INT32_MAX, &e);
 		if (e) {
 			logger(ctx, LOG_ERR, "failed to convert metric %s", arg);
@@ -781,7 +744,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		}
 		break;
 	case 'o':
-		ARG_REQUIRED;
 		arg = set_option_space(ctx, arg, &d, &dl, &od, &odl, ifo,
 		    &request, &require, &no, &reject);
 		if (make_option_mask(d, dl, od, odl, request, arg, 1) != 0 ||
@@ -793,7 +755,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		}
 		break;
 	case O_REJECT:
-		ARG_REQUIRED;
 		arg = set_option_space(ctx, arg, &d, &dl, &od, &odl, ifo,
 		    &request, &require, &no, &reject);
 		if (make_option_mask(d, dl, od, odl, reject, arg, 1) != 0 ||
@@ -817,6 +778,12 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		ifo->req_mask.s_addr = 0;
 		break;
 	case 's':
+		if (ifo->options & DHCPCD_IPV6 &&
+		    !(ifo->options & DHCPCD_IPV4))
+		{
+			ifo->options |= DHCPCD_INFORM;
+			break;
+		}
 		if (arg && *arg != '\0') {
 			if (parse_addr(ctx,
 			    &ifo->req_addr, &ifo->req_mask, arg) != 0)
@@ -826,13 +793,9 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 			ifo->req_mask.s_addr = 0;
 		}
 		ifo->options |= DHCPCD_INFORM | DHCPCD_PERSISTENT;
-		ifo->options &= ~DHCPCD_STATIC;
-		break;
-	case O_INFORM6:
-		ifo->options |= DHCPCD_INFORM6;
+		ifo->options &= ~(DHCPCD_ARP | DHCPCD_STATIC);
 		break;
 	case 't':
-		ARG_REQUIRED;
 		ifo->timeout = (time_t)strtoi(arg, NULL, 0, 0, INT32_MAX, &e);
 		if (e) {
 			logger(ctx, LOG_ERR, "failed to convert timeout");
@@ -853,7 +816,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		}
 		break;
 	case 'v':
-		ARG_REQUIRED;
 		p = strchr(arg, ',');
 		if (!p || !p[1]) {
 			logger(ctx, LOG_ERR, "invalid vendor format: %s", arg);
@@ -925,7 +887,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		}
 		break;
 	case 'y':
-		ARG_REQUIRED;
 		ifo->reboot = (time_t)strtoi(arg, NULL, 0, 0, UINT32_MAX, &e);
 		if (e) {
 			logger(ctx, LOG_ERR, "failed to convert reboot %s", arg);
@@ -933,7 +894,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		}
 		break;
 	case 'z':
-		ARG_REQUIRED;
 		if (ifname == NULL)
 			ctx->ifav = splitv(ctx, &ctx->ifac, ctx->ifav, arg);
 		break;
@@ -946,7 +906,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		ifo->options &= ~DHCPCD_DAEMONISE;
 		break;
 	case 'C':
-		ARG_REQUIRED;
 		/* Commas to spaces for shell */
 		while ((p = strchr(arg, ',')))
 			*p = ' ';
@@ -1018,7 +977,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		ifo->options |= DHCPCD_MASTER;
 		break;
 	case 'O':
-		ARG_REQUIRED;
 		arg = set_option_space(ctx, arg, &d, &dl, &od, &odl, ifo,
 		    &request, &require, &no, &reject);
 		if (make_option_mask(d, dl, od, odl, request, arg, -1) != 0 ||
@@ -1030,7 +988,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		}
 		break;
 	case 'Q':
-		ARG_REQUIRED;
 		arg = set_option_space(ctx, arg, &d, &dl, &od, &odl, ifo,
 		    &request, &require, &no, &reject);
 		if (make_option_mask(d, dl, od, odl, require, arg, 1) != 0 ||
@@ -1043,7 +1000,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		}
 		break;
 	case 'S':
-		ARG_REQUIRED;
 		p = strchr(arg, '=');
 		if (p == NULL) {
 			logger(ctx, LOG_ERR, "static assignment required");
@@ -1094,7 +1050,7 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 				*fp = ' ';
 				return -1;
 			}
-			if (parse_addr(ctx, &rt->dest, &rt->mask, p) == -1 ||
+			if (parse_addr(ctx, &rt->dest, &rt->net, p) == -1 ||
 			    parse_addr(ctx, &rt->gate, NULL, np) == -1)
 			{
 				free(rt);
@@ -1119,40 +1075,12 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 				return -1;
 			}
 			rt->dest.s_addr = INADDR_ANY;
-			rt->mask.s_addr = INADDR_ANY;
+			rt->net.s_addr = INADDR_ANY;
 			if (parse_addr(ctx, &rt->gate, NULL, p) == -1) {
 				free(rt);
 				return -1;
 			}
 			TAILQ_INSERT_TAIL(ifo->routes, rt, next);
-		} else if (strncmp(arg, "interface_mtu=",
-		    strlen("interface_mtu=")) == 0 ||
-		    strncmp(arg, "mtu=", strlen("mtu=")) == 0)
-		{
-			ifo->mtu = (unsigned int)strtou(p, NULL, 0,
-			    MTU_MIN, MTU_MAX, &e);
-			if (e) {
-				logger(ctx, LOG_ERR, "invalid MTU %s", p);
-				return -1;
-			}
-		} else if (strncmp(arg, "ip6_address=", strlen("ip6_address=")) == 0) {
-			np = strchr(p, '/');
-			if (np)
-				*np++ = '\0';
-			if (inet_pton(AF_INET6, p, &ifo->req_addr6) == 1) {
-				if (np) {
-					ifo->req_prefix_len = (uint8_t)strtou(np,
-					    NULL, 0, 0, 128, &e);
-					if (e) {
-						logger(ctx, LOG_ERR,
-						    "%s: failed to "
-						    "convert prefix len",
-						    ifname);
-						return -1;
-					}
-				} else
-					ifo->req_prefix_len = 128;
-			}
 		} else {
 			dl = 0;
 			if (ifo->config != NULL) {
@@ -1181,7 +1109,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 			nconf = realloc(ifo->config, sizeof(char *) * (dl + 2));
 			if (nconf == NULL) {
 				logger(ctx, LOG_ERR, "%s: %m", __func__);
-				free(p);
 				return -1;
 			}
 			ifo->config = nconf;
@@ -1220,13 +1147,9 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		ifo->blacklist[ifo->blacklist_len++] = addr2.s_addr;
 		break;
 	case 'Z':
-		ARG_REQUIRED;
 		if (ifname == NULL)
 			ctx->ifdv = splitv(ctx, &ctx->ifdc, ctx->ifdv, arg);
 		break;
-	case '1':
-		ifo->options |= DHCPCD_ONESHOT;
-		break;
 	case '4':
 		ifo->options &= ~DHCPCD_IPV6;
 		ifo->options |= DHCPCD_IPV4;
@@ -1267,7 +1190,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		}
 		break;
 	case O_DESTINATION:
-		ARG_REQUIRED;
 		arg = set_option_space(ctx, arg, &d, &dl, &od, &odl, ifo,
 		    &request, &require, &no, &reject);
 		if (make_option_mask(d, dl, od, odl,
@@ -1282,7 +1204,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		}
 		break;
 	case O_FALLBACK:
-		ARG_REQUIRED;
 		free(ifo->fallback);
 		ifo->fallback = strdup(arg);
 		if (ifo->fallback == NULL) {
@@ -1292,7 +1213,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		break;
 #endif
 	case O_IAID:
-		ARG_REQUIRED;
 		if (ifname == NULL) {
 			logger(ctx, LOG_ERR,
 			    "IAID must belong in an interface block");
@@ -1319,6 +1239,9 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 	case O_IPV6RA_OWN_D:
 		ifo->options |= DHCPCD_IPV6RA_OWN_DEFAULT;
 		break;
+	case O_IPV6RA_ACCEPT_NOPUBLIC:
+		ifo->options |= DHCPCD_IPV6RA_ACCEPT_NOPUBLIC;
+		break;
 	case O_IPV6RA_AUTOCONF:
 		ifo->options |= DHCPCD_IPV6RA_AUTOCONF;
 		break;
@@ -1328,7 +1251,7 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 	case O_NOALIAS:
 		ifo->options |= DHCPCD_NOALIAS;
 		break;
-#ifdef DHCP6
+#ifdef INET6
 	case O_IA_NA:
 		i = D6_OPTION_IA_NA;
 		/* FALLTHROUGH */
@@ -1368,7 +1291,7 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		ia = NULL;
 		for (sl = 0; sl < ifo->ia_len; sl++) {
 			if ((arg == NULL && !ifo->ia[sl].iaid_set) ||
-			    (arg != NULL && ifo->ia[sl].iaid_set &&
+			    (ifo->ia[sl].iaid_set &&
 			    ifo->ia[sl].iaid[0] == iaid[0] &&
 			    ifo->ia[sl].iaid[1] == iaid[1] &&
 			    ifo->ia[sl].iaid[2] == iaid[2] &&
@@ -1416,15 +1339,15 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 					memset(&ia->addr, 0, sizeof(ia->addr));
 				}
 				if (p && ia->ia_type == D6_OPTION_IA_PD) {
-					ia->prefix_len = (uint8_t)strtou(p,
-					    NULL, 0, 8, 120, &e);
+					i = (int)strtoi(p, NULL, 0, 8, 120, &e);
 					if (e) {
 						logger(ctx, LOG_ERR,
 						    "%s: failed to convert"
 						    " prefix len",
 						    p);
 						ia->prefix_len = 0;
-					}
+					} else
+						ia->prefix_len = (uint8_t)i;
 				}
 			}
 			ia->sla_max = 0;
@@ -1453,19 +1376,18 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 			if (strlcpy(sla->ifname, p,
 			    sizeof(sla->ifname)) >= sizeof(sla->ifname))
 			{
-				logger(ctx, LOG_ERR,
-				    "%s: interface name too long", arg);
+				logger(ctx, LOG_ERR, "%s: interface name too long",
+				    arg);
 				goto err_sla;
 			}
-			sla->sla_set = 0;
-			sla->prefix_len = 0;
-			sla->suffix = 1;
 			p = np;
 			if (p) {
 				np = strchr(p, '/');
 				if (np)
 					*np++ = '\0';
-				if (*p != '\0') {
+				if (*p == '\0')
+					sla->sla_set = 0;
+				else {
 					sla->sla = (uint32_t)strtou(p, NULL,
 					    0, 0, UINT32_MAX, &e);
 					sla->sla_set = 1;
@@ -1476,61 +1398,31 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 						goto err_sla;
 					}
 				}
-				p = np;
-			}
-			if (p) {
-				np = strchr(p, '/');
-				if (np)
-					*np++ = '\0';
-				if (*p != '\0') {
-					sla->prefix_len = (uint8_t)strtou(p,
-				    NULL, 0, 0, 120, &e);
+				if (np) {
+					sla->prefix_len = (uint8_t)strtoi(np,
+					    NULL, 0, 0, 128, &e);
 					if (e) {
-						logger(ctx, LOG_ERR,
-						    "%s: failed to "
+						logger(ctx, LOG_ERR, "%s: failed to "
 						    "convert prefix len",
 						    ifname);
 						goto err_sla;
 					}
-				}
-				p = np;
-			}
-			if (p) {
-				np = strchr(p, '/');
-				if (np)
-					*np = '\0';
-				if (*p != '\0') {
-					sla->suffix = (uint64_t)strtou(p, NULL,
-					    0, 0, UINT64_MAX, &e);
-					if (e) {
-						logger(ctx, LOG_ERR,
-						    "%s: failed to "
-						    "convert suffix",
-						    ifname);
-						goto err_sla;
-					}
-				}
+				} else
+					sla->prefix_len = 0;
+			} else {
+				sla->sla_set = 0;
+				sla->prefix_len = 0;
 			}
 			/* Sanity check */
 			for (sl = 0; sl < ia->sla_len - 1; sl++) {
 				slap = &ia->sla[sl];
 				if (slap->sla_set != sla->sla_set) {
-					logger(ctx, LOG_ERR,
+					logger(ctx, LOG_WARNING,
 					    "%s: cannot mix automatic "
 					    "and fixed SLA",
 					    sla->ifname);
 					goto err_sla;
 				}
-				if (ia->prefix_len &&
-				    (sla->prefix_len == ia->prefix_len ||
-				    slap->prefix_len == ia->prefix_len))
-				{
-					logger(ctx, LOG_ERR,
-					    "%s: cannot delegte the same"
-					    "prefix length more than once",
-					    sla->ifname);
-					goto err_sla;
-				}
 				if (sla->sla_set == 0 &&
 				    strcmp(slap->ifname, sla->ifname) == 0)
 				{
@@ -1541,13 +1433,11 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 					    sla->ifname);
 					goto err_sla;
 				}
-				if (slap->sla_set && sla->sla_set &&
-				    slap->sla == sla->sla)
-				{
+				if (slap->sla == 0 || sla->sla == 0) {
 					logger(ctx, LOG_ERR, "%s: cannot"
-					    " assign the same SLA %u"
-					    " more than once",
-					    sla->ifname, sla->sla);
+					    " assign multiple prefixes"
+					    " with a SLA of 0",
+					    ifname);
 					goto err_sla;
 				}
 			}
@@ -1563,7 +1453,6 @@ err_sla:
 		ifo->options |= DHCPCD_HOSTNAME | DHCPCD_HOSTNAME_SHORT;
 		break;
 	case O_DEV:
-		ARG_REQUIRED;
 #ifdef PLUGIN_DEV
 		if (ctx->dev_load)
 			free(ctx->dev_load);
@@ -1577,12 +1466,6 @@ err_sla:
 		dop = &ifo->dhcp_override;
 		dop_len = &ifo->dhcp_override_len;
 		/* FALLTHROUGH */
-	case O_DEFINEND:
-		if (dop == NULL) {
-			dop = &ifo->nd_override;
-			dop_len = &ifo->nd_override_len;
-		}
-		/* FALLTHROUGH */
 	case O_DEFINE6:
 		if (dop == NULL) {
 			dop = &ifo->dhcp6_override;
@@ -1612,7 +1495,6 @@ err_sla:
 		}
 		/* FALLTHROUGH */
 	case O_ENCAP:
-		ARG_REQUIRED;
 		if (dop == NULL) {
 			if (*ldop == NULL) {
 				logger(ctx, LOG_ERR, "encap must be after a define");
@@ -1653,21 +1535,16 @@ err_sla:
 		/* length */
 		if (np) {
 			*np++ = '\0';
-			bp = NULL; /* No bitflag */
 			l = (long)strtou(np, NULL, 0, 0, LONG_MAX, &e);
 			if (e) {
-				logger(ctx,LOG_ERR, "failed to convert length");
+				logger(ctx, LOG_ERR, "failed to convert length");
 				return -1;
 			}
-		} else {
+		} else
 			l = 0;
-			bp = strchr(arg, '='); /* bitflag assignment */
-			if (bp)
-				*bp++ = '\0';
-		}
 		t = 0;
 		if (strcasecmp(arg, "request") == 0) {
-			t |= OT_REQUEST;
+			t |= REQUEST;
 			arg = strskipwhite(fp);
 			fp = strwhite(arg);
 			if (fp == NULL) {
@@ -1676,7 +1553,7 @@ err_sla:
 			}
 			*fp++ = '\0';
 		} else if (strcasecmp(arg, "norequest") == 0) {
-			t |= OT_NOREQ;
+			t |= NOREQ;
 			arg = strskipwhite(fp);
 			fp = strwhite(arg);
 			if (fp == NULL) {
@@ -1685,19 +1562,8 @@ err_sla:
 			}
 			*fp++ = '\0';
 		}
-		if (strcasecmp(arg, "optional") == 0) {
-			t |= OT_OPTIONAL;
-			arg = strskipwhite(fp);
-			fp = strwhite(arg);
-			if (fp == NULL) {
-				logger(ctx, LOG_ERR,
-				    "incomplete optional type");
-				return -1;
-			}
-			*fp++ = '\0';
-		}
 		if (strcasecmp(arg, "index") == 0) {
-			t |= OT_INDEX;
+			t |= INDEX;
 			arg = strskipwhite(fp);
 			fp = strwhite(arg);
 			if (fp == NULL) {
@@ -1707,7 +1573,7 @@ err_sla:
 			*fp++ = '\0';
 		}
 		if (strcasecmp(arg, "array") == 0) {
-			t |= OT_ARRAY;
+			t |= ARRAY;
 			arg = strskipwhite(fp);
 			fp = strwhite(arg);
 			if (fp == NULL) {
@@ -1717,72 +1583,63 @@ err_sla:
 			*fp++ = '\0';
 		}
 		if (strcasecmp(arg, "ipaddress") == 0)
-			t |= OT_ADDRIPV4;
+			t |= ADDRIPV4;
 		else if (strcasecmp(arg, "ip6address") == 0)
-			t |= OT_ADDRIPV6;
+			t |= ADDRIPV6;
 		else if (strcasecmp(arg, "string") == 0)
-			t |= OT_STRING;
+			t |= STRING;
 		else if (strcasecmp(arg, "byte") == 0)
-			t |= OT_UINT8;
-		else if (strcasecmp(arg, "bitflags") == 0)
-			t |= OT_BITFLAG;
-		else if (strcasecmp(arg, "uint8") == 0)
-			t |= OT_UINT8;
-		else if (strcasecmp(arg, "int8") == 0)
-			t |= OT_INT8;
+			t |= UINT8;
 		else if (strcasecmp(arg, "uint16") == 0)
-			t |= OT_UINT16;
+			t |= UINT16;
 		else if (strcasecmp(arg, "int16") == 0)
-			t |= OT_INT16;
+			t |= SINT16;
 		else if (strcasecmp(arg, "uint32") == 0)
-			t |= OT_UINT32;
+			t |= UINT32;
 		else if (strcasecmp(arg, "int32") == 0)
-			t |= OT_INT32;
+			t |= SINT32;
 		else if (strcasecmp(arg, "flag") == 0)
-			t |= OT_FLAG;
+			t |= FLAG;
 		else if (strcasecmp(arg, "raw") == 0)
-			t |= OT_STRING | OT_RAW;
+			t |= STRING | RAW;
 		else if (strcasecmp(arg, "ascii") == 0)
-			t |= OT_STRING | OT_ASCII;
+			t |= STRING | ASCII;
 		else if (strcasecmp(arg, "domain") == 0)
-			t |= OT_STRING | OT_DOMAIN | OT_RFC1035;
+			t |= STRING | DOMAIN | RFC3397;
 		else if (strcasecmp(arg, "dname") == 0)
-			t |= OT_STRING | OT_DOMAIN;
+			t |= STRING | DOMAIN;
 		else if (strcasecmp(arg, "binhex") == 0)
-			t |= OT_STRING | OT_BINHEX;
+			t |= STRING | BINHEX;
 		else if (strcasecmp(arg, "embed") == 0)
-			t |= OT_EMBED;
+			t |= EMBED;
 		else if (strcasecmp(arg, "encap") == 0)
-			t |= OT_ENCAP;
+			t |= ENCAP;
 		else if (strcasecmp(arg, "rfc3361") ==0)
-			t |= OT_STRING | OT_RFC3361;
+			t |= STRING | RFC3361;
 		else if (strcasecmp(arg, "rfc3442") ==0)
-			t |= OT_STRING | OT_RFC3442;
+			t |= STRING | RFC3442;
+		else if (strcasecmp(arg, "rfc5969") == 0)
+			t |= STRING | RFC5969;
 		else if (strcasecmp(arg, "option") == 0)
-			t |= OT_OPTION;
+			t |= OPTION;
 		else {
 			logger(ctx, LOG_ERR, "unknown type: %s", arg);
 			return -1;
 		}
-		if (l && !(t & (OT_STRING | OT_BINHEX))) {
+		if (l && !(t & (STRING | BINHEX))) {
 			logger(ctx, LOG_WARNING,
 			    "ignoring length for type `%s'", arg);
 			l = 0;
 		}
-		if (t & OT_ARRAY && t & (OT_STRING | OT_BINHEX) &&
-		    !(t & (OT_RFC1035 | OT_DOMAIN)))
+		if (t & ARRAY && t & (STRING | BINHEX) &&
+		    !(t & (RFC3397 | DOMAIN)))
 		{
 			logger(ctx, LOG_WARNING, "ignoring array for strings");
-			t &= ~OT_ARRAY;
-		}
-		if (t & OT_BITFLAG) {
-			if (bp == NULL)
-				logger(ctx, LOG_WARNING,
-				    "missing bitflag assignment");
+			t &= ~ARRAY;
 		}
 		/* variable */
 		if (!fp) {
-			if (!(t & OT_OPTION)) {
+			if (!(t & OPTION)) {
 			        logger(ctx, LOG_ERR,
 				    "type %s requires a variable name", arg);
 				return -1;
@@ -1793,16 +1650,10 @@ err_sla:
 			fp = strwhite(arg);
 			if (fp)
 				*fp++ = '\0';
-			if (strcasecmp(arg, "reserved")) {
-				np = strdup(arg);
-				if (np == NULL) {
-					logger(ctx, LOG_ERR,
-					    "%s: %m", __func__);
-					return -1;
-				}
-			} else {
-				np = NULL;
-				t |= OT_RESERVED;
+			np = strdup(arg);
+			if (np == NULL) {
+				logger(ctx, LOG_ERR, "%s: %m", __func__);
+				return -1;
 			}
 		}
 		if (opt != O_EMBED) {
@@ -1837,28 +1688,13 @@ err_sla:
 		ndop->type = t;
 		ndop->len = (size_t)l;
 		ndop->var = np;
-		if (bp) {
-			dl = strlen(bp);
-			memcpy(ndop->bitflags, bp, dl);
-			memset(ndop->bitflags + dl, 0,
-			    sizeof(ndop->bitflags) - dl);
-		} else
-			memset(ndop->bitflags, 0, sizeof(ndop->bitflags));
 		/* Save the define for embed and encap options */
-		switch (opt) {
-		case O_DEFINE:
-		case O_DEFINEND:
-		case O_DEFINE6:
-		case O_VENDOPT:
+		if (opt == O_DEFINE || opt == O_DEFINE6 || opt == O_VENDOPT)
 			*ldop = ndop;
-			break;
-		case O_ENCAP:
+		else if (opt == O_ENCAP)
 			*edop = ndop;
-			break;
-		}
 		break;
 	case O_VENDCLASS:
-		ARG_REQUIRED;
 		fp = strwhite(arg);
 		if (fp)
 			*fp++ = '\0';
@@ -1901,8 +1737,6 @@ err_sla:
 		vivco->data = (uint8_t *)np;
 		break;
 	case O_AUTHPROTOCOL:
-		ARG_REQUIRED;
-#ifdef AUTH
 		fp = strwhite(arg);
 		if (fp)
 			*fp++ = '\0';
@@ -1951,13 +1785,7 @@ err_sla:
 		}
 		ifo->auth.options |= DHCPCD_AUTH_SEND;
 		break;
-#else
-		logger(ctx, LOG_ERR, "no authentication support");
-		return -1;
-#endif
 	case O_AUTHTOKEN:
-		ARG_REQUIRED;
-#ifdef AUTH
 		fp = strwhite(arg);
 		if (fp == NULL) {
 			logger(ctx, LOG_ERR, "authtoken requires a realm");
@@ -2050,10 +1878,6 @@ err_sla:
 		token->key = malloc(token->key_len);
 		parse_string((char *)token->key, token->key_len, arg);
 		TAILQ_INSERT_TAIL(&ifo->auth.tokens, token, next);
-#else
-		logger(ctx, LOG_ERR, "no authentication support");
-		return -1;
-#endif
 		break;
 	case O_AUTHNOTREQUIRED:
 		ifo->auth.options &= ~DHCPCD_AUTH_REQUIRE;
@@ -2071,7 +1895,6 @@ err_sla:
 		ifo->options &= ~DHCPCD_DHCP6;
 		break;
 	case O_CONTROLGRP:
-		ARG_REQUIRED;
 #ifdef _REENTRANT
 		l = sysconf(_SC_GETGR_R_SIZE_MAX);
 		if (l == -1)
@@ -2126,11 +1949,7 @@ err_sla:
 	case O_GATEWAY:
 		ifo->options |= DHCPCD_GATEWAY;
 		break;
-	case O_NOUP:
-		ifo->options &= ~DHCPCD_IF_UP;
-		break;
 	case O_SLAAC:
-		ARG_REQUIRED;
 		if (strcmp(arg, "private") == 0 ||
 		    strcmp(arg, "stableprivate") == 0 ||
 		    strcmp(arg, "stable") == 0)
@@ -2138,30 +1957,17 @@ err_sla:
 		else
 			ifo->options &= ~DHCPCD_SLAACPRIVATE;
 		break;
+	case O_PFXDLGMIX:
+		ifo->options |= DHCPCD_PFXDLGMIX;
+		break;
 	case O_BOOTP:
 		ifo->options |= DHCPCD_BOOTP;
 		break;
-	case O_NODELAY:
-		ifo->options &= ~DHCPCD_INITIAL_DELAY;
-		break;
-	case O_LASTLEASE_EXTEND:
-		ifo->options |= DHCPCD_LASTLEASE | DHCPCD_LASTLEASE_EXTEND;
-		break;
-	case O_INACTIVE:
-		ifo->options |= DHCPCD_INACTIVE;
-		break;
 	default:
 		return 0;
 	}
 
 	return 1;
-
-#ifdef ARG_REQUIRED
-arg_required:
-	logger(ctx, LOG_ERR, "option %d requires an argument", opt);
-	return -1;
-#undef ARG_REQUIRED
-#endif
 }
 
 static int
@@ -2214,9 +2020,8 @@ static char *
 get_line(char ** __restrict buf, size_t * __restrict buflen,
     FILE * __restrict fp)
 {
-	char *p, *c;
+	char *p;
 	ssize_t bytes;
-	int quoted;
 
 	do {
 		bytes = getline(buf, buflen, fp);
@@ -2227,52 +2032,9 @@ get_line(char ** __restrict buf, size_t * __restrict buflen,
 	} while (*p == '\0' || *p == '\n' || *p == '#' || *p == ';');
 	if ((*buf)[--bytes] == '\n')
 		(*buf)[bytes] = '\0';
-
-	/* Strip embedded comments unless in a quoted string or escaped */
-	quoted = 0;
-	for (c = p; *c != '\0'; c++) {
-		if (*c == '\\') {
-			c++; /* escaped */
-			continue;
-		}
-		if (*c == '"')
-			quoted = !quoted;
-		else if (*c == '#' && !quoted) {
-			*c = '\0';
-			break;
-		}
-	}
 	return p;
 }
 
-struct if_options *
-default_config(struct dhcpcd_ctx *ctx)
-{
-	struct if_options *ifo;
-
-	/* Seed our default options */
-	if ((ifo = calloc(1, sizeof(*ifo))) == NULL) {
-		logger(ctx, LOG_ERR, "%s: %m", __func__);
-		return NULL;
-	}
-	ifo->options |= DHCPCD_IF_UP | DHCPCD_LINK | DHCPCD_INITIAL_DELAY;
-	ifo->timeout = DEFAULT_TIMEOUT;
-	ifo->reboot = DEFAULT_REBOOT;
-	ifo->metric = -1;
-	ifo->auth.options |= DHCPCD_AUTH_REQUIRE;
-#ifdef AUTH
-	TAILQ_INIT(&ifo->auth.tokens);
-#endif
-
-	/* Inherit some global defaults */
-	if (ctx->options & DHCPCD_PERSISTENT)
-		ifo->options |= DHCPCD_PERSISTENT;
-	if (ctx->options & DHCPCD_SLAACPRIVATE)
-		ifo->options |= DHCPCD_SLAACPRIVATE;
-
-	return ifo;
-}
-
 struct if_options *
 read_config(struct dhcpcd_ctx *ctx,
     const char *ifname, const char *ssid, const char *profile)
@@ -2283,7 +2045,7 @@ read_config(struct dhcpcd_ctx *ctx,
 	char *line, *buf, *option, *p;
 	size_t buflen;
 	ssize_t vlen;
-	int skip, have_profile, new_block, had_block;
+	int skip = 0, have_profile = 0;
 #ifndef EMBEDDED_CONFIG
 	const char * const *e;
 	size_t ol;
@@ -2295,9 +2057,12 @@ read_config(struct dhcpcd_ctx *ctx,
 	struct dhcp_opt *ldop, *edop;
 
 	/* Seed our default options */
-	if ((ifo = default_config(ctx)) == NULL)
+	ifo = calloc(1, sizeof(*ifo));
+	if (ifo == NULL) {
+		logger(ctx, LOG_ERR, "%s: %m", __func__);
 		return NULL;
-	ifo->options |= DHCPCD_DAEMONISE;
+	}
+	ifo->options |= DHCPCD_DAEMONISE | DHCPCD_LINK;
 #ifdef PLUGIN_DEV
 	ifo->options |= DHCPCD_DEV;
 #endif
@@ -2310,6 +2075,11 @@ read_config(struct dhcpcd_ctx *ctx,
 	ifo->options |= DHCPCD_IPV6RA_AUTOCONF | DHCPCD_IPV6RA_REQRDNSS;
 	ifo->options |= DHCPCD_DHCP6;
 #endif
+	ifo->timeout = DEFAULT_TIMEOUT;
+	ifo->reboot = DEFAULT_REBOOT;
+	ifo->metric = -1;
+	ifo->auth.options |= DHCPCD_AUTH_REQUIRE;
+	TAILQ_INIT(&ifo->auth.tokens);
 
 	vlen = dhcp_vendor((char *)ifo->vendorclassid + 1,
 	            sizeof(ifo->vendorclassid) - 1);
@@ -2319,7 +2089,7 @@ read_config(struct dhcpcd_ctx *ctx,
 	buflen = 0;
 
 	/* Parse our embedded options file */
-	if (ifname == NULL && !(ctx->options & DHCPCD_PRINT_PIDFILE)) {
+	if (ifname == NULL) {
 		/* Space for initial estimates */
 #if defined(INET) && defined(INITDEFINES)
 		ifo->dhcp_override =
@@ -2330,14 +2100,6 @@ read_config(struct dhcpcd_ctx *ctx,
 			ifo->dhcp_override_len = INITDEFINES;
 #endif
 
-#if defined(INET6) && defined(INITDEFINENDS)
-		ifo->nd_override =
-		    calloc(INITDEFINENDS, sizeof(*ifo->nd_override));
-		if (ifo->nd_override == NULL)
-			logger(ctx, LOG_ERR, "%s: %m", __func__);
-		else
-			ifo->nd_override_len = INITDEFINENDS;
-#endif
 #if defined(INET6) && defined(INITDEFINE6S)
 		ifo->dhcp6_override =
 		    calloc(INITDEFINE6S, sizeof(*ifo->dhcp6_override));
@@ -2359,25 +2121,19 @@ read_config(struct dhcpcd_ctx *ctx,
 		buf = malloc(buflen);
 		if (buf == NULL) {
 			logger(ctx, LOG_ERR, "%s: %m", __func__);
-			free_options(ifo);
 			return NULL;
 		}
 		ldop = edop = NULL;
 		for (e = dhcpcd_embedded_conf; *e; e++) {
 			ol = strlen(*e) + 1;
 			if (ol > buflen) {
-				char *nbuf;
-
 				buflen = ol;
-				nbuf = realloc(buf, buflen);
-				if (nbuf == NULL) {
-					logger(ctx, LOG_ERR,
-					    "%s: %m", __func__);
+				buf = realloc(buf, buflen);
+				if (buf == NULL) {
+					logger(ctx, LOG_ERR, "%s: %m", __func__);
 					free(buf);
-					free_options(ifo);
 					return NULL;
 				}
-				buf = nbuf;
 			}
 			memcpy(buf, *e, ol);
 			line = buf;
@@ -2416,24 +2172,15 @@ read_config(struct dhcpcd_ctx *ctx,
 		ifo->dhcp_override_len = 0;
 
 #ifdef INET6
-		ctx->nd_opts = ifo->nd_override;
-		ctx->nd_opts_len = ifo->nd_override_len;
 		ctx->dhcp6_opts = ifo->dhcp6_override;
 		ctx->dhcp6_opts_len = ifo->dhcp6_override_len;
 #else
-		for (i = 0, opt = ifo->nd_override;
-		    i < ifo->nd_override_len;
-		    i++, opt++)
-			free_dhcp_opt_embenc(opt);
-		free(ifo->nd_override);
 		for (i = 0, opt = ifo->dhcp6_override;
 		    i < ifo->dhcp6_override_len;
 		    i++, opt++)
 			free_dhcp_opt_embenc(opt);
 		free(ifo->dhcp6_override);
 #endif
-		ifo->nd_override = NULL;
-		ifo->nd_override_len = 0;
 		ifo->dhcp6_override = NULL;
 		ifo->dhcp6_override_len = 0;
 
@@ -2455,8 +2202,6 @@ read_config(struct dhcpcd_ctx *ctx,
 		ifo->mtime = sb.st_mtime;
 
 	ldop = edop = NULL;
-	skip = have_profile = new_block = 0;
-	had_block = ifname == NULL ? 1 : 0;
 	while ((line = get_line(&buf, &buflen, fp))) {
 		option = strsep(&line, " \t");
 		if (line)
@@ -2469,16 +2214,10 @@ read_config(struct dhcpcd_ctx *ctx,
 			    *(p - 1) != '\\')
 				*p-- = '\0';
 		}
-		if (skip == 0 && new_block) {
-			had_block = 1;
-			new_block = 0;
-			ifo->options &= ~DHCPCD_WAITOPTS;
-		}
 		/* Start of an interface block, skip if not ours */
 		if (strcmp(option, "interface") == 0) {
 			char **n;
 
-			new_block = 1;
 			if (ifname && line && strcmp(line, ifname) == 0)
 				skip = 0;
 			else
@@ -2499,11 +2238,12 @@ read_config(struct dhcpcd_ctx *ctx,
 				continue;
 			}
 			ctx->ifcc++;
+			logger(ctx, LOG_DEBUG, "allowing interface %s",
+			    ctx->ifcv[ctx->ifcc - 1]);
 			continue;
 		}
 		/* Start of an ssid block, skip if not ours */
 		if (strcmp(option, "ssid") == 0) {
-			new_block = 1;
 			if (ssid && line && strcmp(line, ssid) == 0)
 				skip = 0;
 			else
@@ -2512,7 +2252,6 @@ read_config(struct dhcpcd_ctx *ctx,
 		}
 		/* Start of a profile block, skip if not ours */
 		if (strcmp(option, "profile") == 0) {
-			new_block = 1;
 			if (profile && line && strcmp(line, profile) == 0) {
 				skip = 0;
 				have_profile = 1;
@@ -2537,8 +2276,6 @@ read_config(struct dhcpcd_ctx *ctx,
 		return NULL;
 	}
 
-	if (!had_block)
-		ifo->options &= ~DHCPCD_WAITOPTS;
 	finish_config(ifo);
 	return ifo;
 }
@@ -2548,29 +2285,18 @@ add_options(struct dhcpcd_ctx *ctx, const char *ifname,
     struct if_options *ifo, int argc, char **argv)
 {
 	int oi, opt, r;
-	unsigned long long wait_opts;
 
 	if (argc == 0)
 		return 1;
 
 	optind = 0;
 	r = 1;
-	/* Don't apply the command line wait options to each interface,
-	 * only use the dhcpcd.conf entry for that. */
-	if (ifname != NULL)
-		wait_opts = ifo->options & DHCPCD_WAITOPTS;
-	while ((opt = getopt_long(argc, argv,
-	    ctx->options & DHCPCD_PRINT_PIDFILE ? NOERR_IF_OPTS : IF_OPTS,
-	    cf_options, &oi)) != -1)
+	while ((opt = getopt_long(argc, argv, IF_OPTS, cf_options, &oi)) != -1)
 	{
 		r = parse_option(ctx, ifname, ifo, opt, optarg, NULL, NULL);
 		if (r != 1)
 			break;
 	}
-	if (ifname != NULL) {
-		ifo->options &= ~DHCPCD_WAITOPTS;
-		ifo->options |= wait_opts;
-	}
 
 	finish_config(ifo);
 	return r;
@@ -2582,9 +2308,7 @@ free_options(struct if_options *ifo)
 	size_t i;
 	struct dhcp_opt *opt;
 	struct vivco *vo;
-#ifdef AUTH
 	struct token *token;
-#endif
 
 	if (ifo) {
 		if (ifo->environ) {
@@ -2610,11 +2334,6 @@ free_options(struct if_options *ifo)
 		    opt++, ifo->dhcp_override_len--)
 			free_dhcp_opt_embenc(opt);
 		free(ifo->dhcp_override);
-		for (opt = ifo->nd_override;
-		    ifo->nd_override_len > 0;
-		    opt++, ifo->nd_override_len--)
-			free_dhcp_opt_embenc(opt);
-		free(ifo->nd_override);
 		for (opt = ifo->dhcp6_override;
 		    ifo->dhcp6_override_len > 0;
 		    opt++, ifo->dhcp6_override_len--)
@@ -2637,7 +2356,6 @@ free_options(struct if_options *ifo)
 #endif
 		free(ifo->ia);
 
-#ifdef AUTH
 		while ((token = TAILQ_FIRST(&ifo->auth.tokens))) {
 			TAILQ_REMOVE(&ifo->auth.tokens, token, next);
 			if (token->realm_len)
@@ -2645,7 +2363,6 @@ free_options(struct if_options *ifo)
 			free(token->key);
 			free(token);
 		}
-#endif
 		free(ifo);
 	}
 }
