@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2016 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2015 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -30,11 +30,12 @@
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 
-#include "config.h"
-
 #include <net/if.h>
 #include <net/if_arp.h>
 #include <netinet/in.h>
+#ifdef __FreeBSD__ /* Needed so that including netinet6/in6_var.h works */
+#  include <net/if_var.h>
+#endif
 #ifdef AF_LINK
 #  include <net/if_dl.h>
 #  include <net/if_types.h>
@@ -51,15 +52,14 @@
 #include <ctype.h>
 #include <errno.h>
 #include <ifaddrs.h>
-#include <inttypes.h>
 #include <fnmatch.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <fcntl.h>
 
+#include "config.h"
 #include "common.h"
 #include "dev.h"
 #include "dhcp.h"
@@ -67,18 +67,21 @@
 #include "if.h"
 #include "if-options.h"
 #include "ipv4.h"
-#include "ipv4ll.h"
 #include "ipv6nd.h"
 
+#ifdef __QNX__
+/* QNX carries defines for, but does not actually support PF_LINK */
+#undef IFLR_ACTIVE
+#endif
+
 void
 if_free(struct interface *ifp)
 {
 
 	if (ifp == NULL)
 		return;
-	ipv4ll_free(ifp);
-	dhcp_free(ifp);
 	ipv4_free(ifp);
+	dhcp_free(ifp);
 	dhcp6_free(ifp);
 	ipv6nd_free(ifp);
 	ipv6_free(ifp);
@@ -87,62 +90,28 @@ if_free(struct interface *ifp)
 }
 
 int
-if_opensockets(struct dhcpcd_ctx *ctx)
+if_carrier(struct interface *iface)
 {
-
-	if (if_opensockets_os(ctx) == -1)
-		return -1;
-
-	/* We use this socket for some operations without INET. */
-	ctx->pf_inet_fd = xsocket(PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
-	if (ctx->pf_inet_fd == -1)
-		return -1;
-
-#ifdef IFLR_ACTIVE
-	ctx->pf_link_fd = xsocket(PF_LINK, SOCK_DGRAM | SOCK_CLOEXEC, 0);
-	if (ctx->pf_link_fd == -1)
-		return -1;
-#endif
-
-	return 0;
-}
-
-void
-if_closesockets(struct dhcpcd_ctx *ctx)
-{
-
-	if (ctx->pf_inet_fd != -1)
-		close(ctx->pf_inet_fd);
-#ifdef IFLR_ACTIVE
-	if (ctx->pf_link_fd != -1)
-		close(ctx->pf_link_fd);
-#endif
-
-	if (ctx->priv) {
-		if_closesockets_os(ctx);
-		free(ctx->priv);
-	}
-}
-
-int
-if_carrier(struct interface *ifp)
-{
-	int r;
+	int s, r;
 	struct ifreq ifr;
 #ifdef SIOCGIFMEDIA
 	struct ifmediareq ifmr;
 #endif
 
-	memset(&ifr, 0, sizeof(ifr));
-	strlcpy(ifr.ifr_name, ifp->name, sizeof(ifr.ifr_name));
-	if (ioctl(ifp->ctx->pf_inet_fd, SIOCGIFFLAGS, &ifr) == -1)
+	if ((s = socket(PF_INET, SOCK_DGRAM, 0)) == -1)
 		return LINK_UNKNOWN;
-	ifp->flags = (unsigned int)ifr.ifr_flags;
+	memset(&ifr, 0, sizeof(ifr));
+	strlcpy(ifr.ifr_name, iface->name, sizeof(ifr.ifr_name));
+	if (ioctl(s, SIOCGIFFLAGS, &ifr) == -1) {
+		close(s);
+		return LINK_UNKNOWN;
+	}
+	iface->flags = (unsigned int)ifr.ifr_flags;
 
 #ifdef SIOCGIFMEDIA
 	memset(&ifmr, 0, sizeof(ifmr));
-	strlcpy(ifmr.ifm_name, ifp->name, sizeof(ifmr.ifm_name));
-	if (ioctl(ifp->ctx->pf_inet_fd, SIOCGIFMEDIA, &ifmr) != -1 &&
+	strlcpy(ifmr.ifm_name, iface->name, sizeof(ifmr.ifm_name));
+	if (ioctl(s, SIOCGIFMEDIA, &ifmr) != -1 &&
 	    ifmr.ifm_status & IFM_AVALID)
 		r = (ifmr.ifm_status & IFM_ACTIVE) ? LINK_UP : LINK_DOWN;
 	else
@@ -150,6 +119,7 @@ if_carrier(struct interface *ifp)
 #else
 	r = ifr.ifr_flags & IFF_RUNNING ? LINK_UP : LINK_DOWN;
 #endif
+	close(s);
 	return r;
 }
 
@@ -157,21 +127,24 @@ int
 if_setflag(struct interface *ifp, short flag)
 {
 	struct ifreq ifr;
-	int r;
+	int s, r;
 
+	if ((s = socket(PF_INET, SOCK_DGRAM, 0)) == -1)
+		return -1;
 	memset(&ifr, 0, sizeof(ifr));
 	strlcpy(ifr.ifr_name, ifp->name, sizeof(ifr.ifr_name));
 	r = -1;
-	if (ioctl(ifp->ctx->pf_inet_fd, SIOCGIFFLAGS, &ifr) == 0) {
+	if (ioctl(s, SIOCGIFFLAGS, &ifr) == 0) {
 		if (flag == 0 || (ifr.ifr_flags & flag) == flag)
 			r = 0;
 		else {
 			ifr.ifr_flags |= flag;
-			if (ioctl(ifp->ctx->pf_inet_fd, SIOCSIFFLAGS, &ifr) ==0)
+			if (ioctl(s, SIOCSIFFLAGS, &ifr) == 0)
 				r = 0;
 		}
 		ifp->flags = (unsigned int)ifr.ifr_flags;
 	}
+	close(s);
 	return r;
 }
 
@@ -187,76 +160,61 @@ if_hasconf(struct dhcpcd_ctx *ctx, const char *ifname)
 	return 0;
 }
 
-static void if_learnaddrs(struct dhcpcd_ctx *ctx, struct if_head *ifs,
+static void if_learnaddrs1(struct dhcpcd_ctx *ctx, struct if_head *ifs,
     struct ifaddrs *ifaddrs)
 {
 	struct ifaddrs *ifa;
 	struct interface *ifp;
 #ifdef INET
-	const struct sockaddr_in *addr, *net, *brd;
+	const struct sockaddr_in *addr, *net, *dst;
 #endif
 #ifdef INET6
 	struct sockaddr_in6 *sin6, *net6;
 #endif
-	int addrflags;
+	int ifa_flags;
+
 
 	for (ifa = ifaddrs; ifa; ifa = ifa->ifa_next) {
 		if (ifa->ifa_addr == NULL)
 			continue;
 		if ((ifp = if_find(ifs, ifa->ifa_name)) == NULL)
 			continue;
-#ifdef HAVE_IFADDRS_ADDRFLAGS
-		addrflags = (int)ifa->ifa_addrflags;
-#endif
 		switch(ifa->ifa_addr->sa_family) {
 #ifdef INET
 		case AF_INET:
-			addr = (void *)ifa->ifa_addr;
-			net = (void *)ifa->ifa_netmask;
+			addr = (const struct sockaddr_in *)
+			    (void *)ifa->ifa_addr;
+			net = (const struct sockaddr_in *)
+			    (void *)ifa->ifa_netmask;
 			if (ifa->ifa_flags & IFF_POINTOPOINT)
-				brd = (void *)ifa->ifa_dstaddr;
+				dst = (const struct sockaddr_in *)
+				    (void *)ifa->ifa_dstaddr;
 			else
-				brd = (void *)ifa->ifa_broadaddr;
-#ifndef HAVE_IFADDRS_ADDRFLAGS
-			addrflags = if_addrflags(ifp, &addr->sin_addr,
-			    ifa->ifa_name);
-			if (addrflags == -1) {
-				if (errno != EEXIST)
-					logger(ctx, LOG_ERR,
-					    "%s: if_addrflags: %s: %m",
-					    __func__,
-					    inet_ntoa(addr->sin_addr));
-				continue;
-			}
-#endif
+				dst = NULL;
+			ifa_flags = if_addrflags(&addr->sin_addr, ifp);
 			ipv4_handleifa(ctx, RTM_NEWADDR, ifs, ifa->ifa_name,
-				&addr->sin_addr, &net->sin_addr,
-				brd ? &brd->sin_addr : NULL, addrflags);
+				&addr->sin_addr,
+				&net->sin_addr,
+				dst ? &dst->sin_addr : NULL, ifa_flags);
 			break;
 #endif
 #ifdef INET6
 		case AF_INET6:
-			sin6 = (void *)ifa->ifa_addr;
-			net6 = (void *)ifa->ifa_netmask;
+			sin6 = (struct sockaddr_in6 *)(void *)ifa->ifa_addr;
+			net6 = (struct sockaddr_in6 *)(void *)ifa->ifa_netmask;
 #ifdef __KAME__
 			if (IN6_IS_ADDR_LINKLOCAL(&sin6->sin6_addr))
 				/* Remove the scope from the address */
 				sin6->sin6_addr.s6_addr[2] =
 				    sin6->sin6_addr.s6_addr[3] = '\0';
 #endif
-#ifndef HAVE_IFADDRS_ADDRFLAGS
-			addrflags = if_addrflags6(ifp, &sin6->sin6_addr,
-			    ifa->ifa_name);
-			if (addrflags == -1) {
-				if (errno != EEXIST)
-					logger(ctx, LOG_ERR,
-					    "%s: if_addrflags6:  %m", __func__);
-				continue;
-			}
-#endif
-			ipv6_handleifa(ctx, RTM_NEWADDR, ifs,
-			    ifa->ifa_name, &sin6->sin6_addr,
-			    ipv6_prefixlen(&net6->sin6_addr), addrflags);
+			ifa_flags = if_addrflags6(&sin6->sin6_addr, ifp);
+			if (ifa_flags != -1)
+				ipv6_handleifa(ctx, RTM_NEWADDR, ifs,
+				    ifa->ifa_name,
+				    &sin6->sin6_addr,
+				    ipv6_prefixlen(&net6->sin6_addr),
+				    ifa_flags);
 			break;
 #endif
 		}
@@ -267,21 +225,35 @@ struct if_head *
 if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 {
 	struct ifaddrs *ifaddrs, *ifa;
+	char *p;
 	int i;
-	unsigned int active;
 	struct if_head *ifs;
 	struct interface *ifp;
-	struct if_spec spec;
+#ifdef __linux__
+	char ifn[IF_NAMESIZE];
+#endif
 #ifdef AF_LINK
 	const struct sockaddr_dl *sdl;
 #ifdef SIOCGIFPRIORITY
 	struct ifreq ifr;
+	int s_inet;
 #endif
 #ifdef IFLR_ACTIVE
 	struct if_laddrreq iflr;
+	int s_link;
 #endif
 
+#ifdef SIOCGIFPRIORITY
+	if ((s_inet = socket(PF_INET, SOCK_DGRAM, 0)) == -1)
+		return NULL;
+#endif
 #ifdef IFLR_ACTIVE
+	if ((s_link = socket(PF_LINK, SOCK_DGRAM, 0)) == -1) {
+#ifdef SIOCGIFPRIORITY
+		close(s_inet);
+#endif
+		return NULL;
+	}
 	memset(&iflr, 0, sizeof(iflr));
 #endif
 #elif AF_PACKET
@@ -290,9 +262,9 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 
 	if (getifaddrs(&ifaddrs) == -1)
 		return NULL;
-
-	if ((ifs = malloc(sizeof(*ifs))) == NULL)
-		goto failed;
+	ifs = malloc(sizeof(*ifs));
+	if (ifs == NULL)
+		return NULL;
 	TAILQ_INIT(ifs);
 
 	for (ifa = ifaddrs; ifa; ifa = ifa->ifa_next) {
@@ -305,13 +277,11 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 				continue;
 #endif
 		}
-		if (if_nametospec(ifa->ifa_name, &spec) != 0)
-			continue;
 
 		/* It's possible for an interface to have >1 AF_LINK.
 		 * For our purposes, we use the first one. */
 		TAILQ_FOREACH(ifp, ifs, next) {
-			if (strcmp(ifp->name, spec.devname) == 0)
+			if (strcmp(ifp->name, ifa->ifa_name) == 0)
 				break;
 		}
 		if (ifp)
@@ -319,48 +289,58 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 
 		if (argc > 0) {
 			for (i = 0; i < argc; i++) {
-				if (strcmp(argv[i], spec.devname) == 0)
+#ifdef __linux__
+				/* Check the real interface name */
+				strlcpy(ifn, argv[i], sizeof(ifn));
+				p = strchr(ifn, ':');
+				if (p)
+					*p = '\0';
+				if (strcmp(ifn, ifa->ifa_name) == 0)
 					break;
+#else
+				if (strcmp(argv[i], ifa->ifa_name) == 0)
+					break;
+#endif
 			}
-			active = (i == argc) ? IF_INACTIVE : IF_ACTIVE_USER;
+			if (i == argc)
+				continue;
+			p = argv[i];
 		} else {
+			p = ifa->ifa_name;
+#ifdef __linux__
+			strlcpy(ifn, ifa->ifa_name, sizeof(ifn));
+#endif
 			/* -1 means we're discovering against a specific
 			 * interface, but we still need the below rules
 			 * to apply. */
-			if (argc == -1 && strcmp(argv[0], spec.devname) != 0)
+			if (argc == -1 && strcmp(argv[0], ifa->ifa_name) != 0)
 				continue;
-			active = ctx->options & DHCPCD_INACTIVE ?
-			    IF_INACTIVE: IF_ACTIVE_USER;
 		}
-
 		for (i = 0; i < ctx->ifdc; i++)
-			if (!fnmatch(ctx->ifdv[i], spec.devname, 0))
+			if (!fnmatch(ctx->ifdv[i], p, 0))
 				break;
 		if (i < ctx->ifdc)
-			active = IF_INACTIVE;
+			continue;
 		for (i = 0; i < ctx->ifac; i++)
-			if (!fnmatch(ctx->ifav[i], spec.devname, 0))
+			if (!fnmatch(ctx->ifav[i], p, 0))
 				break;
 		if (ctx->ifac && i == ctx->ifac)
-			active = IF_INACTIVE;
-
-#ifdef PLUGIN_DEV
-		/* Ensure that the interface name has settled */
-		if (!dev_initialized(ctx, spec.devname))
 			continue;
-#endif
+
+		/* Ensure that the interface name has settled */
+		if (!dev_initialized(ctx, p))
+			continue;
 
 		/* Don't allow loopback or pointopoint unless explicit */
 		if (ifa->ifa_flags & (IFF_LOOPBACK | IFF_POINTOPOINT)) {
 			if ((argc == 0 || argc == -1) &&
-			    ctx->ifac == 0 && !if_hasconf(ctx, spec.devname))
-				active = IF_INACTIVE;
+			    ctx->ifac == 0 && !if_hasconf(ctx, p))
+				continue;
 		}
 
-		if (if_vimaster(ctx, spec.devname) == 1) {
+		if (if_vimaster(p) == 1) {
 			logger(ctx, argc ? LOG_ERR : LOG_DEBUG,
-			    "%s: is a Virtual Interface Master, skipping",
-			    spec.devname);
+			    "%s: is a Virtual Interface Master, skipping", p);
 			continue;
 		}
 
@@ -370,12 +350,18 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 			break;
 		}
 		ifp->ctx = ctx;
-		strlcpy(ifp->name, spec.devname, sizeof(ifp->name));
+#ifdef __linux__
+		strlcpy(ifp->name, ifn, sizeof(ifp->name));
+		strlcpy(ifp->alias, p, sizeof(ifp->alias));
+#else
+		strlcpy(ifp->name, p, sizeof(ifp->name));
+#endif
 		ifp->flags = ifa->ifa_flags;
+		ifp->carrier = if_carrier(ifp);
 
 		if (ifa->ifa_addr != NULL) {
 #ifdef AF_LINK
-			sdl = (const void *)ifa->ifa_addr;
+			sdl = (const struct sockaddr_dl *)(void *)ifa->ifa_addr;
 
 #ifdef IFLR_ACTIVE
 			/* We need to check for active address */
@@ -385,7 +371,7 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 			    MIN(ifa->ifa_addr->sa_len, sizeof(iflr.addr)));
 			iflr.flags = IFLR_PREFIX;
 			iflr.prefixlen = (unsigned int)sdl->sdl_alen * NBBY;
-			if (ioctl(ctx->pf_link_fd, SIOCGLIFADDR, &iflr) == -1 ||
+			if (ioctl(s_link, SIOCGLIFADDR, &iflr) == -1 ||
 			    !(iflr.flags & IFLR_ACTIVE))
 			{
 				if_free(ifp);
@@ -407,7 +393,7 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 #if defined(IFT_BRIDGE) || defined(IFT_PPP) || defined(IFT_PROPVIRTUAL)
 				/* Don't allow unless explicit */
 				if ((argc == 0 || argc == -1) &&
-				    ctx->ifac == 0 && active &&
+				    ctx->ifac == 0 &&
 				    !if_hasconf(ctx, ifp->name))
 				{
 					logger(ifp->ctx, LOG_DEBUG,
@@ -415,7 +401,8 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 					    " interface type and"
 					    " no config",
 					    ifp->name);
-					active = IF_INACTIVE;
+					if_free(ifp);
+					continue;
 				}
 				/* FALLTHROUGH */
 #endif
@@ -443,23 +430,24 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 				if ((argc == 0 || argc == -1) &&
 				    ctx->ifac == 0 &&
 				    !if_hasconf(ctx, ifp->name))
-					active = IF_INACTIVE;
-				if (active)
-					logger(ifp->ctx, LOG_WARNING,
-					    "%s: unsupported"
-					    " interface type %.2x",
-					    ifp->name, sdl->sdl_type);
+				{
+					if_free(ifp);
+					continue;
+				}
+				logger(ifp->ctx, LOG_WARNING,
+				    "%s: unsupported interface type %.2x",
+				    ifp->name, sdl->sdl_type);
 				/* Pretend it's ethernet */
 				ifp->family = ARPHRD_ETHER;
 				break;
 			}
 			ifp->hwlen = sdl->sdl_alen;
 #ifndef CLLADDR
-#  define CLLADDR(s) (const void *)((s)->sdl_data + (s)->sdl_nlen)
+#  define CLLADDR(s) ((const char *)((s)->sdl_data + (s)->sdl_nlen))
 #endif
 			memcpy(ifp->hwaddr, CLLADDR(sdl), ifp->hwlen);
 #elif AF_PACKET
-			sll = (const void *)ifa->ifa_addr;
+			sll = (const struct sockaddr_ll *)(void *)ifa->ifa_addr;
 			ifp->index = (unsigned int)sll->sll_ifindex;
 			ifp->family = sll->sll_hatype;
 			ifp->hwlen = sll->sll_halen;
@@ -477,7 +465,10 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 		if (ifp->family != ARPHRD_ETHER) {
 			if ((argc == 0 || argc == -1) &&
 			    ctx->ifac == 0 && !if_hasconf(ctx, ifp->name))
-				active = IF_INACTIVE;
+			{
+				if_free(ifp);
+				continue;
+			}
 			switch (ifp->family) {
 			case ARPHRD_IEEE1394:
 			case ARPHRD_INFINIBAND:
@@ -493,43 +484,36 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 /* IFT already checked */
 #ifndef AF_LINK
 			default:
-				if (active)
-					logger(ifp->ctx, LOG_WARNING,
-					    "%s: unsupported"
-					    " interface family %.2x",
-					    ifp->name, ifp->family);
+				logger(ifp->ctx, LOG_WARNING,
+				    "%s: unsupported interface family %.2x",
+				    ifp->name, ifp->family);
 				break;
 #endif
 			}
 		}
 
-		if (!(ctx->options & (DHCPCD_DUMPLEASE | DHCPCD_TEST))) {
-			/* Handle any platform init for the interface */
-			if (active != IF_INACTIVE && if_init(ifp) == -1) {
-				logger(ifp->ctx, LOG_ERR, "%s: if_init: %m",
-				    ifp->name);
-				if_free(ifp);
-				continue;
-			}
+		/* Handle any platform init for the interface */
+		if (if_init(ifp) == -1) {
+			logger(ifp->ctx, LOG_ERR, "%s: if_init: %m", p);
+			if_free(ifp);
+			continue;
+		}
 
-			/* Ensure that the MTU is big enough for DHCP */
-			if (if_getmtu(ifp) < MTU_MIN && active &&
-			    if_setmtu(ifp, MTU_MIN) == -1)
-			{
-				logger(ifp->ctx, LOG_ERR,
-				    "%s: if_setmtu: %m", ifp->name);
-				if_free(ifp);
-				continue;
-			}
+		/* Ensure that the MTU is big enough for DHCP */
+		if (if_getmtu(ifp->name) < MTU_MIN &&
+		    if_setmtu(ifp->name, MTU_MIN) == -1)
+		{
+			logger(ifp->ctx, LOG_ERR, "%s: set_mtu: %m", p);
+			if_free(ifp);
+			continue;
 		}
 
 #ifdef SIOCGIFPRIORITY
 		/* Respect the interface priority */
 		memset(&ifr, 0, sizeof(ifr));
 		strlcpy(ifr.ifr_name, ifp->name, sizeof(ifr.ifr_name));
-		if (ioctl(ctx->pf_inet_fd, SIOCGIFPRIORITY, &ifr) == 0)
-			ifp->metric = (unsigned int)ifr.ifr_metric;
-		if_getssid(ifp);
+		if (ioctl(s_inet, SIOCGIFPRIORITY, &ifr) == 0)
+			ifp->metric = ifr.ifr_metric;
 #else
 		/* We reserve the 100 range for virtual interfaces, if and when
 		 * we can work them out. */
@@ -540,82 +524,42 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 		}
 #endif
 
-		ifp->active = active;
-		if (ifp->active)
-			ifp->carrier = if_carrier(ifp);
-		else
-			ifp->carrier = LINK_UNKNOWN;
 		TAILQ_INSERT_TAIL(ifs, ifp, next);
 	}
 
-	if_learnaddrs(ctx, ifs, ifaddrs);
-failed:
+	if_learnaddrs1(ctx, ifs, ifaddrs);
 	freeifaddrs(ifaddrs);
+
+#ifdef SIOCGIFPRIORITY
+	close(s_inet);
+#endif
+#ifdef IFLR_ACTIVE
+	close(s_link);
+#endif
+
 	return ifs;
 }
 
-/* Decode bge0:1 as dev = bge, ppa = 0 and lun = 1 */
-int
-if_nametospec(const char *ifname, struct if_spec *spec)
-{
-	char *ep;
-	int e;
-
-	if (ifname == NULL || *ifname == '\0' ||
-	    strlcpy(spec->ifname, ifname, sizeof(spec->ifname)) >=
-	    sizeof(spec->ifname) ||
-	    strlcpy(spec->drvname, ifname, sizeof(spec->drvname)) >=
-	    sizeof(spec->drvname))
-	{
-		errno = EINVAL;
-		return -1;
-	}
-	ep = strchr(spec->drvname, ':');
-	if (ep) {
-		spec->lun = (int)strtoi(ep + 1, NULL, 10, 0, INT_MAX, &e);
-		if (e != 0) {
-			errno = e;
-			return -1;
-		}
-		*ep-- = '\0';
-	} else {
-		spec->lun = -1;
-		ep = spec->drvname + strlen(spec->drvname) - 1;
-	}
-	strlcpy(spec->devname, spec->drvname, sizeof(spec->devname));
-	while (ep > spec->drvname && isdigit((int)*ep))
-		ep--;
-	if (*ep++ == ':') {
-		errno = EINVAL;
-		return -1;
-	}
-	spec->ppa = (int)strtoi(ep, NULL, 10, 0, INT_MAX, &e);
-	if (e != 0)
-		spec->ppa = -1;
-	*ep = '\0';
-
-	return 0;
-}
-
 static struct interface *
 if_findindexname(struct if_head *ifaces, unsigned int idx, const char *name)
 {
 
 	if (ifaces != NULL) {
-		struct if_spec spec;
 		struct interface *ifp;
 
-		if (name && if_nametospec(name, &spec) == -1)
-			return NULL;
-
 		TAILQ_FOREACH(ifp, ifaces, next) {
-			if ((name && strcmp(ifp->name, spec.devname) == 0) ||
-			    (!name && ifp->index == idx))
+			if ((ifp->options == NULL ||
+			    !(ifp->options->options & DHCPCD_PFXDLGONLY)) &&
+			    ((name && strcmp(ifp->name, name) == 0) ||
+#ifdef __linux__
+			    (name && strcmp(ifp->alias, name) == 0) ||
+#endif
+			    (!name && ifp->index == idx)))
 				return ifp;
 		}
 	}
 
-	errno = ENXIO;
+	errno = ESRCH;
 	return NULL;
 }
 
@@ -634,15 +578,18 @@ if_findindex(struct if_head *ifaces, unsigned int idx)
 }
 
 int
-if_domtu(const struct interface *ifp, short int mtu)
+if_domtu(const char *ifname, short int mtu)
 {
-	int r;
+	int s, r;
 	struct ifreq ifr;
 
+	if ((s = socket(PF_INET, SOCK_DGRAM, 0)) == -1)
+		return -1;
 	memset(&ifr, 0, sizeof(ifr));
-	strlcpy(ifr.ifr_name, ifp->name, sizeof(ifr.ifr_name));
+	strlcpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
 	ifr.ifr_mtu = mtu;
-	r = ioctl(ifp->ctx->pf_inet_fd, mtu ? SIOCSIFMTU : SIOCGIFMTU, &ifr);
+	r = ioctl(s, mtu ? SIOCSIFMTU : SIOCGIFMTU, &ifr);
+	close(s);
 	if (r == -1)
 		return -1;
 	return ifr.ifr_mtu;
@@ -656,13 +603,15 @@ if_cmp(const struct interface *si, const struct interface *ti)
 	int r;
 #endif
 
-	/* Check active first */
-	if (si->active > ti->active)
+	/* Always prefer master interfaces */
+	if (!(si->options->options & DHCPCD_PFXDLGONLY) &&
+	    ti->options->options & DHCPCD_PFXDLGONLY)
 		return -1;
-	if (si->active < ti->active)
+	if (si->options->options & DHCPCD_PFXDLGONLY &&
+	    !(ti->options->options & DHCPCD_PFXDLGONLY))
 		return 1;
 
-	/* Check carrier status next */
+	/* Check carrier status first */
 	if (si->carrier > ti->carrier)
 		return -1;
 	if (si->carrier < ti->carrier)
@@ -723,43 +672,3 @@ if_sortinterfaces(struct dhcpcd_ctx *ctx)
 	}
 	TAILQ_CONCAT(ctx->ifaces, &sorted, next);
 }
-
-int
-xsocket(int domain, int type, int protocol)
-{
-	int s;
-#if !defined(HAVE_SOCK_CLOEXEC) || !defined(HAVE_SOCK_NONBLOCK)
-	int xflags, xtype = type;
-#endif
-
-#ifndef HAVE_SOCK_CLOEXEC
-	if (xtype & SOCK_CLOEXEC)
-		type &= ~SOCK_CLOEXEC;
-#endif
-#ifndef HAVE_SOCK_NONBLOCK
-	if (xtype & SOCK_NONBLOCK)
-		type &= ~SOCK_NONBLOCK;
-#endif
-
-	if ((s = socket(domain, type, protocol)) == -1)
-		return -1;
-
-#ifndef HAVE_SOCK_CLOEXEC
-	if ((xtype & SOCK_CLOEXEC) && ((xflags = fcntl(s, F_GETFD)) == -1 ||
-	    fcntl(s, F_SETFD, xflags | FD_CLOEXEC) == -1))
-		goto out;
-#endif
-#ifndef HAVE_SOCK_NONBLOCK
-	if ((xtype & SOCK_NONBLOCK) && ((xflags = fcntl(s, F_GETFL)) == -1 ||
-	    fcntl(s, F_SETFL, xflags | O_NONBLOCK) == -1))
-		goto out;
-#endif
-
-	return s;
-
-#if !defined(HAVE_SOCK_CLOEXEC) || !defined(HAVE_SOCK_NONBLOCK)
-out:
-	close(s);
-	return -1;
-#endif
-}
