@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2016 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2015 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -28,16 +28,12 @@
 #include <sys/socket.h>
 #include <sys/types.h>
 
-#include <arpa/inet.h>
-#include <net/if.h>
-#include <net/route.h>
-#include <netinet/if_ether.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
+#include <net/route.h>
 
-#include <assert.h>
 #include <ctype.h>
 #include <errno.h>
-#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -50,13 +46,11 @@
 #include "if.h"
 #include "if-options.h"
 #include "ipv4.h"
-#include "ipv4ll.h"
 #include "script.h"
 
 #define IPV4_LOOPBACK_ROUTE
-#if defined(__linux__) || defined(__sun) || (defined(BSD) && defined(RTF_LOCAL))
-/* Linux has had loopback routes in the local table since 2.2
- * Solaris does not seem to support loopback routes. */
+#if defined(__linux__) || (defined(BSD) && defined(RTF_LOCAL))
+/* Linux has had loopback routes in the local table since 2.2 */
 #undef IPV4_LOOPBACK_ROUTE
 #endif
 
@@ -115,7 +109,7 @@ ipv4_getnetmask(uint32_t addr)
 
 struct ipv4_addr *
 ipv4_iffindaddr(struct interface *ifp,
-    const struct in_addr *addr, const struct in_addr *mask)
+    const struct in_addr *addr, const struct in_addr *net)
 {
 	struct ipv4_state *state;
 	struct ipv4_addr *ap;
@@ -124,7 +118,7 @@ ipv4_iffindaddr(struct interface *ifp,
 	if (state) {
 		TAILQ_FOREACH(ap, &state->addrs, next) {
 			if ((addr == NULL || ap->addr.s_addr == addr->s_addr) &&
-			    (mask == NULL || ap->mask.s_addr == mask->s_addr))
+			    (net == NULL || ap->net.s_addr == net->s_addr))
 				return ap;
 		}
 	}
@@ -147,23 +141,6 @@ ipv4_iffindlladdr(struct interface *ifp)
 	return NULL;
 }
 
-static struct ipv4_addr *
-ipv4_iffindmaskaddr(struct interface *ifp, const struct in_addr *addr)
-{
-	struct ipv4_state *state;
-	struct ipv4_addr *ap;
-
-	state = IPV4_STATE(ifp);
-	if (state) {
-		TAILQ_FOREACH (ap, &state->addrs, next) {
-			if ((ap->addr.s_addr & ap->mask.s_addr) ==
-			    (addr->s_addr & ap->mask.s_addr))
-				return ap;
-		}
-	}
-	return NULL;
-}
-
 struct ipv4_addr *
 ipv4_findaddr(struct dhcpcd_ctx *ctx, const struct in_addr *addr)
 {
@@ -178,32 +155,25 @@ ipv4_findaddr(struct dhcpcd_ctx *ctx, const struct in_addr *addr)
 	return NULL;
 }
 
-struct ipv4_addr *
-ipv4_findmaskaddr(struct dhcpcd_ctx *ctx, const struct in_addr *addr)
+int
+ipv4_addrexists(struct dhcpcd_ctx *ctx, const struct in_addr *addr)
 {
 	struct interface *ifp;
-	struct ipv4_addr *ap;
+	struct dhcp_state *state;
 
 	TAILQ_FOREACH(ifp, ctx->ifaces, next) {
-		ap = ipv4_iffindmaskaddr(ifp, addr);
-		if (ap)
-			return ap;
+		state = D_STATE(ifp);
+		if (state) {
+			if (addr == NULL) {
+				if (state->addr.s_addr != INADDR_ANY)
+					return 1;
+			} else if (addr->s_addr == state->addr.s_addr)
+				return 1;
+		}
+		if (addr != NULL && ipv4_iffindaddr(ifp, addr, NULL))
+			return 1;
 	}
-	return NULL;
-}
-
-int
-ipv4_hasaddr(const struct interface *ifp)
-{
-	const struct dhcp_state *dstate;
-
-	if (IPV4LL_STATE_RUNNING(ifp))
-		return 1;
-
-	dstate = D_CSTATE(ifp);
-	return (dstate &&
-	    dstate->added == STATE_ADDED &&
-	    dstate->addr != NULL);
+	return 0;
 }
 
 void
@@ -262,9 +232,11 @@ ipv4_ifcmp(const struct interface *si, const struct interface *ti)
 	/* If we are either, they neither have a lease, or they both have.
 	 * We need to check for IPv4LL and make it non-preferred. */
 	if (sis->new && tis->new) {
-		if (IS_DHCP(sis->new) && !IS_DHCP(tis->new))
+		int sill = (sis->new->cookie == htonl(MAGIC_COOKIE));
+		int till = (tis->new->cookie == htonl(MAGIC_COOKIE));
+		if (sill && !till)
 			return -1;
-		if (!IS_DHCP(sis->new) && IS_DHCP(tis->new))
+		if (!sill && till)
 			return 1;
 	}
 	return 0;
@@ -284,7 +256,7 @@ find_route(struct rt_head *rts, const struct rt *r, const struct rt *srt)
 		    rt->iface->metric == r->iface->metric)) &&
 #endif
                     (!srt || srt != rt) &&
-		    rt->mask.s_addr == r->mask.s_addr)
+		    rt->net.s_addr == r->net.s_addr)
 			return rt;
 	}
 	return NULL;
@@ -293,33 +265,28 @@ find_route(struct rt_head *rts, const struct rt *r, const struct rt *srt)
 static void
 desc_route(const char *cmd, const struct rt *rt)
 {
-	char addr[INET_ADDRSTRLEN];
+	char addr[sizeof("000.000.000.000") + 1];
 	struct dhcpcd_ctx *ctx = rt->iface ? rt->iface->ctx : NULL;
 	const char *ifname = rt->iface ? rt->iface->name : NULL;
 
 	strlcpy(addr, inet_ntoa(rt->dest), sizeof(addr));
-	if (rt->mask.s_addr == htonl(INADDR_BROADCAST) &&
+	if (rt->net.s_addr == htonl(INADDR_BROADCAST) &&
 	    rt->gate.s_addr == htonl(INADDR_ANY))
 		logger(ctx, LOG_INFO, "%s: %s host route to %s",
 		    ifname, cmd, addr);
-	else if (rt->mask.s_addr == htonl(INADDR_BROADCAST))
+	else if (rt->net.s_addr == htonl(INADDR_BROADCAST))
 		logger(ctx, LOG_INFO, "%s: %s host route to %s via %s",
 		    ifname, cmd, addr, inet_ntoa(rt->gate));
-	else if (rt->dest.s_addr == htonl(INADDR_ANY) &&
-	    rt->mask.s_addr == htonl(INADDR_ANY) &&
-	    rt->gate.s_addr == htonl(INADDR_ANY))
-		logger(ctx, LOG_INFO, "%s: %s default route",
-		    ifname, cmd);
 	else if (rt->gate.s_addr == htonl(INADDR_ANY))
 		logger(ctx, LOG_INFO, "%s: %s route to %s/%d",
-		    ifname, cmd, addr, inet_ntocidr(rt->mask));
+		    ifname, cmd, addr, inet_ntocidr(rt->net));
 	else if (rt->dest.s_addr == htonl(INADDR_ANY) &&
-	    rt->mask.s_addr == htonl(INADDR_ANY))
+	    rt->net.s_addr == htonl(INADDR_ANY))
 		logger(ctx, LOG_INFO, "%s: %s default route via %s",
 		    ifname, cmd, inet_ntoa(rt->gate));
 	else
 		logger(ctx, LOG_INFO, "%s: %s route to %s/%d via %s",
-		    ifname, cmd, addr, inet_ntocidr(rt->mask),
+		    ifname, cmd, addr, inet_ntocidr(rt->net),
 		    inet_ntoa(rt->gate));
 }
 
@@ -333,12 +300,12 @@ ipv4_findrt(struct dhcpcd_ctx *ctx, const struct rt *rt, int flags)
 	TAILQ_FOREACH(r, ctx->ipv4_kroutes, next) {
 		if (rt->dest.s_addr == r->dest.s_addr &&
 #ifdef HAVE_ROUTE_METRIC
-		    (rt->iface == NULL || rt->iface == r->iface) &&
+		    rt->iface == r->iface &&
 		    (!flags || rt->metric == r->metric) &&
 #else
 		    (!flags || rt->iface == r->iface) &&
 #endif
-		    rt->mask.s_addr == r->mask.s_addr)
+		    rt->net.s_addr == r->net.s_addr)
 			return r;
 	}
 	return NULL;
@@ -358,7 +325,7 @@ ipv4_freerts(struct rt_head *routes)
 /* If something other than dhcpcd removes a route,
  * we need to remove it from our internal table. */
 int
-ipv4_handlert(struct dhcpcd_ctx *ctx, int cmd, const struct rt *rt, int flags)
+ipv4_handlert(struct dhcpcd_ctx *ctx, int cmd, struct rt *rt)
 {
 	struct rt *f;
 
@@ -383,27 +350,13 @@ ipv4_handlert(struct dhcpcd_ctx *ctx, int cmd, const struct rt *rt, int flags)
 
 		/* If we manage the route, remove it */
 		if ((f = find_route(ctx->ipv4_routes, rt, NULL))) {
-			desc_route("deleted", f);
+			desc_route("removing", f);
 			TAILQ_REMOVE(ctx->ipv4_routes, f, next);
 			free(f);
 		}
 		break;
 	}
-
-	return flags ? 0 : ipv4ll_handlert(ctx, cmd, rt);
-}
-
-static void
-d_kroute(struct rt *rt)
-{
-	struct dhcpcd_ctx *ctx;
-
-	ctx = rt->iface->ctx;
-	rt = ipv4_findrt(ctx, rt, 1);
-	if (rt != NULL) {
-		TAILQ_REMOVE(ctx->ipv4_kroutes, rt, next);
-		free(rt);
-	}
+	return 0;
 }
 
 #define n_route(a)	 nc_route(NULL, a)
@@ -411,17 +364,15 @@ d_kroute(struct rt *rt)
 static int
 nc_route(struct rt *ort, struct rt *nrt)
 {
-	int change;
 
 	/* Don't set default routes if not asked to */
 	if (nrt->dest.s_addr == 0 &&
-	    nrt->mask.s_addr == 0 &&
+	    nrt->net.s_addr == 0 &&
 	    !(nrt->iface->options->options & DHCPCD_GATEWAY))
 		return -1;
 
 	desc_route(ort == NULL ? "adding" : "changing", nrt);
 
-	change = 0;
 	if (ort == NULL) {
 		ort = ipv4_findrt(nrt->iface->ctx, nrt, 0);
 		if (ort &&
@@ -431,45 +382,21 @@ nc_route(struct rt *ort, struct rt *nrt)
 		    ort->metric == nrt->metric &&
 #endif
 		    ort->gate.s_addr == nrt->gate.s_addr)))
-		{
-			if (ort->mtu == nrt->mtu)
-				return 0;
-			change = 1;
-		}
-	} else if (ort->state & STATE_FAKE && !(nrt->state & STATE_FAKE) &&
+			return 0;
+	} else if (ort->flags & STATE_FAKE && !(nrt->flags & STATE_FAKE) &&
 	    ort->iface == nrt->iface &&
 #ifdef HAVE_ROUTE_METRIC
 	    ort->metric == nrt->metric &&
 #endif
 	    ort->dest.s_addr == nrt->dest.s_addr &&
-	    ort->mask.s_addr ==  nrt->mask.s_addr &&
+	    ort->net.s_addr ==  nrt->net.s_addr &&
 	    ort->gate.s_addr == nrt->gate.s_addr)
-	{
-		if (ort->mtu == nrt->mtu)
-			return 0;
-		change = 1;
-	}
-
-#ifdef RTF_CLONING
-	/* BSD can set routes to be cloning routes.
-	 * Cloned routes inherit the parent flags.
-	 * As such, we need to delete and re-add the route to flush children
-	 * to correct the flags. */
-	if (change && ort != NULL && ort->flags & RTF_CLONING)
-		change = 0;
-#endif
-
-	if (change) {
-		if (if_route(RTM_CHANGE, nrt) != -1)
-			return 0;
-		if (errno != ESRCH)
-			logger(nrt->iface->ctx, LOG_ERR, "if_route (CHG): %m");
-	}
+		return 0;
 
 #ifdef HAVE_ROUTE_METRIC
 	/* With route metrics, we can safely add the new route before
 	 * deleting the old route. */
-	if (if_route(RTM_ADD, nrt) != -1) {
+	if (if_route(RTM_ADD, nrt)  == 0) {
 		if (ort && if_route(RTM_DELETE, ort) == -1 && errno != ESRCH)
 			logger(nrt->iface->ctx, LOG_ERR, "if_route (DEL): %m");
 		return 0;
@@ -483,27 +410,9 @@ nc_route(struct rt *ort, struct rt *nrt)
 
 	/* No route metrics, we need to delete the old route before
 	 * adding the new one. */
-#ifdef ROUTE_PER_GATEWAY
-	errno = 0;
-#endif
-	if (ort) {
-		if (if_route(RTM_DELETE, ort) == -1 && errno != ESRCH)
-			logger(nrt->iface->ctx, LOG_ERR, "if_route (DEL): %m");
-		else
-			d_kroute(ort);
-	}
-#ifdef ROUTE_PER_GATEWAY
-	/* The OS allows many routes to the same dest with different gateways.
-	 * dhcpcd does not support this yet, so for the time being just keep on
-	 * deleting the route until there is an error. */
-	if (ort && errno == 0) {
-		for (;;) {
-			if (if_route(RTM_DELETE, ort) == -1)
-				break;
-		}
-	}
-#endif
-	if (if_route(RTM_ADD, nrt) != -1)
+	if (ort && if_route(RTM_DELETE, ort) == -1 && errno != ESRCH)
+		logger(nrt->iface->ctx, LOG_ERR, "if_route (DEL): %m");
+	if (if_route(RTM_ADD, nrt) == 0)
 		return 0;
 #ifdef HAVE_ROUTE_METRIC
 logerr:
@@ -518,50 +427,40 @@ d_route(struct rt *rt)
 	int retval;
 
 	desc_route("deleting", rt);
-	retval = if_route(RTM_DELETE, rt) == -1 ? -1 : 0;
-	if (retval == -1 && errno != ENOENT && errno != ESRCH)
+	retval = if_route(RTM_DELETE, rt);
+	if (retval != 0 && errno != ENOENT && errno != ESRCH)
 		logger(rt->iface->ctx, LOG_ERR,
 		    "%s: if_delroute: %m", rt->iface->name);
-	/* Remove the route from our kernel table so we can add a
-	 * IPv4LL default route if possible. */
-	else
-		d_kroute(rt);
 	return retval;
 }
 
 static struct rt_head *
 add_subnet_route(struct rt_head *rt, const struct interface *ifp)
 {
-	const struct dhcp_state *state;
+	const struct dhcp_state *s;
 	struct rt *r;
 
 	if (rt == NULL) /* earlier malloc failed */
 		return NULL;
 
-	/* P2P interfaces don't have subnet routes as such. */
-	if (ifp->flags & IFF_POINTOPOINT)
-		return rt;
-
-	state = D_CSTATE(ifp);
+	s = D_CSTATE(ifp);
 	/* Don't create a subnet route for these addresses */
-	if (state->addr->mask.s_addr == INADDR_ANY)
+	if (s->net.s_addr == INADDR_ANY)
 		return rt;
 #ifndef BSD
 	/* BSD adds a route in this instance */
-	if (state->addr->mask.s_addr == INADDR_BROADCAST)
+	if (s->net.s_addr == INADDR_BROADCAST)
 		return rt;
 #endif
 
-	if ((r = calloc(1, sizeof(*r))) == NULL) {
+	if ((r = malloc(sizeof(*r))) == NULL) {
 		logger(ifp->ctx, LOG_ERR, "%s: %m", __func__);
 		ipv4_freeroutes(rt);
 		return NULL;
 	}
-	r->dest.s_addr = state->addr->addr.s_addr & state->addr->mask.s_addr;
-	r->mask.s_addr = state->addr->mask.s_addr;
+	r->dest.s_addr = s->addr.s_addr & s->net.s_addr;
+	r->net.s_addr = s->net.s_addr;
 	r->gate.s_addr = INADDR_ANY;
-	r->mtu = dhcp_get_mtu(ifp);
-	r->src = state->addr->addr;
 
 	TAILQ_INSERT_HEAD(rt, r, next);
 	return rt;
@@ -572,25 +471,24 @@ static struct rt_head *
 add_loopback_route(struct rt_head *rt, const struct interface *ifp)
 {
 	struct rt *r;
-	const struct dhcp_state *state;
+	const struct dhcp_state *s;
 
 	if (rt == NULL) /* earlier malloc failed */
 		return NULL;
 
-	state = D_CSTATE(ifp);
-	if (state->addr == NULL)
+	s = D_CSTATE(ifp);
+	if (s->addr.s_addr == INADDR_ANY)
 		return rt;
 
-	if ((r = calloc(1, sizeof(*r))) == NULL) {
+	r = malloc(sizeof(*r));
+	if (r == NULL) {
 		logger(ifp->ctx, LOG_ERR, "%s: %m", __func__);
 		ipv4_freeroutes(rt);
 		return NULL;
 	}
-	r->dest = state->addr->addr;
-	r->mask.s_addr = INADDR_BROADCAST;
+	r->dest.s_addr = s->addr.s_addr;
+	r->net.s_addr = INADDR_BROADCAST;
 	r->gate.s_addr = htonl(INADDR_LOOPBACK);
-	r->mtu = dhcp_get_mtu(ifp);
-	r->src = state->addr->addr;
 	TAILQ_INSERT_HEAD(rt, r, next);
 	return rt;
 }
@@ -601,16 +499,15 @@ get_routes(struct interface *ifp)
 {
 	struct rt_head *nrt;
 	struct rt *rt, *r = NULL;
-	const struct dhcp_state *state;
 
 	if (ifp->options->routes && TAILQ_FIRST(ifp->options->routes)) {
-		if ((nrt = malloc(sizeof(*nrt))) == NULL)
-			return NULL;
+		nrt = malloc(sizeof(*nrt));
 		TAILQ_INIT(nrt);
 		TAILQ_FOREACH(rt, ifp->options->routes, next) {
 			if (rt->gate.s_addr == 0)
 				break;
-			if ((r = calloc(1, sizeof(*r))) == NULL) {
+			r = malloc(sizeof(*r));
+			if (r == NULL) {
 				logger(ifp->ctx, LOG_ERR, "%s: %m", __func__);
 				ipv4_freeroutes(nrt);
 				return NULL;
@@ -618,42 +515,53 @@ get_routes(struct interface *ifp)
 			memcpy(r, rt, sizeof(*r));
 			TAILQ_INSERT_TAIL(nrt, r, next);
 		}
-	} else
-		nrt = dhcp_get_routes(ifp);
-
-	/* Copy our address as the source address */
-	if (nrt) {
-		state = D_CSTATE(ifp);
-		TAILQ_FOREACH(rt, nrt, next) {
-			rt->src = state->addr->addr;
-		}
+		return nrt;
 	}
 
-	return nrt;
+	return get_option_routes(ifp, D_STATE(ifp)->new);
 }
 
+/* Some DHCP servers add set host routes by setting the gateway
+ * to the assigned IP address or the destination address.
+ * We need to change this. */
+static struct rt_head *
+massage_host_routes(struct rt_head *rt, const struct interface *ifp)
+{
+	struct rt *r;
+
+	if (rt) {
+		TAILQ_FOREACH(r, rt, next) {
+			if (r->gate.s_addr == D_CSTATE(ifp)->addr.s_addr ||
+			    r->gate.s_addr == r->dest.s_addr)
+			{
+				r->gate.s_addr = htonl(INADDR_ANY);
+				r->net.s_addr = htonl(INADDR_BROADCAST);
+			}
+		}
+	}
+	return rt;
+}
+
+
 static struct rt_head *
 add_destination_route(struct rt_head *rt, const struct interface *ifp)
 {
 	struct rt *r;
-	const struct dhcp_state *state;
 
 	if (rt == NULL || /* failed a malloc earlier probably */
 	    !(ifp->flags & IFF_POINTOPOINT) ||
-	    !has_option_mask(ifp->options->dstmask, DHO_ROUTER) ||
-	    (state = D_CSTATE(ifp)) == NULL)
+	    !has_option_mask(ifp->options->dstmask, DHO_ROUTER))
 		return rt;
 
-	if ((r = calloc(1, sizeof(*r))) == NULL) {
+	r = malloc(sizeof(*r));
+	if (r == NULL) {
 		logger(ifp->ctx, LOG_ERR, "%s: %m", __func__);
 		ipv4_freeroutes(rt);
 		return NULL;
 	}
 	r->dest.s_addr = INADDR_ANY;
-	r->mask.s_addr = INADDR_ANY;
-	r->gate = state->addr->brd;
-	r->mtu = dhcp_get_mtu(ifp);
-	r->src = state->addr->addr;
+	r->net.s_addr = INADDR_ANY;
+	r->gate.s_addr = D_CSTATE(ifp)->dst.s_addr;
 	TAILQ_INSERT_HEAD(rt, r, next);
 	return rt;
 }
@@ -671,13 +579,8 @@ add_router_host_route(struct rt_head *rt, const struct interface *ifp)
 	if (rt == NULL) /* earlier malloc failed */
 		return NULL;
 
-	/* Don't add a host route for these interfaces. */
-	if (ifp->flags & (IFF_LOOPBACK | IFF_POINTOPOINT))
-		return rt;
-
 	TAILQ_FOREACH(rtp, rt, next) {
-		if (rtp->dest.s_addr != INADDR_ANY ||
-		    rtp->gate.s_addr == INADDR_ANY)
+		if (rtp->dest.s_addr != INADDR_ANY)
 			continue;
 		/* Scan for a route to match */
 		TAILQ_FOREACH(rtn, rt, next) {
@@ -689,8 +592,8 @@ add_router_host_route(struct rt_head *rt, const struct interface *ifp)
 			/* match subnet */
 			cp = (const char *)&rtp->gate.s_addr;
 			cp2 = (const char *)&rtn->dest.s_addr;
-			cp3 = (const char *)&rtn->mask.s_addr;
-			cplim = cp3 + sizeof(rtn->mask.s_addr);
+			cp3 = (const char *)&rtn->net.s_addr;
+			cplim = cp3 + sizeof(rtn->net.s_addr);
 			while (cp3 < cplim) {
 				if ((*cp++ ^ *cp2++) & *cp3++)
 					break;
@@ -700,8 +603,7 @@ add_router_host_route(struct rt_head *rt, const struct interface *ifp)
 		}
 		if (rtn != rtp)
 			continue;
-		if ((state = D_CSTATE(ifp)) == NULL)
-			continue;
+		state = D_CSTATE(ifp);
 		ifo = ifp->options;
 		if (ifp->flags & IFF_NOARP) {
 			if (!(ifo->options & DHCPCD_ROUTER_HOST_ROUTE_WARNED) &&
@@ -723,113 +625,46 @@ add_router_host_route(struct rt_head *rt, const struct interface *ifp)
 			    "%s: router %s requires a host route",
 			    ifp->name, inet_ntoa(rtp->gate));
 		}
-		if ((rtn = calloc(1, sizeof(*rtn))) == NULL) {
+		rtn = malloc(sizeof(*rtn));
+		if (rtn == NULL) {
 			logger(ifp->ctx, LOG_ERR, "%s: %m", __func__);
 			ipv4_freeroutes(rt);
 			return NULL;
 		}
 		rtn->dest.s_addr = rtp->gate.s_addr;
-		rtn->mask.s_addr = htonl(INADDR_BROADCAST);
+		rtn->net.s_addr = htonl(INADDR_BROADCAST);
 		rtn->gate.s_addr = htonl(INADDR_ANY);
-		rtn->mtu = dhcp_get_mtu(ifp);
-		rtn->src = state->addr->addr;
 		TAILQ_INSERT_BEFORE(rtp, rtn, next);
 	}
 	return rt;
 }
 
-static int
-ipv4_doroute(struct rt *rt, struct rt_head *nrs)
-{
-	const struct dhcp_state *state;
-	struct rt *or;
-
-	state = D_CSTATE(rt->iface);
-	rt->state = state->added & STATE_FAKE;
-#ifdef HAVE_ROUTE_METRIC
-	rt->metric = rt->iface->metric;
-#endif
-	/* Is this route already in our table? */
-	if ((find_route(nrs, rt, NULL)) != NULL)
-		return 0;
-	/* Do we already manage it? */
-	if ((or = find_route(rt->iface->ctx->ipv4_routes, rt, NULL))) {
-		if (state->added & STATE_FAKE)
-			return 0;
-		if (or->state & STATE_FAKE ||
-		    or->iface != rt->iface ||
-#ifdef HAVE_ROUTE_METRIC
-		    or->metric != rt->metric ||
-#endif
-		    or->gate.s_addr != rt->gate.s_addr ||
-		    or->src.s_addr != rt->src.s_addr ||
-		    or->mtu != rt->mtu)
-		{
-			if (c_route(or, rt) != 0)
-				return 0;
-		}
-		TAILQ_REMOVE(rt->iface->ctx->ipv4_routes, or, next);
-		free(or);
-	} else {
-		if (state->added & STATE_FAKE) {
-			if ((or = ipv4_findrt(rt->iface->ctx, rt, 1)) == NULL)
-				return 0;
-			rt->iface = or->iface;
-			rt->gate.s_addr = or->gate.s_addr;
-#ifdef HAVE_ROUTE_METRIC
-			rt->metric = or->metric;
-#endif
-			rt->mtu = or->mtu;
-			rt->flags = or->flags;
-		} else {
-			if (n_route(rt) != 0)
-				return 0;
-		}
-	}
-	return 1;
-}
-
 void
 ipv4_buildroutes(struct dhcpcd_ctx *ctx)
 {
 	struct rt_head *nrs, *dnr;
-	struct rt *rt, *rtn;
+	struct rt *or, *rt, *rtn;
 	struct interface *ifp;
 	const struct dhcp_state *state;
-	int has_default;
 
 	/* We need to have the interfaces in the correct order to ensure
 	 * our routes are managed correctly. */
 	if_sortinterfaces(ctx);
 
-	if ((nrs = malloc(sizeof(*nrs))) == NULL) {
+	nrs = malloc(sizeof(*nrs));
+	if (nrs == NULL) {
 		logger(ctx, LOG_ERR, "%s: %m", __func__);
 		return;
 	}
 	TAILQ_INIT(nrs);
 
-	has_default = 0;
 	TAILQ_FOREACH(ifp, ctx->ifaces, next) {
 		state = D_CSTATE(ifp);
-		if (state != NULL && state->new != NULL && state->added) {
-			dnr = get_routes(ifp);
-			dnr = add_subnet_route(dnr, ifp);
-		} else
-			dnr = NULL;
-		if ((rt = ipv4ll_subnet_route(ifp)) != NULL) {
-			if (dnr == NULL) {
-				if ((dnr = malloc(sizeof(*dnr))) == NULL) {
-					logger(ifp->ctx, LOG_ERR,
-					    "%s: malloc %m", __func__);
-					free(rt);
-					continue;
-				}
-				TAILQ_INIT(dnr);
-			}
-			TAILQ_INSERT_HEAD(dnr, rt, next);
-		}
-		if (dnr == NULL)
+		if (state == NULL || state->new == NULL || !state->added)
 			continue;
+		dnr = get_routes(ifp);
+		dnr = massage_host_routes(dnr, ifp);
+		dnr = add_subnet_route(dnr, ifp);
 #ifdef IPV4_LOOPBACK_ROUTE
 		dnr = add_loopback_route(dnr, ifp);
 #endif
@@ -837,42 +672,56 @@ ipv4_buildroutes(struct dhcpcd_ctx *ctx)
 			dnr = add_router_host_route(dnr, ifp);
 			dnr = add_destination_route(dnr, ifp);
 		}
-		if (dnr == NULL)
+		if (dnr == NULL) /* failed to malloc all new routes */
 			continue;
 		TAILQ_FOREACH_SAFE(rt, dnr, next, rtn) {
 			rt->iface = ifp;
-			if (ipv4_doroute(rt, nrs) == 1) {
-				TAILQ_REMOVE(dnr, rt, next);
-				TAILQ_INSERT_TAIL(nrs, rt, next);
-				if (rt->dest.s_addr == INADDR_ANY)
-					has_default = 1;
+#ifdef HAVE_ROUTE_METRIC
+			rt->metric = ifp->metric;
+#endif
+			rt->flags = state->added & STATE_FAKE;
+			/* Is this route already in our table? */
+			if ((find_route(nrs, rt, NULL)) != NULL)
+				continue;
+			rt->src.s_addr = state->addr.s_addr;
+			/* Do we already manage it? */
+			if ((or = find_route(ctx->ipv4_routes, rt, NULL))) {
+				if (state->added & STATE_FAKE)
+					continue;
+				if (or->flags & STATE_FAKE ||
+				    or->iface != ifp ||
+#ifdef HAVE_ROUTE_METRIC
+				    rt->metric != or->metric ||
+#endif
+				    or->src.s_addr != state->addr.s_addr ||
+				    rt->gate.s_addr != or->gate.s_addr)
+				{
+					if (c_route(or, rt) != 0)
+						continue;
+				}
+				TAILQ_REMOVE(ctx->ipv4_routes, or, next);
+				free(or);
+			} else {
+				if (state->added & STATE_FAKE) {
+					or = ipv4_findrt(ctx, rt, 1);
+					if (or == NULL ||
+					    or->gate.s_addr != rt->gate.s_addr)
+ 						continue;
+				} else {
+					if (n_route(rt) != 0)
+						continue;
+				}
 			}
+			rt->flags |= STATE_ADDED;
+			TAILQ_REMOVE(dnr, rt, next);
+			TAILQ_INSERT_TAIL(nrs, rt, next);
 		}
 		ipv4_freeroutes(dnr);
 	}
 
-	/* If there is no default route, grab one without a
-	 * gateway for any IPv4LL enabled interfaces. */
-	if (!has_default) {
-		struct rt def;
-
-		memset(&def, 0, sizeof(def));
-		if (ipv4_findrt(ctx, &def, 0) == NULL) {
-			TAILQ_FOREACH(ifp, ctx->ifaces, next) {
-				if ((rt = ipv4ll_default_route(ifp)) != NULL) {
-					if (ipv4_doroute(rt, nrs) == 1) {
-						TAILQ_INSERT_TAIL(nrs, rt, next);
-						break;
-					} else
-						free(rt);
-				}
-			}
-		}
-	}
-
 	/* Remove old routes we used to manage */
 	if (ctx->ipv4_routes) {
-		TAILQ_FOREACH_REVERSE(rt, ctx->ipv4_routes, rt_head, next) {
+		TAILQ_FOREACH(rt, ctx->ipv4_routes, next) {
 			if (find_route(nrs, rt, NULL) == NULL &&
 			    (rt->iface->options->options &
 			    (DHCPCD_EXITING | DHCPCD_PERSISTENT)) !=
@@ -885,43 +734,41 @@ ipv4_buildroutes(struct dhcpcd_ctx *ctx)
 }
 
 int
-ipv4_deladdr(struct ipv4_addr *addr, int keeparp)
+ipv4_deladdr(struct interface *ifp,
+    const struct in_addr *addr, const struct in_addr *net)
 {
+	struct dhcp_state *dstate;
 	int r;
 	struct ipv4_state *state;
 	struct ipv4_addr *ap;
-	struct arp_state *astate;
 
-	logger(addr->iface->ctx, LOG_DEBUG,
-	    "%s: deleting IP address %s", addr->iface->name, addr->saddr);
+	logger(ifp->ctx, LOG_DEBUG, "%s: deleting IP address %s/%d",
+	    ifp->name, inet_ntoa(*addr), inet_ntocidr(*net));
 
-	r = if_address(RTM_DELADDR, addr);
-	if (r == -1 &&
-	    errno != EADDRNOTAVAIL && errno != ESRCH &&
-	    errno != ENXIO && errno != ENODEV)
-		logger(addr->iface->ctx, LOG_ERR, "%s: %s: %m",
-		    addr->iface->name, __func__);
+	r = if_deladdress(ifp, addr, net);
+	if (r == -1 && errno != EADDRNOTAVAIL && errno != ENXIO &&
+	    errno != ENODEV)
+		logger(ifp->ctx, LOG_ERR, "%s: %s: %m", ifp->name, __func__);
 
-	if (!keeparp && (astate = arp_find(addr->iface, &addr->addr)) != NULL)
-		arp_free(astate);
+	dstate = D_STATE(ifp);
+	if (dstate->addr.s_addr == addr->s_addr &&
+	    dstate->net.s_addr == net->s_addr)
+	{
+		dstate->added = 0;
+		dstate->addr.s_addr = 0;
+		dstate->net.s_addr = 0;
+	}
 
-	state = IPV4_STATE(addr->iface);
+	state = IPV4_STATE(ifp);
 	TAILQ_FOREACH(ap, &state->addrs, next) {
-		if (IPV4_MASK_EQ(ap, addr)) {
-			struct dhcp_state *dstate;
-
-			dstate = D_STATE(ap->iface);
+		if (ap->addr.s_addr == addr->s_addr &&
+		    ap->net.s_addr == net->s_addr)
+		{
 			TAILQ_REMOVE(&state->addrs, ap, next);
 			free(ap);
-
-			if (dstate && dstate->addr == ap) {
-				dstate->added = 0;
-				dstate->addr = NULL;
-			}
 			break;
 		}
 	}
-
 	return r;
 }
 
@@ -934,17 +781,14 @@ delete_address(struct interface *ifp)
 
 	state = D_STATE(ifp);
 	ifo = ifp->options;
-	/* The lease could have been added, but the address deleted
-	 * by a 3rd party. */
-	if (state->addr == NULL ||
-	    ifo->options & DHCPCD_INFORM ||
+	if (ifo->options & DHCPCD_INFORM ||
 	    (ifo->options & DHCPCD_STATIC && ifo->req_addr.s_addr == 0))
 		return 0;
-	r = ipv4_deladdr(state->addr, 0);
+	r = ipv4_deladdr(ifp, &state->addr, &state->net);
 	return r;
 }
 
-struct ipv4_state *
+static struct ipv4_state *
 ipv4_getstate(struct interface *ifp)
 {
 	struct ipv4_state *state;
@@ -959,185 +803,89 @@ ipv4_getstate(struct interface *ifp)
 		}
 		TAILQ_INIT(&state->addrs);
 		TAILQ_INIT(&state->routes);
-#ifdef BSD
-		state->buffer_size = state->buffer_len = state->buffer_pos = 0;
-		state->buffer = NULL;
-#endif
 	}
 	return state;
 }
 
-#ifdef ALIAS_ADDR
-/* Find the next logical aliase address we can use. */
 static int
-ipv4_aliasaddr(struct ipv4_addr *ia, struct ipv4_addr **repl)
-{
-	struct ipv4_state *state;
-	struct ipv4_addr *iap;
-	unsigned int lun;
-	char alias[IF_NAMESIZE];
-
-	if (ia->alias[0] != '\0')
-		return 0;
-
-	lun = 0;
-	state = IPV4_STATE(ia->iface);
-find_lun:
-	if (lun == 0)
-		strlcpy(alias, ia->iface->name, sizeof(alias));
-	else
-		snprintf(alias, sizeof(alias), "%s:%u", ia->iface->name, lun);
-	TAILQ_FOREACH(iap, &state->addrs, next) {
-		if (iap->alias[0] != '\0' && iap->addr.s_addr == INADDR_ANY) {
-			/* No address assigned? Lets use it. */
-			strlcpy(ia->alias, iap->alias, sizeof(ia->alias));
-			if (repl)
-				*repl = iap;
-			return 1;
-		}
-		if (strcmp(iap->alias, alias) == 0)
-			break;
-	}
-
-	if (iap != NULL) {
-		if (lun == UINT_MAX) {
-			errno = ERANGE;
-			return -1;
-		}
-		lun++;
-		goto find_lun;
-	}
-
-	strlcpy(ia->alias, alias, sizeof(ia->alias));
-	return 0;
-}
-#endif
-
-struct ipv4_addr *
-ipv4_addaddr(struct interface *ifp, const struct in_addr *addr,
-    const struct in_addr *mask, const struct in_addr *bcast)
+ipv4_addaddr(struct interface *ifp, const struct dhcp_lease *lease)
 {
 	struct ipv4_state *state;
 	struct ipv4_addr *ia;
-#ifdef ALIAS_ADDR
-	int replaced, blank;
-	struct ipv4_addr *replaced_ia;
-#endif
 
 	if ((state = ipv4_getstate(ifp)) == NULL) {
 		logger(ifp->ctx, LOG_ERR, "%s: ipv4_getstate: %m", __func__);
-		return NULL;
+		return -1;
 	}
 	if (ifp->options->options & DHCPCD_NOALIAS) {
 		struct ipv4_addr *ian;
 
 		TAILQ_FOREACH_SAFE(ia, &state->addrs, next, ian) {
-			if (ia->addr.s_addr != addr->s_addr)
-				ipv4_deladdr(ia, 0);
+			if (ia->addr.s_addr != lease->addr.s_addr)
+				ipv4_deladdr(ifp, &ia->addr, &ia->net);
 		}
 	}
 
 	if ((ia = malloc(sizeof(*ia))) == NULL) {
 		logger(ifp->ctx, LOG_ERR, "%s: %m", __func__);
-		return NULL;
+		return -1;
 	}
 
-	ia->iface = ifp;
-	ia->addr = *addr;
-	ia->mask = *mask;
-	ia->brd = *bcast;
-#ifdef IN_IFF_TENTATIVE
-	ia->addr_flags = IN_IFF_TENTATIVE;
-#endif
-	snprintf(ia->saddr, sizeof(ia->saddr), "%s/%d",
-	    inet_ntoa(*addr), inet_ntocidr(*mask));
-
-#ifdef ALIAS_ADDR
-	blank = (ia->alias[0] == '\0');
-	if ((replaced = ipv4_aliasaddr(ia, &replaced_ia)) == -1) {
-		logger(ifp->ctx, LOG_ERR, "%s: ipv4_aliasaddr: %m", ifp->name);
-		free(ia);
-		return NULL;
-	}
-	if (blank)
-		logger(ia->iface->ctx, LOG_DEBUG, "%s: aliased %s",
-		    ia->alias, ia->saddr);
-#endif
-
-	logger(ifp->ctx, LOG_DEBUG, "%s: adding IP address %s broadcast %s",
-	    ifp->name, ia->saddr, inet_ntoa(*bcast));
-	if (if_address(RTM_NEWADDR, ia) == -1) {
+	logger(ifp->ctx, LOG_DEBUG, "%s: adding IP address %s/%d",
+	    ifp->name, inet_ntoa(lease->addr),
+	    inet_ntocidr(lease->net));
+	if (if_addaddress(ifp, &lease->addr, &lease->net, &lease->brd) == -1) {
 		if (errno != EEXIST)
 			logger(ifp->ctx, LOG_ERR, "%s: if_addaddress: %m",
 			    __func__);
 		free(ia);
-		return NULL;
-	}
-
-#ifdef ALIAS_ADDR
-	if (replaced) {
-		TAILQ_REMOVE(&state->addrs, replaced_ia, next);
-		free(replaced_ia);
-	}
-#endif
-
-	TAILQ_INSERT_TAIL(&state->addrs, ia, next);
-	return ia;
-}
-
-static int
-ipv4_daddaddr(struct interface *ifp, const struct dhcp_lease *lease)
-{
-	struct dhcp_state *state;
-	struct ipv4_addr *ia;
-
-	ia = ipv4_addaddr(ifp, &lease->addr, &lease->mask, &lease->brd);
-	if (ia == NULL)
 		return -1;
+	}
 
-	state = D_STATE(ifp);
-	state->added = STATE_ADDED;
-	state->addr = ia;
+	ia->iface = ifp;
+	ia->addr = lease->addr;
+	ia->net = lease->net;
+#ifdef IN_IFF_TENTATIVE
+	ia->addr_flags = IN_IFF_TENTATIVE;
+#endif
+	TAILQ_INSERT_TAIL(&state->addrs, ia, next);
 	return 0;
 }
 
-int
-ipv4_preferanother(struct interface *ifp)
+static void
+ipv4_finalisert(struct interface *ifp)
 {
-	struct dhcp_state *state = D_STATE(ifp), *nstate;
-	struct interface *ifn;
-	int preferred;
+	const struct dhcp_state *state = D_CSTATE(ifp);
 
-	if (state == NULL)
-		return 0;
-
-	preferred = 0;
-	if (!state->added)
-		goto out;
-
-	TAILQ_FOREACH(ifn, ifp->ctx->ifaces, next) {
-		if (ifn == ifp)
-			break; /* We are already the most preferred */
-		nstate = D_STATE(ifn);
-		if (nstate && !nstate->added &&
-		    state->addr != NULL &&
-		    nstate->lease.addr.s_addr == state->addr->addr.s_addr)
-		{
-			preferred = 1;
-			delete_address(ifp);
-			if (ifn->options->options & DHCPCD_ARP)
-				dhcp_bind(ifn);
-			else {
-				ipv4_daddaddr(ifn, &nstate->lease);
-				nstate->added = STATE_ADDED;
-			}
-			break;
-		}
-	}
-
-out:
+	/* Find any freshly added routes, such as the subnet route.
+	 * We do this because we cannot rely on recieving the kernel
+	 * notification right now via our link socket. */
+	if_initrt(ifp);
 	ipv4_buildroutes(ifp->ctx);
-	return preferred;
+	script_runreason(ifp, state->reason);
+
+	dhcpcd_daemonise(ifp->ctx);
+}
+
+void
+ipv4_finaliseaddr(struct interface *ifp)
+{
+	struct dhcp_state *state = D_STATE(ifp);
+	struct dhcp_lease *lease;
+
+	lease = &state->lease;
+
+	/* Delete the old address if different */
+	if (state->addr.s_addr != lease->addr.s_addr &&
+	    state->addr.s_addr != 0 &&
+	    ipv4_iffindaddr(ifp, &lease->addr, NULL))
+		delete_address(ifp);
+
+	state->added = STATE_ADDED;
+	state->defend = 0;
+	state->addr.s_addr = lease->addr.s_addr;
+	state->net.s_addr = lease->net.s_addr;
+	ipv4_finalisert(ifp);
 }
 
 void
@@ -1145,25 +893,49 @@ ipv4_applyaddr(void *arg)
 {
 	struct interface *ifp = arg, *ifn;
 	struct dhcp_state *state = D_STATE(ifp), *nstate;
+	struct dhcp_message *dhcp;
 	struct dhcp_lease *lease;
 	struct if_options *ifo = ifp->options;
-	struct ipv4_addr *ia;
+	struct ipv4_addr *ap;
 	int r;
 
 	if (state == NULL)
 		return;
-
+	dhcp = state->new;
 	lease = &state->lease;
-	if_sortinterfaces(ifp->ctx);
 
-	if (state->new == NULL) {
+	if (dhcp == NULL) {
 		if ((ifo->options & (DHCPCD_EXITING | DHCPCD_PERSISTENT)) !=
 		    (DHCPCD_EXITING | DHCPCD_PERSISTENT))
 		{
-			if (state->added && !ipv4_preferanother(ifp)) {
+			if (state->added) {
+				struct in_addr addr;
+
+				addr = state->addr;
 				delete_address(ifp);
-				ipv4_buildroutes(ifp->ctx);
+				TAILQ_FOREACH(ifn, ifp->ctx->ifaces, next) {
+					if (ifn == ifp ||
+					    strcmp(ifn->name, ifp->name) == 0)
+						continue;
+					nstate = D_STATE(ifn);
+					if (nstate && !nstate->added &&
+					    nstate->addr.s_addr == addr.s_addr)
+					{
+						if (ifn->options->options &
+						    DHCPCD_ARP)
+						{
+							dhcp_bind(ifn, NULL);
+						} else {
+							ipv4_addaddr(ifn,
+							    &nstate->lease);
+							nstate->added =
+							    STATE_ADDED;
+						}
+						break;
+					}
+				}
 			}
+			ipv4_buildroutes(ifp->ctx);
 			script_runreason(ifp, state->reason);
 		} else
 			ipv4_buildroutes(ifp->ctx);
@@ -1171,30 +943,29 @@ ipv4_applyaddr(void *arg)
 	}
 
 	/* Ensure only one interface has the address */
-	r = 0;
 	TAILQ_FOREACH(ifn, ifp->ctx->ifaces, next) {
-		if (ifn == ifp) {
-			r = 1; /* past ourselves */
+		if (ifn == ifp || strcmp(ifn->name, ifp->name) == 0)
 			continue;
-		}
 		nstate = D_STATE(ifn);
 		if (nstate && nstate->added &&
-		    nstate->addr &&
-		    nstate->addr->addr.s_addr == lease->addr.s_addr)
+		    nstate->addr.s_addr == lease->addr.s_addr)
 		{
-			if (r == 0) {
+			if (ifn->metric <= ifp->metric) {
 				logger(ifp->ctx, LOG_INFO,
 				    "%s: preferring %s on %s",
 				    ifp->name,
 				    inet_ntoa(lease->addr),
 				    ifn->name);
-				return;
+				state->addr.s_addr = lease->addr.s_addr;
+				state->net.s_addr = lease->net.s_addr;
+				ipv4_finalisert(ifp);
 			}
 			logger(ifp->ctx, LOG_INFO, "%s: preferring %s on %s",
 			    ifn->name,
 			    inet_ntoa(lease->addr),
 			    ifp->name);
-			ipv4_deladdr(nstate->addr, 0);
+			ipv4_deladdr(ifn, &nstate->addr, &nstate->net);
+			nstate->added = 0;
 			break;
 		}
 	}
@@ -1202,83 +973,52 @@ ipv4_applyaddr(void *arg)
 	/* Does another interface already have the address from a prior boot? */
 	if (ifn == NULL) {
 		TAILQ_FOREACH(ifn, ifp->ctx->ifaces, next) {
-			if (ifn == ifp)
+			if (ifn == ifp || strcmp(ifn->name, ifp->name) == 0)
 				continue;
-			ia = ipv4_iffindaddr(ifn, &lease->addr, NULL);
-			if (ia != NULL)
-				ipv4_deladdr(ia, 0);
+			ap = ipv4_iffindaddr(ifn, &lease->addr, NULL);
+			if (ap)
+				ipv4_deladdr(ifn, &ap->addr, &ap->net);
 		}
 	}
 
-	/* If the netmask or broadcast is different, re-add the addresss */
-	ia = ipv4_iffindaddr(ifp, &lease->addr, NULL);
-	if (ia &&
-	    ia->mask.s_addr == lease->mask.s_addr &&
-	    ia->brd.s_addr == lease->brd.s_addr)
+	/* If the netmask is different, delete the addresss */
+	ap = ipv4_iffindaddr(ifp, &lease->addr, NULL);
+	if (ap && ap->net.s_addr != lease->net.s_addr)
+		ipv4_deladdr(ifp, &ap->addr, &ap->net);
+
+	if (ipv4_iffindaddr(ifp, &lease->addr, &lease->net))
 		logger(ifp->ctx, LOG_DEBUG,
-		    "%s: IP address %s already exists",
-		    ifp->name, ia->saddr);
+		    "%s: IP address %s/%d already exists",
+		    ifp->name, inet_ntoa(lease->addr),
+		    inet_ntocidr(lease->net));
 	else {
-#if __linux__
-		/* Linux does not change netmask/broadcast address
-		 * for re-added addresses, so we need to delete the old one
-		 * first. */
-		if (ia != NULL)
-			ipv4_deladdr(ia, 0);
-#endif
-		r = ipv4_daddaddr(ifp, lease);
+		r = ipv4_addaddr(ifp, lease);
 		if (r == -1 && errno != EEXIST)
 			return;
 	}
 
-	ia = ipv4_iffindaddr(ifp, &lease->addr, NULL);
-	if (ia == NULL) {
+#ifdef IN_IFF_NOTUSEABLE
+	ap = ipv4_iffindaddr(ifp, &lease->addr, NULL);
+	if (ap == NULL) {
 		logger(ifp->ctx, LOG_ERR, "%s: added address vanished",
 		    ifp->name);
 		return;
-	}
-#ifdef IN_IFF_NOTUSEABLE
-	if (ia->addr_flags & IN_IFF_NOTUSEABLE)
+	} else if (ap->addr_flags & IN_IFF_NOTUSEABLE)
 		return;
 #endif
 
-	/* Delete the old address if different */
-	if (state->addr &&
-	    state->addr->addr.s_addr != lease->addr.s_addr &&
-	    ipv4_iffindaddr(ifp, &lease->addr, NULL))
-		delete_address(ifp);
-
-	state->addr = ia;
-	state->added = STATE_ADDED;
-
-	/* Find any freshly added routes, such as the subnet route.
-	 * We do this because we cannot rely on recieving the kernel
-	 * notification right now via our link socket. */
-	if_initrt(ifp->ctx);
-	ipv4_buildroutes(ifp->ctx);
-	/* Announce the address */
-	if (ifo->options & DHCPCD_ARP) {
-		struct arp_state *astate;
-
-		if ((astate = arp_new(ifp, &state->addr->addr)) != NULL)
-			arp_announce(astate);
-	}
-	if (state->state == DHS_BOUND) {
-		script_runreason(ifp, state->reason);
-		dhcpcd_daemonise(ifp->ctx);
-	}
+	ipv4_finaliseaddr(ifp);
 }
 
 void
 ipv4_handleifa(struct dhcpcd_ctx *ctx,
     int cmd, struct if_head *ifs, const char *ifname,
-    const struct in_addr *addr, const struct in_addr *mask,
-    const struct in_addr *brd, const int addrflags)
+    const struct in_addr *addr, const struct in_addr *net,
+    const struct in_addr *dst, int flags)
 {
 	struct interface *ifp;
 	struct ipv4_state *state;
-	struct ipv4_addr *ia;
-	bool ia_is_new;
+	struct ipv4_addr *ap;
 
 	if (ifs == NULL)
 		ifs = ctx->ifaces;
@@ -1286,6 +1026,10 @@ ipv4_handleifa(struct dhcpcd_ctx *ctx,
 		errno = ESRCH;
 		return;
 	}
+	if (addr->s_addr == INADDR_ANY) {
+		errno = EINVAL;
+		return;
+	}
 	if ((ifp = if_find(ifs, ifname)) == NULL)
 		return;
 	if ((state = ipv4_getstate(ifp)) == NULL) {
@@ -1293,74 +1037,48 @@ ipv4_handleifa(struct dhcpcd_ctx *ctx,
 		return;
 	}
 
-	ia = ipv4_iffindaddr(ifp, addr, NULL);
-	switch (cmd) {
-	case RTM_NEWADDR:
-		if (ia == NULL) {
-			if ((ia = malloc(sizeof(*ia))) == NULL) {
+	ap = ipv4_iffindaddr(ifp, addr, net);
+	if (cmd == RTM_NEWADDR) {
+		if (ap == NULL) {
+			if ((ap = malloc(sizeof(*ap))) == NULL) {
 				logger(ifp->ctx, LOG_ERR, "%s: %m", __func__);
 				return;
 			}
-			ia->iface = ifp;
-			ia->addr = *addr;
-			ia->mask = *mask;
-			ia_is_new = true;
-#ifdef ALIAS_ADDR
-			strlcpy(ia->alias, ifname, sizeof(ia->alias));
-#endif
-			TAILQ_INSERT_TAIL(&state->addrs, ia, next);
-		} else
-			ia_is_new = false;
-		/* Mask could have changed */
-		if (ia_is_new ||
-		    (mask->s_addr != INADDR_ANY &&
-		    mask->s_addr != ia->mask.s_addr))
-		{
-			ia->mask = *mask;
-			snprintf(ia->saddr, sizeof(ia->saddr), "%s/%d",
-			    inet_ntoa(*addr), inet_ntocidr(*mask));
+			ap->iface = ifp;
+			ap->addr = *addr;
+			ap->net = *net;
+			if (dst)
+				ap->dst.s_addr = dst->s_addr;
+			else
+				ap->dst.s_addr = INADDR_ANY;
+			TAILQ_INSERT_TAIL(&state->addrs, ap, next);
+		}
+		ap->addr_flags = flags;
+	} else if (cmd == RTM_DELADDR) {
+		if (ap) {
+			TAILQ_REMOVE(&state->addrs, ap, next);
+			free(ap);
 		}
-		if (brd != NULL)
-			ia->brd = *brd;
-		else
-			ia->brd.s_addr = INADDR_ANY;
-		ia->addr_flags = addrflags;
-		break;
-	case RTM_DELADDR:
-		if (ia == NULL)
-			return;
-		TAILQ_REMOVE(&state->addrs, ia, next);
-		break;
-	default:
-		return;
 	}
 
-	if (addr->s_addr != INADDR_ANY && addr->s_addr != INADDR_BROADCAST) {
-		arp_handleifa(cmd, ia);
-		dhcp_handleifa(cmd, ia);
-	}
-
-	if (cmd == RTM_DELADDR)
-		free(ia);
+	dhcp_handleifa(cmd, ifp, addr, net, dst, flags);
+	arp_handleifa(cmd, ifp, addr, flags);
 }
 
 void
 ipv4_free(struct interface *ifp)
 {
 	struct ipv4_state *state;
-	struct ipv4_addr *ia;
+	struct ipv4_addr *addr;
 
 	if (ifp) {
 		state = IPV4_STATE(ifp);
 		if (state) {
-		        while ((ia = TAILQ_FIRST(&state->addrs))) {
-				TAILQ_REMOVE(&state->addrs, ia, next);
-				free(ia);
+		        while ((addr = TAILQ_FIRST(&state->addrs))) {
+				TAILQ_REMOVE(&state->addrs, addr, next);
+				free(addr);
 			}
 			ipv4_freerts(&state->routes);
-#ifdef BSD
-			free(state->buffer);
-#endif
 			free(state);
 		}
 	}
