@@ -154,31 +154,6 @@ static void ipv6nd_handledata(void *);
 #define IPV6_RECVPKTINFO IPV6_PKTINFO
 #endif
 
-void
-ipv6nd_printoptions(const struct dhcpcd_ctx *ctx,
-    const struct dhcp_opt *opts, size_t opts_len)
-{
-	size_t i, j;
-	const struct dhcp_opt *opt, *opt2;
-	int cols;
-
-	for (i = 0, opt = ctx->nd_opts;
-	    i < ctx->nd_opts_len; i++, opt++)
-	{
-		for (j = 0, opt2 = opts; j < opts_len; j++, opt2++)
-			if (opt2->option == opt->option)
-				break;
-		if (j == opts_len) {
-			cols = printf("%03d %s", opt->option, opt->var);
-			dhcp_print_option_encoding(opt, cols);
-		}
-	}
-	for (i = 0, opt = opts; i < opts_len; i++, opt++) {
-		cols = printf("%03d %s", opt->option, opt->var);
-		dhcp_print_option_encoding(opt, cols);
-	}
-}
-
 static int
 ipv6nd_open(struct dhcpcd_ctx *dctx)
 {
@@ -189,11 +164,29 @@ ipv6nd_open(struct dhcpcd_ctx *dctx)
 	ctx = dctx->ipv6;
 	if (ctx->nd_fd != -1)
 		return ctx->nd_fd;
-#define SOCK_FLAGS	SOCK_CLOEXEC | SOCK_NONBLOCK
-	ctx->nd_fd = xsocket(PF_INET6, SOCK_RAW | SOCK_FLAGS, IPPROTO_ICMPV6);
-#undef SOCK_FLAGS
+#ifdef SOCK_CLOEXEC
+	ctx->nd_fd = socket(PF_INET6, SOCK_RAW | SOCK_CLOEXEC | SOCK_NONBLOCK,
+	    IPPROTO_ICMPV6);
 	if (ctx->nd_fd == -1)
 		return -1;
+#else
+	if ((ctx->nd_fd = socket(PF_INET6, SOCK_RAW, IPPROTO_ICMPV6)) == -1)
+		return -1;
+	if ((on = fcntl(ctx->nd_fd, F_GETFD, 0)) == -1 ||
+	    fcntl(ctx->nd_fd, F_SETFD, on | FD_CLOEXEC) == -1)
+	{
+		close(ctx->nd_fd);
+		ctx->nd_fd = -1;
+	        return -1;
+	}
+	if ((on = fcntl(ctx->nd_fd, F_GETFL, 0)) == -1 ||
+	    fcntl(ctx->nd_fd, F_SETFL, on | O_NONBLOCK) == -1)
+	{
+		close(ctx->nd_fd);
+		ctx->nd_fd = -1;
+	        return -1;
+	}
+#endif
 
 	/* RFC4861 4.1 */
 	on = 255;
@@ -218,12 +211,13 @@ ipv6nd_open(struct dhcpcd_ctx *dctx)
 	    &filt, sizeof(filt)) == -1)
 		goto eexit;
 
-	eloop_event_add(dctx->eloop, ctx->nd_fd,  ipv6nd_handledata, dctx);
+	eloop_event_add(dctx->eloop, ctx->nd_fd,
+	    ipv6nd_handledata, dctx, NULL, NULL);
 	return ctx->nd_fd;
 
 eexit:
 	if (ctx->nd_fd != -1) {
-		eloop_event_delete(dctx->eloop, ctx->nd_fd);
+		eloop_event_delete(dctx->eloop, ctx->nd_fd, 0);
 		close(ctx->nd_fd);
 		ctx->nd_fd = -1;
 	}
@@ -243,7 +237,7 @@ ipv6nd_makersprobe(struct interface *ifp)
 	state->rs = calloc(1, state->rslen);
 	if (state->rs == NULL)
 		return -1;
-	rs = (void *)state->rs;
+	rs = (struct nd_router_solicit *)(void *)state->rs;
 	rs->nd_rs_type = ND_ROUTER_SOLICIT;
 	rs->nd_rs_code = 0;
 	rs->nd_rs_cksum = 0;
@@ -318,7 +312,7 @@ ipv6nd_sendrsprobe(void *arg)
 		logger(ifp->ctx, LOG_WARNING,
 		    "%s: no IPv6 Routers available", ifp->name);
 		ipv6nd_drop(ifp);
-		dhcp6_dropnondelegates(ifp);
+		dhcp6_drop(ifp, "EXPIRE6");
 	}
 }
 
@@ -328,16 +322,14 @@ ipv6nd_expire(struct interface *ifp, uint32_t seconds)
 	struct ra *rap;
 	struct timespec now;
 
-	if (ifp->ctx->ipv6 == NULL)
-		return;
-
-	clock_gettime(CLOCK_MONOTONIC, &now);
+	get_monotonic(&now);
 
 	TAILQ_FOREACH(rap, ifp->ctx->ipv6->ra_routers, next) {
 		if (rap->iface == ifp) {
-			rap->acquired = now;
+			rap->received = now;
 			rap->expired = seconds ? 0 : 1;
 			if (seconds) {
+				struct ra_opt *rao;
 				struct ipv6_addr *ap;
 
 				rap->lifetime = seconds;
@@ -348,6 +340,9 @@ ipv6nd_expire(struct interface *ifp, uint32_t seconds)
 					}
 				}
 				ipv6_addaddrs(&rap->addrs);
+				TAILQ_FOREACH(rao, &rap->options, next) {
+					timespecclear(&rao->expire);
+				}
 			}
 		}
 	}
@@ -399,25 +394,16 @@ ipv6nd_neighbour(struct dhcpcd_ctx *ctx, struct in6_addr *addr, int flags)
 	}
 }
 
-const struct ipv6_addr *
-ipv6nd_iffindaddr(const struct interface *ifp, const struct in6_addr *addr,
-    short flags)
+static void
+ipv6nd_free_opts(struct ra *rap)
 {
-	struct ra *rap;
-	struct ipv6_addr *ap;
+	struct ra_opt *rao;
 
-	if (ifp->ctx->ipv6 == NULL)
-		return NULL;
-
-	TAILQ_FOREACH(rap, ifp->ctx->ipv6->ra_routers, next) {
-		if (rap->iface != ifp)
-			continue;
-		TAILQ_FOREACH(ap, &rap->addrs, next) {
-			if (ipv6_findaddrmatch(ap, addr, flags))
-				return ap;
-		}
+	while ((rao = TAILQ_FIRST(&rap->options))) {
+		TAILQ_REMOVE(&rap->options, rao, next);
+		free(rao->option);
+		free(rao);
 	}
-	return NULL;
 }
 
 struct ipv6_addr *
@@ -432,33 +418,33 @@ ipv6nd_findaddr(struct dhcpcd_ctx *ctx, const struct in6_addr *addr,
 
 	TAILQ_FOREACH(rap, ctx->ipv6->ra_routers, next) {
 		TAILQ_FOREACH(ap, &rap->addrs, next) {
-			if (ipv6_findaddrmatch(ap, addr, flags))
+			if (addr == NULL) {
+				if ((ap->flags &
+				    (IPV6_AF_ADDED | IPV6_AF_DADCOMPLETED)) ==
+				    (IPV6_AF_ADDED | IPV6_AF_DADCOMPLETED))
+					return ap;
+			} else if (ap->prefix_vltime &&
+			    IN6_ARE_ADDR_EQUAL(&ap->addr, addr) &&
+			    (!flags || ap->flags & flags))
 				return ap;
 		}
 	}
 	return NULL;
 }
 
-static void
-ipv6nd_removefreedrop_ra(struct ra *rap, int remove_ra, int drop_ra)
+void ipv6nd_freedrop_ra(struct ra *rap, int drop)
 {
 
 	eloop_timeout_delete(rap->iface->ctx->eloop, NULL, rap->iface);
 	eloop_timeout_delete(rap->iface->ctx->eloop, NULL, rap);
-	if (remove_ra && !drop_ra)
+	if (!drop)
 		TAILQ_REMOVE(rap->iface->ctx->ipv6->ra_routers, rap, next);
-	ipv6_freedrop_addrs(&rap->addrs, drop_ra, NULL);
+	ipv6_freedrop_addrs(&rap->addrs, drop, NULL);
+	ipv6nd_free_opts(rap);
 	free(rap->data);
 	free(rap);
 }
 
-void
-ipv6nd_freedrop_ra(struct ra *rap, int drop)
-{
-
-	ipv6nd_removefreedrop_ra(rap, 1, drop);
-}
-
 ssize_t
 ipv6nd_free(struct interface *ifp)
 {
@@ -491,7 +477,7 @@ ipv6nd_free(struct interface *ifp)
 	}
 	if (ifp == NULL) {
 		if (ctx->ipv6->nd_fd != -1) {
-			eloop_event_delete(ctx->eloop, ctx->ipv6->nd_fd);
+			eloop_event_delete(ctx->eloop, ctx->ipv6->nd_fd, 0);
 			close(ctx->ipv6->nd_fd);
 			ctx->ipv6->nd_fd = -1;
 		}
@@ -542,6 +528,7 @@ ipv6nd_scriptrun(struct ra *rap)
 {
 	int hasdns, hasaddress, pid;
 	struct ipv6_addr *ap;
+	const struct ra_opt *rao;
 
 	hasaddress = 0;
 	/* If all addresses have completed DAD run the script */
@@ -551,7 +538,7 @@ ipv6nd_scriptrun(struct ra *rap)
 		{
 			hasaddress = 1;
 			if (!(ap->flags & IPV6_AF_DADCOMPLETED) &&
-			    ipv6_iffindaddr(ap->iface, &ap->addr, IN6_IFF_TENTATIVE))
+			    ipv6_iffindaddr(ap->iface, &ap->addr))
 				ap->flags |= IPV6_AF_DADCOMPLETED;
 			if ((ap->flags & IPV6_AF_DADCOMPLETED) == 0) {
 				logger(ap->iface->ctx, LOG_DEBUG,
@@ -567,7 +554,16 @@ ipv6nd_scriptrun(struct ra *rap)
 	if (!(rap->iface->options->options & DHCPCD_IPV6RA_REQRDNSS))
 		hasdns = 1;
 	else {
-		hasdns = rap->hasdns;
+		hasdns = 0;
+		TAILQ_FOREACH(rao, &rap->options, next) {
+			if (rao->type == ND_OPT_RDNSS &&
+			    rao->option &&
+			    timespecisset(&rao->expire))
+			{
+				hasdns = 1;
+				break;
+			}
+		}
 	}
 
 	script_runreason(rap->iface, "ROUTERADVERT");
@@ -648,9 +644,9 @@ ipv6nd_dadcallback(void *arg)
 			}
 			logger(ifp->ctx, LOG_INFO, "%s: deleting address %s",
 				ifp->name, ap->saddr);
-			if (if_address6(RTM_DELADDR, ap) == -1 &&
+			if (if_deladdress6(ap) == -1 &&
 			    errno != EADDRNOTAVAIL && errno != ENXIO)
-				logger(ifp->ctx, LOG_ERR, "if_address6: %m");
+				logger(ifp->ctx, LOG_ERR, "if_deladdress6: %m");
 			dadcounter = ap->dadcounter;
 			if (ipv6_makestableprivate(&ap->addr,
 			    &ap->prefix, ap->prefix_len,
@@ -712,69 +708,66 @@ try_script:
 	}
 }
 
-#ifndef DHCP6
-/* If DHCPv6 is compiled out, supply a shim to provide an error message
- * if IPv6RA requests DHCPv6. */
-#undef dhcp6_start
 static int
-dhcp6_start(__unused struct interface *ifp, __unused enum DH6S init_state)
+ipv6nd_ra_has_public_addr(const struct ra *rap)
 {
+	const struct ipv6_addr *ia;
 
-	errno = ENOTSUP;
-	return -1;
+	TAILQ_FOREACH(ia, &rap->addrs, next) {
+		if (ia->flags & IPV6_AF_AUTOCONF &&
+		    ipv6_publicaddr(ia))
+			return 1;
+	}
+	return 0;
 }
-#endif
 
 static void
 ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
-    struct icmp6_hdr *icp, size_t len, int hoplimit)
+    struct icmp6_hdr *icp, size_t len)
 {
 	struct ipv6_ctx *ctx = dctx->ipv6;
-	size_t i, olen;
+	size_t olen, l, n;
+	ssize_t r;
 	struct nd_router_advert *nd_ra;
 	struct nd_opt_prefix_info *pi;
 	struct nd_opt_mtu *mtu;
 	struct nd_opt_rdnss *rdnss;
-	uint32_t mtuv;
-	uint8_t *p;
+	struct nd_opt_dnssl *dnssl;
+	uint32_t lifetime, mtuv;
+	uint8_t *p, *op;
+	struct in6_addr addr;
 	char buf[INET6_ADDRSTRLEN];
 	const char *cbp;
 	struct ra *rap;
 	struct nd_opt_hdr *ndo;
+	struct ra_opt *rao;
 	struct ipv6_addr *ap;
-	struct dhcp_opt *dho;
+	char *opt, *opt2, *tmp;
+	struct timespec expire;
 	uint8_t new_rap, new_data;
 #ifdef IPV6_MANAGETEMPADDR
 	uint8_t new_ap;
 #endif
 
-	if (ifp == NULL) {
-#ifdef DEBUG_RS
-		logger(dctx, LOG_DEBUG,
-		    "RA for unexpected interface from %s", ctx->sfrom);
-#endif
-		return;
-	}
-
 	if (len < sizeof(struct nd_router_advert)) {
 		logger(dctx, LOG_ERR,
 		    "IPv6 RA packet too short from %s", ctx->sfrom);
 		return;
 	}
 
-	/* RFC 4861 7.1.2 */
-	if (hoplimit != 255) {
-		logger(dctx, LOG_ERR,
-		    "invalid hoplimit(%d) in RA from %s", hoplimit, ctx->sfrom);
-		return;
-	}
-
 	if (!IN6_IS_ADDR_LINKLOCAL(&ctx->from.sin6_addr)) {
 		logger(dctx, LOG_ERR,
 		    "RA from non local address %s", ctx->sfrom);
 		return;
 	}
 
+	if (ifp == NULL) {
+#ifdef DEBUG_RS
+		logger(dctx, LOG_DEBUG,
+		    "RA for unexpected interface from %s", ctx->sfrom);
+#endif
+		return;
+	}
 	if (!(ifp->options->options & DHCPCD_IPV6RS)) {
 #ifdef DEBUG_RS
 		logger(ifp->ctx, LOG_DEBUG, "%s: unexpected RA from %s",
@@ -793,7 +786,7 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 		return;
 	}
 
-	if (ipv6_iffindaddr(ifp, &ctx->from.sin6_addr, IN6_IFF_TENTATIVE)) {
+	if (ipv6_iffindaddr(ifp, &ctx->from.sin6_addr)) {
 		logger(ifp->ctx, LOG_DEBUG,
 		    "%s: ignoring RA from ourself %s", ifp->name, ctx->sfrom);
 		return;
@@ -815,10 +808,15 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 		if (rap) {
 			free(rap->data);
 			rap->data_len = 0;
+			rap->no_public_warned = 0;
 		}
 		new_data = 1;
 	} else
 		new_data = 0;
+	if (new_data || ifp->options->options & DHCPCD_DEBUG)
+		logger(ifp->ctx, LOG_INFO, "%s: Router Advertisement from %s",
+		    ifp->name, ctx->sfrom);
+
 	if (rap == NULL) {
 		rap = calloc(1, sizeof(*rap));
 		if (rap == NULL) {
@@ -829,6 +827,7 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 		rap->from = ctx->from.sin6_addr;
 		strlcpy(rap->sfrom, ctx->sfrom, sizeof(rap->sfrom));
 		TAILQ_INIT(&rap->addrs);
+		TAILQ_INIT(&rap->options);
 		new_rap = 1;
 	} else
 		new_rap = 0;
@@ -844,16 +843,11 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 		rap->data_len = len;
 	}
 
-	/* We could change the debug level based on new_data, but some
-	 * routers like to decrease the advertised valid and preferred times
-	 * in accordance with the own prefix times which would result in too
-	 * much needless log spam. */
-	logger(ifp->ctx, new_rap ? LOG_INFO : LOG_DEBUG,
-	    "%s: Router Advertisement from %s",
-	    ifp->name, ctx->sfrom);
-
-	clock_gettime(CLOCK_MONOTONIC, &rap->acquired);
+	get_monotonic(&rap->received);
 	rap->flags = nd_ra->nd_ra_flags_reserved;
+	if (new_rap == 0 && rap->lifetime == 0)
+		logger(ifp->ctx, LOG_WARNING, "%s: %s router available",
+		   ifp->name, rap->sfrom);
 	rap->lifetime = ntohs(nd_ra->nd_ra_router_lifetime);
 	if (nd_ra->nd_ra_reachable) {
 		rap->reachable = ntohl(nd_ra->nd_ra_reachable);
@@ -864,7 +858,6 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 		rap->retrans = ntohl(nd_ra->nd_ra_retransmit);
 	if (rap->lifetime)
 		rap->expired = 0;
-	rap->hasdns = 0;
 
 	ipv6_settempstale(ifp);
 	TAILQ_FOREACH(ap, &rap->addrs, next) {
@@ -873,6 +866,7 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 
 	len -= sizeof(struct nd_router_advert);
 	p = ((uint8_t *)icp) + sizeof(struct nd_router_advert);
+	lifetime = ~0U;
 	for (; len > 0; p += olen, len -= olen) {
 		if (len < sizeof(struct nd_opt_hdr)) {
 			logger(ifp->ctx, LOG_ERR,
@@ -892,37 +886,10 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 			break;
 		}
 
-		if (has_option_mask(ifp->options->rejectmasknd,
-		    ndo->nd_opt_type))
-		{
-			for (i = 0, dho = dctx->nd_opts;
-			    i < dctx->nd_opts_len;
-			    i++, dho++)
-			{
-				if (dho->option == ndo->nd_opt_type)
-					break;
-			}
-			if (dho != NULL)
-				logger(ifp->ctx, LOG_WARNING,
-				    "%s: reject RA (option %s) from %s",
-				    ifp->name, dho->var, ctx->sfrom);
-			else
-				logger(ifp->ctx, LOG_WARNING,
-				    "%s: reject RA (option %d) from %s",
-				    ifp->name, ndo->nd_opt_type, ctx->sfrom);
-			if (new_rap)
-				ipv6nd_removefreedrop_ra(rap, 0, 0);
-			else
-				ipv6nd_free_ra(rap);
-			return;
-		}
-
-		if (has_option_mask(ifp->options->nomasknd, ndo->nd_opt_type))
-			continue;
-
+		opt = opt2 = NULL;
 		switch (ndo->nd_opt_type) {
 		case ND_OPT_PREFIX_INFORMATION:
-			pi = (void *)ndo;
+			pi = (struct nd_opt_prefix_info *)(void *)ndo;
 			if (pi->nd_opt_pi_len != 4) {
 				logger(ifp->ctx, new_data ? LOG_ERR : LOG_DEBUG,
 				    "%s: invalid option len for prefix",
@@ -996,7 +963,7 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 					ap->saddr[0] = '\0';
 				}
 				ap->dadcallback = ipv6nd_dadcallback;
-				ap->created = ap->acquired = rap->acquired;
+				ap->created = ap->acquired = rap->received;
 				TAILQ_INSERT_TAIL(&rap->addrs, ap, next);
 
 #ifdef IPV6_MANAGETEMPADDR
@@ -1005,8 +972,7 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 				 * temporary address also exists then
 				 * extend the existing one rather than
 				 * create a new one */
-				if (ipv6_iffindaddr(ifp, &ap->addr,
-				    IN6_IFF_NOTUSEABLE) &&
+				if (ipv6_iffindaddr(ifp, &ap->addr) &&
 				    ipv6_settemptime(ap, 0))
 					new_ap = 0;
 				else
@@ -1017,7 +983,7 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 				new_ap = 0;
 #endif
 				ap->flags &= ~IPV6_AF_STALE;
-				ap->acquired = rap->acquired;
+				ap->acquired = rap->received;
 			}
 			if (pi->nd_opt_pi_flags_reserved &
 			    ND_OPT_PI_FLAG_ONLINK)
@@ -1027,6 +993,16 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 			ap->prefix_pltime =
 			    ntohl(pi->nd_opt_pi_preferred_time);
 			ap->nsprobes = 0;
+			cbp = inet_ntop(AF_INET6, &ap->prefix, buf, sizeof(buf));
+			if (cbp) {
+				l = strlen(cbp);
+				opt = malloc(l + 5);
+				if (opt) {
+					snprintf(opt, l + 5, "%s/%d", cbp,
+					    ap->prefix_len);
+					opt2 = strdup(ap->saddr);
+				}
+			}
 
 #ifdef IPV6_MANAGETEMPADDR
 			/* RFC4941 Section 3.3.3 */
@@ -1046,10 +1022,12 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 				}
 			}
 #endif
+
+			lifetime = ap->prefix_vltime;
 			break;
 
 		case ND_OPT_MTU:
-			mtu = (void *)p;
+			mtu = (struct nd_opt_mtu *)(void *)p;
 			mtuv = ntohl(mtu->nd_opt_mtu_mtu);
 			if (mtuv < IPV6_MMTU) {
 				logger(ifp->ctx, LOG_ERR, "%s: invalid MTU %d",
@@ -1057,53 +1035,166 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 				break;
 			}
 			rap->mtu = mtuv;
+			snprintf(buf, sizeof(buf), "%d", mtuv);
+			opt = strdup(buf);
 			break;
 
 		case ND_OPT_RDNSS:
-			rdnss = (void *)p;
-			if (rdnss->nd_opt_rdnss_lifetime &&
-			    rdnss->nd_opt_rdnss_len > 1)
-				rap->hasdns = 1;
+			rdnss = (struct nd_opt_rdnss *)p;
+			lifetime = ntohl(rdnss->nd_opt_rdnss_lifetime);
+			op = (uint8_t *)ndo;
+			op += offsetof(struct nd_opt_rdnss,
+			    nd_opt_rdnss_lifetime);
+			op += sizeof(rdnss->nd_opt_rdnss_lifetime);
+			l = 0;
+			for (n = (size_t)ndo->nd_opt_len - 1; n > 1; n -= 2,
+			    op += sizeof(addr))
+			{
+				r = ipv6_printaddr(NULL, 0, op, ifp->name);
+				if (r != -1)
+					l += (size_t)r + 1;
+			}
+			op = (uint8_t *)ndo;
+			op += offsetof(struct nd_opt_rdnss,
+			    nd_opt_rdnss_lifetime);
+			op += sizeof(rdnss->nd_opt_rdnss_lifetime);
+			tmp = opt = malloc(l);
+			if (opt == NULL)
+				continue;
+			for (n = (size_t)ndo->nd_opt_len - 1; n > 1; n -= 2,
+			    op += sizeof(addr))
+			{
+				r = ipv6_printaddr(tmp, l, op,
+				    ifp->name);
+				if (r != -1) {
+					l -= ((size_t)r + 1);
+					tmp += (size_t)r;
+					*tmp++ = ' ';
+				}
+			}
+			if (tmp != opt)
+				(*--tmp) = '\0';
+			else
+				*opt = '\0';
+			break;
+
+		case ND_OPT_DNSSL:
+			dnssl = (struct nd_opt_dnssl *)p;
+			lifetime = ntohl(dnssl->nd_opt_dnssl_lifetime);
+			op = p + offsetof(struct nd_opt_dnssl,
+			    nd_opt_dnssl_lifetime);
+			op += sizeof(dnssl->nd_opt_dnssl_lifetime);
+			n = (size_t)(dnssl->nd_opt_dnssl_len - 1) * 8;
+			r = decode_rfc3397(NULL, 0, op, n);
+			if (r < 1) {
+				logger(ifp->ctx, new_data ? LOG_ERR : LOG_DEBUG,
+				    "%s: invalid DNSSL option",
+				    ifp->name);
+				continue;
+			} else {
+				l = (size_t)r + 1;
+				tmp = malloc(l);
+				if (tmp) {
+					decode_rfc3397(tmp, l, op, n);
+					l -= 1;
+					n = (size_t)print_string(NULL, 0,
+					    STRING | ARRAY | DOMAIN,
+					    (const uint8_t *)tmp, l);
+					n++;
+					opt = malloc(n);
+					if (opt) {
+						print_string(opt, n,
+						    STRING | ARRAY | DOMAIN,
+						    (const uint8_t *)tmp, l);
+					} else
+						logger(ifp->ctx, LOG_ERR,
+						    "%s: %m", __func__);
+					free(tmp);
+				}
+			}
+			break;
 
 		default:
 			continue;
 		}
-	}
 
-	for (i = 0, dho = dctx->nd_opts;
-	    i < dctx->nd_opts_len;
-	    i++, dho++)
-	{
-		if (has_option_mask(ifp->options->requiremasknd,
-		    dho->option))
-		{
-			logger(ifp->ctx, LOG_WARNING,
-			    "%s: reject RA (no option %s) from %s",
-			    ifp->name, dho->var, ctx->sfrom);
-			if (new_rap)
-				ipv6nd_removefreedrop_ra(rap, 0, 0);
-			else
-				ipv6nd_free_ra(rap);
-			return;
+		if (opt == NULL) {
+			logger(ifp->ctx, LOG_ERR, "%s: %m", __func__);
+			continue;
+		}
+
+		n = ndo->nd_opt_type;
+extra_opt:
+		TAILQ_FOREACH(rao, &rap->options, next) {
+			if (rao->type == n &&
+			    strcmp(rao->option, opt) == 0)
+				break;
+		}
+		if (lifetime == 0 || *opt == '\0') {
+			if (rao) {
+				TAILQ_REMOVE(&rap->options, rao, next);
+				free(rao->option);
+				free(rao);
+			}
+			free(opt);
+			free(opt2);
+			continue;
+		}
+
+		if (rao == NULL) {
+			rao = malloc(sizeof(*rao));
+			if (rao == NULL) {
+				logger(ifp->ctx, LOG_ERR, "%s: %m", __func__);
+				continue;
+			}
+			rao->type = (uint16_t)n;
+			rao->option = opt;
+			TAILQ_INSERT_TAIL(&rap->options, rao, next);
+		} else
+			free(opt);
+		if (lifetime == ~0U)
+			timespecclear(&rao->expire);
+		else {
+			expire.tv_sec = (time_t)lifetime;
+			expire.tv_nsec = 0;
+			timespecadd(&rap->received, &expire, &rao->expire);
+		}
+		if (rao && rao->type == ND_OPT_PREFIX_INFORMATION && opt2) {
+			n = _ND_OPT_PREFIX_ADDR;
+			opt = opt2;
+			opt2 = NULL;
+			goto extra_opt;
 		}
 	}
 
 	if (new_rap)
 		add_router(ifp->ctx->ipv6, rap);
-
+	if (!ipv6nd_ra_has_public_addr(rap) &&
+	    !(rap->iface->options->options & DHCPCD_IPV6RA_ACCEPT_NOPUBLIC) &&
+	    (!(rap->flags & ND_RA_FLAG_MANAGED) ||
+	    !dhcp6_has_public_addr(rap->iface)))
+	{
+		logger(rap->iface->ctx,
+		    rap->no_public_warned ? LOG_DEBUG : LOG_WARNING,
+		    "%s: ignoring RA from %s"
+		    " (no public prefix, no managed address)",
+		    rap->iface->name, rap->sfrom);
+		rap->no_public_warned = 1;
+		return;
+	}
 	if (ifp->ctx->options & DHCPCD_TEST) {
 		script_runreason(ifp, "TEST");
 		goto handle_flag;
 	}
 	ipv6_addaddrs(&rap->addrs);
 #ifdef IPV6_MANAGETEMPADDR
-	ipv6_addtempaddrs(ifp, &rap->acquired);
+	ipv6_addtempaddrs(ifp, &rap->received);
 #endif
 
 	/* Find any freshly added routes, such as the subnet route.
 	 * We do this because we cannot rely on recieving the kernel
 	 * notification right now via our link socket. */
-	if_initrt6(ifp->ctx);
+	if_initrt6(ifp);
 
 	ipv6_buildroutes(ifp->ctx);
 	if (ipv6nd_scriptrun(rap))
@@ -1115,19 +1206,13 @@ ipv6nd_handlera(struct dhcpcd_ctx *dctx, struct interface *ifp,
 handle_flag:
 	if (!(ifp->options->options & DHCPCD_DHCP6))
 		goto nodhcp6;
-/* Only log a DHCPv6 start error if compiled in or debugging is enabled. */
-#ifdef DHCP6
-#define LOG_DHCP6	LOG_ERR
-#else
-#define LOG_DHCP6	LOG_DEBUG
-#endif
 	if (rap->flags & ND_RA_FLAG_MANAGED) {
 		if (new_data && dhcp6_start(ifp, DH6S_INIT) == -1)
-			logger(ifp->ctx, LOG_DHCP6,
+			logger(ifp->ctx, LOG_ERR,
 			    "dhcp6_start: %s: %m", ifp->name);
 	} else if (rap->flags & ND_RA_FLAG_OTHER) {
 		if (new_data && dhcp6_start(ifp, DH6S_INFORM) == -1)
-			logger(ifp->ctx, LOG_DHCP6,
+			logger(ifp->ctx, LOG_ERR,
 			    "dhcp6_start: %s: %m", ifp->name);
 	} else {
 		if (new_data)
@@ -1144,6 +1229,34 @@ nodhcp6:
 	ipv6nd_expirera(ifp);
 }
 
+/* Run RA's we ignored becuase they had no public addresses
+ * This should only be called when DHCPv6 applies a public address */
+void
+ipv6nd_runignoredra(struct interface *ifp)
+{
+	struct ra *rap;
+
+	TAILQ_FOREACH(rap, ifp->ctx->ipv6->ra_routers, next) {
+		if (rap->iface == ifp &&
+		    !rap->expired &&
+		    rap->no_public_warned)
+		{
+			rap->no_public_warned = 0;
+			logger(rap->iface->ctx, LOG_INFO,
+			    "%s: applying ignored RA from %s",
+			    rap->iface->name, rap->sfrom);
+			if (ifp->ctx->options & DHCPCD_TEST) {
+				script_runreason(ifp, "TEST");
+				continue;
+			}
+			if (ipv6nd_scriptrun(rap))
+				return;
+			eloop_timeout_delete(ifp->ctx->eloop, NULL, ifp);
+			eloop_timeout_delete(ifp->ctx->eloop, NULL, rap);
+		}
+	}
+}
+
 int
 ipv6nd_hasra(const struct interface *ifp)
 {
@@ -1173,171 +1286,133 @@ ipv6nd_hasradhcp(const struct interface *ifp)
 	return 0;
 }
 
-static const uint8_t *
-ipv6nd_getoption(struct dhcpcd_ctx *ctx,
-    size_t *os, unsigned int *code, size_t *len,
-    const uint8_t *od, size_t ol, struct dhcp_opt **oopt)
-{
-	const struct nd_opt_hdr *o;
-	size_t i;
-	struct dhcp_opt *opt;
-
-	if (od) {
-		*os = sizeof(*o);
-		if (ol < *os) {
-			errno = EINVAL;
-			return NULL;
-		}
-		o = (const struct nd_opt_hdr *)od;
-		if (o->nd_opt_len > ol) {
-			errno = EINVAL;
-			return NULL;
-		}
-		*len = ND_OPTION_LEN(o);
-		*code = o->nd_opt_type;
-	} else
-		o = NULL;
-
-	for (i = 0, opt = ctx->nd_opts;
-	    i < ctx->nd_opts_len; i++, opt++)
-	{
-		if (opt->option == *code) {
-			*oopt = opt;
-			break;
-		}
-	}
-
-	if (o)
-		return ND_COPTION_DATA(o);
-	return NULL;
-}
-
 ssize_t
 ipv6nd_env(char **env, const char *prefix, const struct interface *ifp)
 {
-	size_t i, j, n, len;
-	struct ra *rap;
-	char ndprefix[32], abuf[24];
-	struct dhcp_opt *opt;
-	const struct nd_opt_hdr *o;
-	struct ipv6_addr *ia;
-	struct timespec now;
+	size_t i, l, len;
+	const struct ra *rap;
+	const struct ra_opt *rao;
+	char buffer[32];
+	const char *optn;
+	char **pref, **addr, **mtu, **rdnss, **dnssl, ***var, *new;
 
-	clock_gettime(CLOCK_MONOTONIC, &now);
-	i = n = 0;
+	i = l = 0;
 	TAILQ_FOREACH(rap, ifp->ctx->ipv6->ra_routers, next) {
 		if (rap->iface != ifp)
 			continue;
 		i++;
-		if (prefix != NULL)
-			snprintf(ndprefix, sizeof(ndprefix),
-			    "%s_nd%zu", prefix, i);
-		else
-			snprintf(ndprefix, sizeof(ndprefix),
-			    "nd%zu", i);
-		if (env)
-			setvar(rap->iface->ctx, &env[n], ndprefix,
-			    "from", rap->sfrom);
-		n++;
-		if (env)
-			setvard(rap->iface->ctx, &env[n], ndprefix,
-			    "acquired", (size_t)rap->acquired.tv_sec);
-		n++;
-		if (env)
-			setvard(rap->iface->ctx, &env[n], ndprefix,
-			    "now", (size_t)now.tv_sec);
-		n++;
-
-		/* Zero our indexes */
 		if (env) {
-			for (j = 0, opt = rap->iface->ctx->nd_opts;
-			    j < rap->iface->ctx->nd_opts_len;
-			    j++, opt++)
-				dhcp_zero_index(opt);
-			for (j = 0, opt = rap->iface->options->nd_override;
-			    j < rap->iface->options->nd_override_len;
-			    j++, opt++)
-				dhcp_zero_index(opt);
+			snprintf(buffer, sizeof(buffer),
+			    "ra%zu_from", i);
+			setvar(ifp->ctx, &env, prefix, buffer, rap->sfrom);
 		}
+		l++;
 
-		/* Unlike DHCP, ND6 options *may* occur more than once.
-		 * There is also no provision for option concatenation
-		 * unlike DHCP. */
-		len = rap->data_len -
-		    ((size_t)((const uint8_t *)ND_CFIRST_OPTION(rap) -
-		    rap->data));
-
-		for (o = ND_CFIRST_OPTION(rap);
-		    len >= (ssize_t)sizeof(*o);
-		    o = ND_CNEXT_OPTION(o))
-		{
-			if ((size_t)o->nd_opt_len * 8 > len) {
-				errno =	EINVAL;
+		pref = addr = mtu = rdnss = dnssl = NULL;
+		TAILQ_FOREACH(rao, &rap->options, next) {
+			if (rao->option == NULL)
+				continue;
+			var = NULL;
+			switch(rao->type) {
+			case ND_OPT_PREFIX_INFORMATION:
+				optn = "prefix";
+				var = &pref;
 				break;
-			}
-			len -= (size_t)(o->nd_opt_len * 8);
-			if (has_option_mask(rap->iface->options->nomasknd,
-			    o->nd_opt_type))
+			case _ND_OPT_PREFIX_ADDR:
+				optn = "addr";
+				var = &addr;
+				break;
+			case ND_OPT_MTU:
+				optn = "mtu";
+				var = &mtu;
+				break;
+			case ND_OPT_RDNSS:
+				optn = "rdnss";
+				var = &rdnss;
+				break;
+			case ND_OPT_DNSSL:
+				optn = "dnssl";
+				var = &dnssl;
+				break;
+			default:
 				continue;
-			for (j = 0, opt = rap->iface->options->nd_override;
-			    j < rap->iface->options->nd_override_len;
-			    j++, opt++)
-				if (opt->option == o->nd_opt_type)
-					break;
-			if (j == rap->iface->options->nd_override_len) {
-				for (j = 0, opt = rap->iface->ctx->nd_opts;
-				    j < rap->iface->ctx->nd_opts_len;
-				    j++, opt++)
-					if (opt->option == o->nd_opt_type)
-						break;
-				if (j == rap->iface->ctx->nd_opts_len)
-					opt = NULL;
 			}
-			if (opt) {
-				n += dhcp_envoption(rap->iface->ctx,
-				    env == NULL ? NULL : &env[n],
-				    ndprefix, rap->iface->name,
-				    opt, ipv6nd_getoption,
-				    ND_COPTION_DATA(o), ND_OPTION_LEN(o));
-			}
-		}
-
-		/* We need to output the addresses we actually made
-		 * from the prefix information options as well. */
-		j = 0;
-		TAILQ_FOREACH(ia, &rap->addrs, next) {
-			if (!(ia->flags & IPV6_AF_AUTOCONF)
-#ifdef IPV6_AF_TEMPORARY
-			    || ia->flags & IPV6_AF_TEMPORARY
-#endif
-			    )
+			if (*var == NULL) {
+				*var = env ? env : &new;
+				l++;
+			} else if (env) {
+				/* With single only options, last one takes
+				 * precedence */
+				if (rao->type == ND_OPT_MTU) {
+					new = strchr(**var, '=');
+					if (new == NULL) {
+						logger(ifp->ctx, LOG_ERR,
+						    "new is null");
+						continue;
+					} else
+						new++;
+					len = (size_t)(new - **var) +
+					    strlen(rao->option) + 1;
+					if (len > strlen(**var))
+						new = realloc(**var, len);
+					else
+						new = **var;
+					if (new) {
+						**var = new;
+						new = strchr(**var, '=');
+						if (new) {
+							len -=
+							    (size_t)
+							    (new - **var);
+							strlcpy(new + 1,
+							    rao->option,
+							    len - 1);
+						} else
+							logger(ifp->ctx,
+							    LOG_ERR,
+							    "new is null");
+					}
+					continue;
+				}
+				len = strlen(rao->option) + 1;
+				new = realloc(**var, strlen(**var) + 1 + len);
+				if (new) {
+					**var = new;
+					new += strlen(new);
+					*new++ = ' ';
+					strlcpy(new, rao->option, len);
+				} else
+					logger(ifp->ctx, LOG_ERR,
+					    "%s: %m", __func__);
 				continue;
-			j++;
+			}
 			if (env) {
-				snprintf(abuf, sizeof(abuf), "addr%zu", j);
-				setvar(rap->iface->ctx, &env[n], ndprefix,
-				    abuf, ia->saddr);
+				snprintf(buffer, sizeof(buffer),
+				    "ra%zu_%s", i, optn);
+				setvar(ifp->ctx, &env,
+				    prefix, buffer, rao->option);
 			}
-			n++;
 		}
 	}
-	return (ssize_t)n;
+
+	if (env)
+		setvard(ifp->ctx, &env, prefix, "ra_count", i);
+	l++;
+	return (ssize_t)l;
 }
 
 void
-ipv6nd_handleifa(int cmd, struct ipv6_addr *addr)
+ipv6nd_handleifa(struct dhcpcd_ctx *ctx, int cmd, const char *ifname,
+    const struct in6_addr *addr, int flags)
 {
 	struct ra *rap;
 
-	/* IPv6 init may not have happened yet if we are learning
-	 * existing addresses when dhcpcd starts. */
-	if (addr->iface->ctx->ipv6 == NULL)
+	if (ctx->ipv6 == NULL)
 		return;
-
-	TAILQ_FOREACH(rap, addr->iface->ctx->ipv6->ra_routers, next) {
-		if (rap->iface != addr->iface)
+	TAILQ_FOREACH(rap, ctx->ipv6->ra_routers, next) {
+		if (strcmp(rap->iface->name, ifname))
 			continue;
-		ipv6_handleifa_addrs(cmd, &rap->addrs, addr);
+		ipv6_handleifa_addrs(cmd, &rap->addrs, addr, flags);
 	}
 }
 
@@ -1346,12 +1421,12 @@ ipv6nd_expirera(void *arg)
 {
 	struct interface *ifp;
 	struct ra *rap, *ran;
+	struct ra_opt *rao, *raon;
 	struct timespec now, lt, expire, next;
 	uint8_t expired, valid, validone;
-	struct ipv6_addr *ia;
 
 	ifp = arg;
-	clock_gettime(CLOCK_MONOTONIC, &now);
+	get_monotonic(&now);
 	expired = 0;
 	timespecclear(&next);
 
@@ -1363,7 +1438,7 @@ ipv6nd_expirera(void *arg)
 		if (rap->lifetime) {
 			lt.tv_sec = (time_t)rap->lifetime;
 			lt.tv_nsec = 0;
-			timespecadd(&rap->acquired, &lt, &expire);
+			timespecadd(&rap->received, &lt, &expire);
 			if (rap->lifetime == 0 || timespeccmp(&now, &expire, >))
 			{
 				if (!rap->expired) {
@@ -1382,44 +1457,44 @@ ipv6nd_expirera(void *arg)
 			}
 		}
 
-		/* Not every prefix is tied to an address which
-		 * the kernel can expire, so we need to handle it ourself.
-		 * Also, some OS don't support address lifetimes (Solaris). */
-		TAILQ_FOREACH(ia, &rap->addrs, next) {
-			if (ia->prefix_vltime == ND6_INFINITE_LIFETIME ||
-			    ia->prefix_vltime == 0)
-				continue;
-			lt.tv_sec = (time_t)ia->prefix_vltime;
-			lt.tv_nsec = 0;
-			timespecadd(&ia->acquired, &lt, &expire);
-			if (timespeccmp(&now, &expire, >)) {
-				if (ia->flags & IPV6_AF_ADDED) {
-					logger(ia->iface->ctx, LOG_WARNING,
-					    "%s: expired address %s",
-					    ia->iface->name, ia->saddr);
-					if (if_address6(RTM_DELADDR, ia)== -1 &&
-					    errno != EADDRNOTAVAIL &&
-					    errno != ENXIO)
-						logger(ia->iface->ctx, LOG_ERR,
-						    "if_address6: %m");
+		/* Addresses are expired in ipv6_addaddrs
+		 * so that DHCPv6 addresses can be removed also. */
+		TAILQ_FOREACH_SAFE(rao, &rap->options, next, raon) {
+			if (rap->expired) {
+				switch(rao->type) {
+				case ND_OPT_RDNSS: /* FALLTHROUGH */
+				case ND_OPT_DNSSL:
+					/* RFC6018 end of section 5.2 states
+					 * that if tha RA has a lifetime of 0
+					 * then we should expire these
+					 * options */
+					TAILQ_REMOVE(&rap->options, rao, next);
+					expired = 1;
+					free(rao->option);
+					free(rao);
+					continue;
 				}
-				ia->prefix_vltime = ia->prefix_pltime = 0;
-				ia->flags &=
-				    ~(IPV6_AF_ADDED | IPV6_AF_DADCOMPLETED);
-				expired = 1;
-			} else {
-				timespecsub(&expire, &now, &lt);
-				if (!timespecisset(&next) ||
-				    timespeccmp(&next, &lt, >))
-					next = lt;
 			}
+			if (!timespecisset(&rao->expire))
+				continue;
+			if (timespeccmp(&now, &rao->expire, >)) {
+				/* Expired prefixes are logged above */
+				if (rao->type != ND_OPT_PREFIX_INFORMATION)
+					logger(ifp->ctx, LOG_WARNING,
+					    "%s: %s: expired option %d",
+					    ifp->name, rap->sfrom, rao->type);
+				TAILQ_REMOVE(&rap->options, rao, next);
+				expired = 1;
+				free(rao->option);
+				free(rao);
+				continue;
+			}
+			valid = 1;
+			timespecsub(&rao->expire, &now, &lt);
+			if (!timespecisset(&next) || timespeccmp(&next, &lt, >))
+				next = lt;
 		}
 
-		/* XXX FixMe!
-		 * We need to extract the lifetime from each option and check
-		 * if that has expired or not.
-		 * If it has, zero the option out in the returned data. */
-
 		/* No valid lifetimes are left on the RA, so we might
 		 * as well punt it. */
 		if (!valid && TAILQ_FIRST(&rap->addrs) == NULL)
@@ -1438,7 +1513,7 @@ ipv6nd_expirera(void *arg)
 
 	/* No valid routers? Kill any DHCPv6. */
 	if (!validone)
-		dhcp6_dropnondelegates(ifp);
+		dhcp6_drop(ifp, "EXPIRE6");
 }
 
 void
@@ -1466,14 +1541,16 @@ ipv6nd_drop(struct interface *ifp)
 			ipv6nd_drop_ra(rap);
 		}
 		ipv6_buildroutes(ifp->ctx);
-		if ((ifp->options->options & DHCPCD_NODROP) != DHCPCD_NODROP)
+		if ((ifp->options->options &
+		    (DHCPCD_EXITING | DHCPCD_PERSISTENT)) !=
+		    (DHCPCD_EXITING | DHCPCD_PERSISTENT))
 			script_runreason(ifp, "ROUTERADVERT");
 	}
 }
 
 static void
 ipv6nd_handlena(struct dhcpcd_ctx *dctx, struct interface *ifp,
-    struct icmp6_hdr *icp, size_t len, int hoplimit)
+    struct icmp6_hdr *icp, size_t len)
 {
 	struct ipv6_ctx *ctx = dctx->ipv6;
 	struct nd_neighbor_advert *nd_na;
@@ -1496,13 +1573,6 @@ ipv6nd_handlena(struct dhcpcd_ctx *dctx, struct interface *ifp,
 		return;
 	}
 
-	/* RFC 4861 7.1.2 */
-	if (hoplimit != 255) {
-		logger(dctx, LOG_ERR,
-		    "invalid hoplimit(%d) in NA from %s", hoplimit, ctx->sfrom);
-		return;
-	}
-
 	nd_na = (struct nd_neighbor_advert *)icp;
 	is_router = nd_na->nd_na_flags_reserved & ND_NA_FLAG_ROUTER;
 	is_solicited = nd_na->nd_na_flags_reserved & ND_NA_FLAG_SOLICITED;
@@ -1570,10 +1640,10 @@ ipv6nd_handledata(void *arg)
 	ctx = dctx->ipv6;
 	ctx->rcvhdr.msg_controllen = CMSG_SPACE(sizeof(struct in6_pktinfo)) +
 	    CMSG_SPACE(sizeof(int));
-	len = recvmsg_realloc(ctx->nd_fd, &ctx->rcvhdr, 0);
+	len = recvmsg(ctx->nd_fd, &ctx->rcvhdr, 0);
 	if (len == -1) {
 		logger(dctx, LOG_ERR, "recvmsg: %m");
-		eloop_event_delete(dctx->eloop, ctx->nd_fd);
+		eloop_event_delete(dctx->eloop, ctx->nd_fd, 0);
 		close(ctx->nd_fd);
 		ctx->nd_fd = -1;
 		return;
@@ -1606,16 +1676,17 @@ ipv6nd_handledata(void *arg)
 		}
 	}
 
-	if (pkt.ipi6_ifindex == 0) {
+	if (pkt.ipi6_ifindex == 0 || hoplimit == 0) {
 		logger(dctx, LOG_ERR,
-		    "IPv6 RA/NA did not contain index from %s",
+		    "IPv6 RA/NA did not contain index or hop limit from %s",
 		    ctx->sfrom);
 		return;
 	}
 
 	TAILQ_FOREACH(ifp, dctx->ifaces, next) {
-		if (ifp->active &&
-		    ifp->index == (unsigned int)pkt.ipi6_ifindex) {
+		if (ifp->index == (unsigned int)pkt.ipi6_ifindex &&
+		    !(ifp->options->options & DHCPCD_PFXDLGONLY))
+		{
 			if (!(ifp->options->options & DHCPCD_IPV6))
 				return;
 			break;
@@ -1626,12 +1697,10 @@ ipv6nd_handledata(void *arg)
 	if (icp->icmp6_code == 0) {
 		switch(icp->icmp6_type) {
 			case ND_NEIGHBOR_ADVERT:
-				ipv6nd_handlena(dctx, ifp, icp, (size_t)len,
-				   hoplimit);
+				ipv6nd_handlena(dctx, ifp, icp, (size_t)len);
 				return;
 			case ND_ROUTER_ADVERT:
-				ipv6nd_handlera(dctx, ifp, icp, (size_t)len,
-				   hoplimit);
+				ipv6nd_handlera(dctx, ifp, icp, (size_t)len);
 				return;
 		}
 	}
@@ -1681,11 +1750,6 @@ ipv6nd_startrs(struct interface *ifp)
 	struct timespec tv;
 
 	eloop_timeout_delete(ifp->ctx->eloop, NULL, ifp);
-	if (!(ifp->options->options & DHCPCD_INITIAL_DELAY)) {
-		ipv6nd_startrs1(ifp);
-		return;
-	}
-
 	tv.tv_sec = 0;
 	tv.tv_nsec = (suseconds_t)arc4random_uniform(
 	    MAX_RTR_SOLICITATION_DELAY * NSEC_PER_SEC);
