@@ -47,7 +47,6 @@
 #include "dhcp6.h"
 #include "if.h"
 #include "if-options.h"
-#include "ipv4ll.h"
 #include "ipv6nd.h"
 #include "script.h"
 
@@ -92,9 +91,8 @@ exec_script(const struct dhcpcd_ctx *ctx, char *const *argv, char *const *env)
 {
 	pid_t pid;
 	posix_spawnattr_t attr;
-	int r;
+	int i;
 #ifdef USE_SIGNALS
-	size_t i;
 	short flags;
 	sigset_t defsigs;
 #else
@@ -109,15 +107,15 @@ exec_script(const struct dhcpcd_ctx *ctx, char *const *argv, char *const *env)
 	flags = POSIX_SPAWN_SETSIGMASK | POSIX_SPAWN_SETSIGDEF;
 	posix_spawnattr_setflags(&attr, flags);
 	sigemptyset(&defsigs);
-	for (i = 0; i < dhcpcd_signals_len; i++)
-		sigaddset(&defsigs, dhcpcd_signals[i]);
+	for (i = 0; dhcpcd_handlesigs[i]; i++)
+		sigaddset(&defsigs, dhcpcd_handlesigs[i]);
 	posix_spawnattr_setsigdefault(&attr, &defsigs);
 	posix_spawnattr_setsigmask(&attr, &ctx->sigset);
 #endif
 	errno = 0;
-	r = posix_spawn(&pid, argv[0], NULL, &attr, argv, env);
-	if (r) {
-		errno = r;
+	i = posix_spawn(&pid, argv[0], NULL, &attr, argv, env);
+	if (i) {
+		errno = i;
 		return -1;
 	}
 	return pid;
@@ -131,7 +129,8 @@ make_var(struct dhcpcd_ctx *ctx, const char *prefix, const char *var)
 	char *v;
 
 	len = strlen(prefix) + strlen(var) + 2;
-	if ((v = malloc(len)) == NULL) {
+	v = malloc(len);
+	if (v == NULL) {
 		logger(ctx, LOG_ERR, "%s: %m", __func__);
 		return NULL;
 	}
@@ -157,10 +156,8 @@ append_config(struct dhcpcd_ctx *ctx, char ***env, size_t *len,
 		eq = strchr(config[i], '=');
 		e1 = (size_t)(eq - config[i] + 1);
 		for (j = 0; j < *len; j++) {
-			if (strncmp(ne[j], prefix, strlen(prefix)) == 0 &&
-			    ne[j][strlen(prefix)] == '_' &&
-			    strncmp(ne[j] + strlen(prefix) + 1,
-			    config[i], e1) == 0)
+			if (strncmp(ne[j] + strlen(prefix) + 1,
+				config[i], e1) == 0)
 			{
 				p = make_var(ctx, prefix, config[i]);
 				if (p == NULL) {
@@ -232,28 +229,21 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 #endif
 	const struct if_options *ifo = ifp->options;
 	const struct interface *ifp2;
-	int af;
 #ifdef INET
-	int dhcp, ipv4ll;
+	int dhcp;
 	const struct dhcp_state *state;
-#ifdef IPV4LL
-	const struct ipv4ll_state *istate;
-#endif
 #endif
 #ifdef INET6
 	const struct dhcp6_state *d6_state;
-	int static6, dhcp6, ra;
+	int dhcp6, ra;
 #endif
 
 #ifdef INET
-	dhcp = ipv4ll = 0;
+	dhcp = 0;
 	state = D_STATE(ifp);
-#ifdef IPV4LL
-	istate = IPV4LL_CSTATE(ifp);
-#endif
 #endif
 #ifdef INET6
-	static6 = dhcp6 = ra = 0;
+	dhcp6 = ra = 0;
 	d6_state = D6_CSTATE(ifp);
 #endif
 	if (strcmp(reason, "TEST") == 0) {
@@ -265,17 +255,11 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 			ra = 1;
 #endif
 #ifdef INET
-#ifdef IPV4LL
-		else if (istate && istate->addr != NULL)
-			ipv4ll = 1;
-#endif
 		else
 			dhcp = 1;
 #endif
 	}
 #ifdef INET6
-	else if (strcmp(reason, "STATIC6") == 0)
-		static6 = 1;
 	else if (reason[strlen(reason) - 1] == '6')
 		dhcp6 = 1;
 	else if (strcmp(reason, "ROUTERADVERT") == 0)
@@ -291,10 +275,6 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 		/* This space left intentionally blank */
 	}
 #ifdef INET
-#ifdef IPV4LL
-	else if (strcmp(reason, "IPV4LL") == 0)
-		ipv4ll = 1;
-#endif
 	else
 		dhcp = 1;
 #endif
@@ -304,11 +284,11 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 	if (ifp->ctx->options & DHCPCD_DUMPLEASE)
 		elen = 2;
 	else
-		elen = 11;
+		elen = 13;
 
 #define EMALLOC(i, l) if ((env[(i)] = malloc((l))) == NULL) goto eexit;
 	/* Make our env + space for profile, wireless and debug */
-	env = calloc(1, sizeof(char *) * (elen + 4 + 1));
+	env = calloc(1, sizeof(char *) * (elen + 3 + 1));
 	if (env == NULL)
 		goto eexit;
 	e = strlen("interface") + strlen(ifp->name) + 2;
@@ -333,10 +313,11 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 	EMALLOC(6, e);
 	snprintf(env[6], e, "ifflags=%u", ifp->flags);
 	EMALLOC(7, e);
-	snprintf(env[7], e, "ifmtu=%d", if_getmtu(ifp));
+	snprintf(env[7], e, "ifmtu=%d", if_getmtu(ifp->name));
 	l = e = strlen("interface_order=");
 	TAILQ_FOREACH(ifp2, ifp->ctx->ifaces, next) {
-		e += strlen(ifp2->name) + 1;
+		if (!(ifp2->options->options & DHCPCD_PFXDLGONLY))
+			e += strlen(ifp2->name) + 1;
 	}
 	EMALLOC(8, e);
 	p = env[8];
@@ -344,11 +325,13 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 	e -= l;
 	p += l;
 	TAILQ_FOREACH(ifp2, ifp->ctx->ifaces, next) {
-		l = strlcpy(p, ifp2->name, e);
-		p += l;
-		e -= l;
-		*p++ = ' ';
-		e--;
+		if (!(ifp2->options->options & DHCPCD_PFXDLGONLY)) {
+			l = strlcpy(p, ifp2->name, e);
+			p += l;
+			e -= l;
+			*p++ = ' ';
+			e--;
+		}
 	}
 	*--p = '\0';
 	if (strcmp(reason, "STOPPED") == 0) {
@@ -367,12 +350,8 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 	} else if (1 == 2 /* appease ifdefs */
 #ifdef INET
 	    || (dhcp && state && state->new)
-#ifdef IPV4LL
-	    || (ipv4ll && IPV4LL_STATE_RUNNING(ifp))
-#endif
 #endif
 #ifdef INET6
-	    || (static6 && IPV6_STATE_RUNNING(ifp))
 	    || (dhcp6 && d6_state && d6_state->new)
 	    || (ra && ipv6nd_hasra(ifp))
 #endif
@@ -386,22 +365,18 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 	}
 	if (env[9] == NULL || env[10] == NULL)
 		goto eexit;
-	if ((af = dhcpcd_ifafwaiting(ifp)) != AF_MAX) {
-		e = 20;
-		EMALLOC(elen, e);
-		snprintf(env[elen++], e, "if_afwaiting=%d", af);
-	}
-	if ((af = dhcpcd_afwaiting(ifp->ctx)) != AF_MAX) {
-		TAILQ_FOREACH(ifp2, ifp->ctx->ifaces, next) {
-			if ((af = dhcpcd_ifafwaiting(ifp2)) != AF_MAX)
-				break;
-		}
-	}
-	if (af != AF_MAX) {
-		e = 20;
-		EMALLOC(elen, e);
-		snprintf(env[elen++], e, "af_waiting=%d", af);
-	}
+	if (dhcpcd_oneup(ifp->ctx))
+		env[11] = strdup("if_oneup=true");
+	else
+		env[11] = strdup("if_oneup=false");
+	if (env[11] == NULL)
+		goto eexit;
+	if (dhcpcd_ipwaited(ifp->ctx))
+		env[12] = strdup("if_ipwaited=true");
+	else
+		env[12] = strdup("if_ipwaited=false");
+	if (env[12] == NULL)
+		goto eexit;
 	if (ifo->options & DHCPCD_DEBUG) {
 		e = strlen("syslog_debug=true") + 1;
 		EMALLOC(elen, e);
@@ -418,20 +393,20 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 		ssize_t psl;
 
 		pfx_len = strlen(pfx);
-		psl = print_string(NULL, 0, OT_ESCSTRING,
+		psl = print_string(NULL, 0, ESCSTRING,
 		    (const uint8_t *)ifp->ssid, ifp->ssid_len);
 		if (psl != -1) {
 			EMALLOC(elen, pfx_len + (size_t)psl + 1);
 			memcpy(env[elen], pfx, pfx_len);
 			print_string(env[elen] + pfx_len, (size_t)psl + 1,
-			    OT_ESCSTRING,
+			    ESCSTRING,
 			    (const uint8_t *)ifp->ssid, ifp->ssid_len);
 			elen++;
 		}
 	}
 #ifdef INET
 	if (dhcp && state && state->old) {
-		n = dhcp_env(NULL, NULL, state->old, state->old_len, ifp);
+		n = dhcp_env(NULL, NULL, state->old, ifp);
 		if (n == -1)
 			goto eexit;
 		if (n > 0) {
@@ -440,8 +415,7 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 			if (nenv == NULL)
 				goto eexit;
 			env = nenv;
-			n = dhcp_env(env + elen, "old",
-			    state->old, state->old_len, ifp);
+			n = dhcp_env(env + elen, "old", state->old, ifp);
 			if (n == -1)
 				goto eexit;
 			elen += (size_t)n;
@@ -452,6 +426,16 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 	}
 #endif
 #ifdef INET6
+	if (dhcp6 && d6_state && ifo->options & DHCPCD_PFXDLGONLY) {
+		nenv = realloc(env, sizeof(char *) * (elen + 2));
+		if (nenv == NULL)
+			goto eexit;
+		env = nenv;
+		env[elen] = strdup("ifclass=pd");
+		if (env[elen] == NULL)
+			goto eexit;
+		elen++;
+	}
 	if (dhcp6 && d6_state && d6_state->old) {
 		n = dhcp6_env(NULL, NULL, ifp,
 		    d6_state->old, d6_state->old_len);
@@ -472,24 +456,8 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 
 dumplease:
 #ifdef INET
-#ifdef IPV4LL
-	if (ipv4ll) {
-		n = ipv4ll_env(NULL, NULL, ifp);
-		if (n > 0) {
-			nenv = realloc(env, sizeof(char *) *
-			    (elen + (size_t)n + 1));
-			if (nenv == NULL)
-				goto eexit;
-			env = nenv;
-			if ((n = ipv4ll_env(env + elen,
-			    istate->down ? "old" : "new", ifp)) == -1)
-				goto eexit;
-			elen += (size_t)n;
-		}
-	}
-#endif
 	if (dhcp && state && state->new) {
-		n = dhcp_env(NULL, NULL, state->new, state->new_len, ifp);
+		n = dhcp_env(NULL, NULL, state->new, ifp);
 		if (n > 0) {
 			nenv = realloc(env, sizeof(char *) *
 			    (elen + (size_t)n + 1));
@@ -497,7 +465,7 @@ dumplease:
 				goto eexit;
 			env = nenv;
 			n = dhcp_env(env + elen, "new",
-			    state->new, state->new_len, ifp);
+			    state->new, ifp);
 			if (n == -1)
 				goto eexit;
 			elen += (size_t)n;
@@ -508,20 +476,6 @@ dumplease:
 	}
 #endif
 #ifdef INET6
-	if (static6) {
-		n = ipv6_env(NULL, NULL, ifp);
-		if (n > 0) {
-			nenv = realloc(env, sizeof(char *) *
-			    (elen + (size_t)n + 1));
-			if (nenv == NULL)
-				goto eexit;
-			env = nenv;
-			n = ipv6_env(env + elen, "new", ifp);
-			if (n == -1)
-				goto eexit;
-			elen += (size_t)n;
-		}
-	}
 	if (dhcp6 && D6_STATE_RUNNING(ifp)) {
 		n = dhcp6_env(NULL, NULL, ifp,
 		    d6_state->new, d6_state->new_len);
@@ -602,9 +556,9 @@ send_interface1(struct fd_list *fd, const struct interface *iface,
 	elen = (size_t)arraytostr((const char *const *)env, &s);
 	if ((ssize_t)elen == -1) {
 		free(s);
-		retval = -1;
-	} else
-		retval = control_queue(fd, s, elen, 1);
+		return -1;
+	}
+	retval = control_queue(fd, s, elen, 1);
 	ep = env;
 	while (*ep)
 		free(*ep++);
@@ -643,19 +597,9 @@ send_interface(struct fd_list *fd, const struct interface *ifp)
 		if (send_interface1(fd, ifp, d->reason) == -1)
 			retval = -1;
 	}
-#ifdef IPV4LL
-	if (IPV4LL_STATE_RUNNING(ifp)) {
-		if (send_interface1(fd, ifp, "IPV4LL") == -1)
-			retval = -1;
-	}
-#endif
 #endif
 
 #ifdef INET6
-	if (IPV6_STATE_RUNNING(ifp)) {
-		if (send_interface1(fd, ifp, "STATIC6") == -1)
-			retval = -1;
-	}
 	if (RS_STATE_RUNNING(ifp)) {
 		if (send_interface1(fd, ifp, "ROUTERADVERT") == -1)
 			retval = -1;
@@ -683,30 +627,23 @@ script_runreason(const struct interface *ifp, const char *reason)
 
 	if (ifp->options->script &&
 	    (ifp->options->script[0] == '\0' ||
-	    strcmp(ifp->options->script, "/dev/null") == 0) &&
-	    TAILQ_FIRST(&ifp->ctx->control_fds) == NULL)
+	    strcmp(ifp->options->script, "/dev/null") == 0))
 		return 0;
 
+	argv[0] = ifp->options->script ? ifp->options->script : UNCONST(SCRIPT);
+	argv[1] = NULL;
+	logger(ifp->ctx, LOG_DEBUG, "%s: executing `%s' %s",
+	    ifp->name, argv[0], reason);
+
 	/* Make our env */
 	elen = (size_t)make_env(ifp, reason, &env);
 	if (elen == (size_t)-1) {
 		logger(ifp->ctx, LOG_ERR, "%s: make_env: %m", ifp->name);
 		return -1;
 	}
-
-	if (ifp->options->script &&
-	    (ifp->options->script[0] == '\0' ||
-	    strcmp(ifp->options->script, "/dev/null") == 0))
-	    	goto send_listeners;
-
-	argv[0] = ifp->options->script ? ifp->options->script : UNCONST(SCRIPT);
-	argv[1] = NULL;
-	logger(ifp->ctx, LOG_DEBUG, "%s: executing `%s' %s",
-	    ifp->name, argv[0], reason);
-
 	/* Resize for PATH and RC_SVCNAME */
 	svcname = getenv(RC_SVCNAME);
-	ep = reallocarray(env, elen + 2 + (svcname ? 1 : 0), sizeof(char *));
+	ep = realloc(env, sizeof(char *) * (elen + 2 + (svcname ? 1 : 0)));
 	if (ep == NULL) {
 		elen = 0;
 		goto out;
@@ -762,7 +699,6 @@ script_runreason(const struct interface *ifp, const char *reason)
 			    __func__, argv[0], strsignal(WTERMSIG(status)));
 	}
 
-send_listeners:
 	/* Send to our listeners */
 	bigenv = NULL;
 	status = 0;
@@ -793,9 +729,7 @@ out:
 	while (*ep)
 		free(*ep++);
 	free(env);
-	if (elen == 0) {
-		logger(ifp->ctx, LOG_ERR, "%s: malloc: %m", __func__);
+	if (elen == 0)
 		return -1;
-	}
 	return WEXITSTATUS(status);
 }
